/*cdInit routine to initialize variables, called at start of calculation */
#include <string.h>
#include "cddefines.h"
/* NB - following must appear after cddefines.h since code there
 * ensures that __unix is defined on any unix box */
#ifdef __unix
#	include <unistd.h>
#	include <stdlib.h>
#endif
#include "cddrive.h"
#include "vary.h"
#include "optimr.h"
#include "blockdata.h"
#include "setfpenv.h" 
#include "called.h"
#include "dustop.h"
#include "input.h"
#include "parse.h"
#include "pnunit.h"
#include "phfit.h"
#include "grainvar.h"
#include "zero.h"
#include "varypar.h"
/* include mpi header if mpi true */
/* This flag indicates where we are multi-processing.
 * if this is true then all output happens at the end,
 * if false then each set of results is printed and flushed when it happens.
 * must be set true on parallel machines, false on pc */
#ifdef MPI
#	include <mpi.h>
#endif


/*************************************************************************
 *
 * cdMPI sets flag to tell exit handler to call MPI_Finalize 
 *
 ************************************************************************/

/* flag set true when cdMPI called, tells exit handler to clean up mpi */
static int lgMPI;

/*cdMPI sets flag telling exit handler to call MPI_Finalize, must
 * call after cdInit when MPI is used */
void cdMPI(void) /* set flag so that exit handler will clean up MPI */
{
	lgMPI = TRUE;
	return;
}

#if 0
/*this is the Cloudy exit handler routine.  This must be used 
 * if mpi is to function correctly. */
void exithandler(int iexit) 
{
	/*@-redef@*/
	enum {DEBUG=FALSE};
	/*@+redef@*/
	if( DEBUG )
	{
		fprintf(ioQQQ," exithandler called\n");
	}

	/* close any open units */
	/* flush ioQQQ */
	fflush( ioQQQ );

	ClosePunchFiles();

	/* this routine must be called upon exit or machine will appear to "hang" */
#	ifdef MPI
	if( lgMPI )
	{
		fprintf(ioQQQ," MPI_Finalize called.\n");
		MPI_Finalize();
	}
#	endif
	exit( iexit );
}
#endif

/* =================================================================== */
/* this is the routine that should be called to exit cloudy */
void cdEXIT(int iexit)
{
	/*@-redef@*/
	enum {DEBUG=FALSE};
	/*@+redef@*/
	if( DEBUG )
	{
		fprintf(ioQQQ," cdEXIT called\n");
	}

	/* close any open units */
	/* flush ioQQQ */
	fflush( ioQQQ );

	ClosePunchFiles();

	/* this routine must be called upon exit or machine will appear to "hang" */
#	ifdef MPI
	if( lgMPI )
	{
		fprintf(ioQQQ," MPI_Finalize called.\n");
		MPI_Finalize();
	}
#	endif
	exit( iexit );
}

/* =================================================================== */
void cdInit()
{
	long i;
	double vtest;
	float cs;
#	ifdef __unix
	char *ptr;
#	endif

#if	0
	static int lgFIRST=TRUE;
	/* if this is very first call then set up handler */
	if( lgFIRST )
	{
		/* register the exit handler */
		if( atexit( exithandler ) )
		{
			fprintf(ioQQQ," cdInit got error on return from atexit \n");
			cdEXIT(1);
		}

		lgFIRST = FALSE;
	}
#	endif

	/* set ioQQQ to standard output */
	ioQQQ = stdout;

	/* set flag saying that we have been called */
	lgcdInitCalled = TRUE;

	/* set ioPrnErr to stderr */
	ioPrnErr = stderr;

	/* but don't what to usually do this */
	lgPrnErr = FALSE;

	/* set MPI flag false, set true with cdMPI()*/
	lgMPI = FALSE;

	cdDefines();
	blkdata2();		/* fe2data */
	blkdata4();		/* hpdata */

	/* >>chng 00 mar 10, moved call for grain block data init to Cloudy, PvH */

	/*need a fake call to phfit so that ionization potentials are stored in 
	 * proper structures */
	phfit(1,1,1,13.7,&cs,0);

	/*set FP environment to crash on div by zero*/
	SetFPEnv();

	/*********************************************************
	 *  on a VAX compile with /G_FLOATING option on FORTRAN; *
	 *  following makes sure this happened.                  *
	 *********************************************************/
	vtest = 1e-35;
	vtest /= 1e35;
	if( vtest == 0. )
	{
		fprintf( ioQQQ, " Something is wrong with the double precision.  Use /g_floating on a VAX\n" );
	}

	/*********************************************************
	 *  check that we are being compiled with ANSI C or C++  *
	 *********************************************************/
#	if !defined(__STDC__) && !defined(__cplusplus)
	{
		/* following should be set true to allow this test,
		 * set false to have non-ANSI compilers work */
		/*@-redef@*/
		enum {CHECKIT=TRUE};
		/*@+redef@*/
		if( CHECKIT )
		{
			fprintf(ioQQQ," I must be compiled using ANSI C options.  This was not enabled.\n");
			fprintf(ioQQQ," Visual Studio use the /Za flag.\n");
			fprintf(ioQQQ," GNU gcc use the -ansi flag.\n");
			fprintf(ioQQQ," Dec alpha use the -std1 flag.\n");
			fprintf(ioQQQ," Sparc use the -Xc flag.\n");
			puts( "[Stop in cdInit]" );
			cdEXIT(1);
		}
	}
#	endif

	/* initialize some variables dealing with cloudy's interaction with machine environment */
	/* if TALK is TRUE then do standard printout
	 * if FALSE then never say anything */
	called.lgTalk = TRUE;
	called.lgBusted = FALSE;
	/* this flag is needed to turn print on to ahave effect */
	called.lgTalkIsOK = TRUE;
	/* means talk not forced off by call to cdTalk*/
	called.lgTalkForce = FALSE;

	vary.lgNoVary = FALSE;
	vary.lgVaryOn = FALSE;
	optimr.lgOptimr = FALSE;

#	ifdef __unix
	/* determine hostname; used by PHYMIR */
        if( (ptr = getenv("HOST")) != NULL )
		strcpy(VaryPar.HostName,ptr);
	else
		strcpy(VaryPar.HostName,"unknown");
	/* determine the no. of CPUs on this machine; used by PHYMIR */
#	if defined(_SC_NPROCESSORS_ONLN)  /* Sun Sparc, DEC Alpha */
	VaryPar.maxCPU = sysconf(_SC_NPROCESSORS_ONLN);
#	elif defined(_SC_NPROC_ONLN)      /* SGI Iris */
	VaryPar.maxCPU = sysconf(_SC_NPROC_ONLN);
#	elif defined(_SC_CRAY_NCPU)       /* Cray */
	VaryPar.maxCPU = sysconf(_SC_CRAY_NCPU);
#	else                              
	/* Other systems, supply no. of CPUs on OPTIMIZE PHYMIR command line */
	VaryPar.maxCPU = 1;
#	endif
#	else
	strcpy(VaryPar.HostName,"unknown");
	VaryPar.maxCPU = 1;
#	endif

	/* number of lines entered with cdLine
	 * both check that number less than NKRD, the limit
	 * the line save array is devined from 0 through input.nSave */
	input.nSave = -1;

	/* nRead is the number of the command in the input stream - many vary options
	 * point to it to refer to the original command.  it is incremented before
	 * it is used, so will become 0.  it is the array element within the stack
	 * of emission lines */
	input.nRead = -1;

	/* this is number of init lines read in */
	input.nSaveIni = 0;

	/* flag identifying whether any line images were too long, tested in cdRead */
	lgCardSavTooLong = FALSE;

	/* this is sanity check that lines are read in ok */
	for( i=0; i < NKRD; i++ )
	{
		strcpy( input.chCardSav[i], "error! - no line image input" );
	}

	/* initialize stuff for opening punch files, should not be done in zero() */
	pnunit.lgOpenUnits = TRUE;
	InitPunchFiles();

	/* start the timer to log execution time */
	cdSetExecTime();

	/* zero out lots of variables */
	zero();

#	ifdef DEBUG_FUN
	fputs( " <->cdInit()\n", debug_fp );
#	endif
	return;
}

