/*ContCreatePointers set up pointers for lines and continua called by cloudy after input read in */
/*fill define the continuum energy grid over a specified range */
/*ChckFill perform sanity check confirming that the energy array has been properly filled */
/*fiddle adjust energy bounds of certain cells so that match ionization edges exactly */
/*eina convert a gf into an Einstein A */
#include <string.h>
#include <limits.h>
#include "cddefines.h"
#include "hydrogenic.h"
#include "secondaries.h"
#include "nfeii.h"
#include "nhe1lvl.h"
#include "taulines.h"
#include "physconst.h"
#include "elementnames.h"
#include "chionlbl.h"
#include "getgf.h"
#include "abscf.h"
#include "helike.h" 
#include "fston.h"
#include "opacity.h"
#include "chlinelbl.h"
#include "ipvfil.h"
#include "em2nu.h"
#include "twophoton.h"
#include "ipfe2.h"
#include "fe2pmp.h"
#include "abundances.h"
#include "xry.h"
#include "pumpfe.h"
#include "pbowen.h"
#include "ip626.h"
#include "ircoil.h"
#include "rfield.h"
#include "nh.h"
#include "he1nionryd.h"
#include "iphe1l.h"
#include "nhe1.h"
#include "he.h"
#include "ip2gam.h"
#include "i3727.h"
#include "pop371.h"
#include "ipoi.h"
#include "fekems.h"
#include "trace.h"
#include "interpolate.h"
#include "iphmin.h"
#include "egray.h"
#include "kshllenr.h"
#include "fillnu.h"
#include "hmi.h"
#include "heavy.h"
#include "he3lines.h"
#include "nsshells.h"
#include "predcont.h"
#include "ph1com.h"
#include "ipoint.h"
#include "ipshells.h"
#include "continuum.h"

/*fill define the continuum energy grid over a specified range */
static void fill(double fenlo, 
  double fenhi, 
  double resolv, 
  long int *n0, 
  long int *ipnt);

/*ChckFill perform sanity check confirming that the energy array has been properly filled */
static void ChckFill(void);

/*fiddle adjust energy bounds of certain cells so that match ionization edges exactly */
static void fiddle(long int ipnt, 
  double exact);

/*eina convert a gf into an Einstein A */
static double eina(double gf, 
	  double enercm, 
	  double gup);

void ContCreatePointers(void)
{
	char chLab[5];
	long int 
	  i, 
	  ipHi, 
	  ipLo, 
	  ipZ, 
	  ipnt, 
	  ipISO,
	  iWL_Ang,
	  j, 
	  limit ,
	  n0, 
	  nelem,
	  nshells;
	double energy,
		xnew;
	/* flag to say whether pointers have ever been evaluated */
	static int lgPntEval = FALSE;

#	ifdef DEBUG_FUN
	fputs( "<+>ContCreatePointers()\n", debug_fp );
#	endif

	/* create the hydrogen atom for this coreload, routine creates space then zeros it out
	 * on first call, on second and later calls it only zeros things out */
	HydroCreate();

	/* create he-like series */
	HeCreate();

	/* lgPntEval is local static variable defined FALSE when defined. 
	 * it is set TRUE below, so that pointers only created one time in the
	 * history of this coreload. */
	if( lgPntEval )
	{
		if( trace.lgTrace )
		{
			fprintf( ioQQQ, " ContCreatePointers called, not evaluating.\n" );
		}
		/* now save current form of energy array */
		for( i=0; i < NCELL; i++ )
		{
			rfield.anu[i] = rfield.AnuOrg[i];
		}
		
#		ifdef DEBUG_FUN
		fputs( " <->ContCreatePointers()\n", debug_fp );
#		endif
		return;
	}
	else
	{
		if( trace.lgTrace )
		{
			fprintf( ioQQQ, " ContCreatePointers called first time.\n" );
		}
		lgPntEval = TRUE;
	}

	for( i=0; i < NCELL; i++ )
	{
		/* this is array of labels for lines and continua, set to blanks at first */
		strcpy( rfield.chContLabel[i], "    ");
		strcpy( rfield.chLineLabel[i], "    ");
	}

	/* fill in continuum with freq points
	 * arg are range pointer, 2 energy limits, resolution
	 * first argument is lower energy of the continuum range to be defined
	 * second argument is upper range; both are in Rydbergs
	 * third number is the relative energy resolution, dnu/nu,
	 * for that range of the continuum
	 * last two numbers are internal book keeping
	 * N0 is number of energy cells used so far
	 * IPNT is a counter for the number of fills used
	 *
	 * if this is changed, then also change warning in GetTable about using
	 * transmitted continuum - it says version number where continuum changed
	 * */
	n0 = 1;
	ipnt = 0;
	fillnu.nrange = 0;/* this is number of ranges that will be introduced below*/
#	if 0
/*#	if !defined(NDEBUG)*/
	/* ================================================================ */
	/* >>>chng 99 oct 13, need finer resolution in FIR */
	/* following were bounds before sept 00 */
	fill(rfield.emm, 1e-4,  0.1,&n0,&ipnt);/**/
	fill(1e-4      , 0.05,  0.02,&n0,&ipnt);/**/
	fill(     0.05 , 600.,  0.01,&n0,&ipnt);/**/
	fill(600.,rfield.egamry, 0.03,&n0,&ipnt);
/*#	else*/
#	endif
	/* ================================================================ */
	/* energy bounds as changed in sept 00 */
	/* >>>chng 00 sep 13, made resolution across FIR/MIR 1% to resolve grain emission*/
	/* >>>chng 00 oct 16, made resolution 1% to 2e-5 Ryd to resolve CO rotation lines*/
	fill(rfield.emm, 2e-5,  0.1,&n0,&ipnt);
	fill(2e-5      , 600.,  0.01,&n0,&ipnt);
	fill(600.,rfield.egamry, 0.03,&n0,&ipnt);
	/* ================================================================ */

	/* at this point debugger shows that anu and widflx are defined 
	 * up through n0-2 - due to c offset -1 from fortran! */
	rfield.nupper = n0 - 1;
	rfield.nflux = rfield.nupper;

	/* there must be a cell above nflux for us to pass unity through the vol integrator
	 * as a sanity check.  assert that this is true so we will crash if ever changed */
	assert( rfield.nupper +1 <= NCELL );

	/* finish off rest of array, even if not used, N0 is next element to fill
	 * NUPPER is number of NCELL cells actually used for freq mesh */
	if( n0 <= NCELL )
	{
		for( i=rfield.nupper; i < NCELL; i++ )
		{
			rfield.widflx[i] = rfield.widflx[i-1];
			rfield.anu[i] = rfield.anu[i-1] + rfield.widflx[i];
		}
	}

	/* this is a sanity check for results produced above by fill */
	ChckFill();

	/* we will generate a set of pointers to ionization edges for
	 * the first thirty elements.  First set all pointers to
	 * totally bogus values so we will crash if misused */
	for( nelem=0; nelem<LIMELM; ++nelem )
	{
		if( abundances.lgElmtOn[nelem] )
		{
			for( ipZ=0; ipZ<LIMELM; ++ipZ )
			{
				for( nshells=0; nshells<7; ++nshells )
				{
					for( j=0; j<3; ++j )
					{
						opac.ipElement[j][nshells][ipZ][nelem] = INT_MIN;
					}
				}
			}
		}
	}

	/* pointer to excited state of O+2 */
	opac.ipo3exc[0] = ipContEnergy(3.855,"O3ex");

	/* main hydrogenic arrays - THIS OCCURS TWICE!! H and He here, then the
	 * remaining hydrogenic species near the bottom.  This is so that H and HeII get
	 * their labels stuffed into the arrays, and the rest of the hydrogenic series 
	 * get whatever is left over after the level 1 lines.
	 * to find second block, search on "ipZ=2" */
	/* NB note that no test for H or He being on exists here - we will always
	 * define the H and He arrays even when he is off, since we need to
	 * know where the he edges are for the bookkeeping that occurs in continuum
	 * binning routines */
	for( ipZ=0; ipZ < 2; ipZ++ )
	{
		/* generate label for this ion */
		sprintf( chLab, "%2s%2ld",elementnames.chElementSym[ipZ], ipZ+1 );
		/* Lya itself is the only transition below n=3 - we explicitly do not
		 * want to generate pointers for 2s-1s or 2p-2s */
		HydroLines[ipZ][ipH2p][ipH1s].ipCont = 
			(int)ipLineEnergy(HydroLines[ipZ][ipH2p][ipH1s].EnergyRyd , chLab , 0);
		/* set large negative pointers for 2p-2s and 2s-1s so we blow
		 * if we try to address 2s-2p or 1s-2s */
		HydroLines[ipZ][ipH2s][ipH1s].ipCont = INT_MIN;
		/* 2p-2s set to zero so that finite but not important */
		HydroLines[ipZ][ipH2p][ipH2s].ipCont = 1;
		for( ipLo=ipH1s; ipLo <= nHLevels[ipZ]; ipLo++ )
		{
			/* HLevNIonRyd is level energy ryd 
			 * this is pointer to continuum ionization threshold 
			 * also puts label into continuum array */
			nh.ipHn[ipZ][ipLo] = ipContEnergy(hydro.HLevNIonRyd[ipZ][ipLo],chLab);
			assert(nh.ipHn[ipZ][ipLo] > 0 );

			/* define all hydrogenic line pointers, MAX2 is to avoid creating
			 * pointers for two photon emission or 2s-2p transition */
			for( ipHi=MAX2(3,ipLo+1); ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				HydroLines[ipZ][ipHi][ipLo].ipCont = 
					(int)ipLineEnergy(HydroLines[ipZ][ipHi][ipLo].EnergyRyd , chLab ,
					nh.ipHn[ipZ][ipLo]);

				/* do not let pointer go into next higher continuum */
				/*HydroLines[ipZ][ipHi][ipLo].ipCont = 
					MIN2( j , (nh.ipHn[ipZ][ipLo]-1) );*/
			}
		}
		/* set pointer to two photon continuum as emission line to 
		 * completely bad value - if ever used should cause code to crash */
		HydroLines[ipZ][ipH2s][ipH1s].ipCont = INT_MIN;
	}

	/* do helium itself in he-iso seq, rest will follow later*/
	ipZ = 1;
	/* generate label for this ion */
	sprintf( chLab, "%2s%2ld",elementnames.chElementSym[ipZ], ipZ );
	for( ipLo=0; ipLo < nHeLevels[ipZ]-1; ipLo++ )
	{
		/* define all hydrogenic line pointers, MAX2 is to avoid creating
		 * pointers for two photon emission or 2s-2p transition */
		for( ipHi=ipLo+1; ipHi < nHeLevels[ipZ]; ipHi++ )
		{
			/* some energies are negative for inverted levels */
			HeLines[ipZ][ipHi][ipLo].ipCont = 
				(int)ipLineEnergy(fabs(HeLines[ipZ][ipHi][ipLo].EnergyRyd) , chLab ,
				nh.ipHn[ipZ][ipLo]);
		}
	}


	/* need to increase the cell for the HeII balmer continuum by one, so that
	 * hydrogen lyman continuum will pick it up. */
	ipZ = 1;
	/* copy label over to next higher cell */
	if( strcmp( rfield.chContLabel[nh.ipHn[ipZ][ipH2s]-1] , "He 2" ) == 0)
	{
		strcpy( rfield.chContLabel[nh.ipHn[ipZ][ipH2s]], 
				 rfield.chContLabel[nh.ipHn[ipZ][ipH2s]-1] );
		/* set previous spot to blank */
		strcpy( rfield.chContLabel[nh.ipHn[ipZ][ipH2s]-1] , "    ");
	}
	else if( strcmp( rfield.chContLabel[nh.ipHn[ipZ][ipH2s]-1] , "H  1" ) == 0)
	{
		/* set previous spot to blank */
		strcpy( rfield.chContLabel[nh.ipHn[ipZ][ipH2s]] , "He 2");
	}
	else
	{
		fprintf(ioQQQ," insanity heii pointer fix contcreatepointers\n");
	}
	/* finally increment the two HeII pointers so that they are above the Lyman continuum */
	++nh.ipHn[ipZ][ipH2s];
	++nh.ipHn[ipZ][ipH2p];

	/* array of either 1 or 0 for energy of elec to be counted
	 * as able to produce secondary ionizations
	 * below 100eV no secondary ionization */
	Secondaries.ipSecIon = ipoint(7.353);

	/* this is highest energy where k-shell opacities are counted
	 * can be adjusted with "set kshell" command */
	KshllEnr.KshellLimit = ipoint(KshllEnr.EnergyKshell);

	/* pointers for molecules
	 * H2+ dissociation energy 2.647 eV but cs small below 0.638 Ryd */
	opac.ih2pnt[0] = ipContEnergy(0.502,"H2+ ");
	opac.ih2pnt[1] = ipoint(1.03);

	/* pointers for most prominent PAH features
	 * energies given to ipContEnergy are only to put lave in the right place
	 * wavelenghts are rough observed values of blends
	 * 7.6 microns */
	i = ipContEnergy(0.0117, "PAH " );

	/* feature near 6.2 microns */
	i = ipContEnergy(0.0147, "PAH " );

	/* 3.3 microns */
	i = ipContEnergy(0.028, "PAH " );

	/* 11.2 microns */
	i = ipContEnergy(0.0080, "PAH " );

	/* 12.3 microns */
	i = ipContEnergy(0.0077, "PAH " );

	/* 13.5 microns */
	i = ipContEnergy(0.0069, "PAH " );


	/* fix pointers for hydrogen and helium */

	/* pointers to He I triplet lines */
	he3lines.ipHe3l[0] = ipLineEnergy(0.08416,"HeI3",0);
	he3lines.ipHe3l[1] = ipLineEnergy(RYDLAM/5876.,"HeI3",0);
	he3lines.ipHe3l[2] = ipLineEnergy(RYDLAM/7065.,"HeI3",0);
	he3lines.ipHe3l[3] = ipLineEnergy(RYDLAM/3889.,"HeI3",0);

	/* pointer to Bowen 374A resonance line */
	pbowen.ip374 = ipLineEnergy((1.92),"He 2",0);
	pbowen.ip660 = ipLineEnergy((1.38),"He 2",0);

	/* set up series of continuum pointers to be used in routine lines to
	 * enter continnum points into line array */
	for( i=0; i < NPREDCONT; i++ )
	{
		/* EnrPredCont contains array of energy points, as set in zerologic.c */
		ipPredCont[i] = ipoint(EnrPredCont[i]) - 1;
	}

	/* do HeI */
	for( i=0; i < (NHE1LVL - 1); i++ )
	{
		nhe1Com.nhe1[i] = ipContEnergy(He1NIonRyd.He1IonRyd[i],"He 1" );
		for( j=i + 1; j < NHE1LVL; j++ )
		{
			/* get energies from table, in zero */
			iphe1lCom.iphe1l[i][j] = ipLineEnergy((He1NIonRyd.He1IonRyd[i]-
			  He1NIonRyd.He1IonRyd[j]),"He 1" , nhe1Com.nhe1[i]);
			/* make sure pointer not in next higher continuum */
			/*if( iphe1lCom.iphe1l[i][j] >= nhe1Com.nhe1[i] )
			{
				fprintf( ioQQQ, " He1 line%3ld%3ld spills over into%3ld continuum, energy=%11.3e\n", 
				  j, i, i, rfield.anu[iphe1lCom.iphe1l[i][j]-1] );
			}*/
		}
	}

	nhe1Com.nhe1[NHE1LVL-1] = ipContEnergy(He1NIonRyd.He1IonRyd[NHE1LVL-1],
	  "He 1");

	nhe1Com.nhe1[NHE1LVL] = nhe1Com.nhe1[1];

	/* pointer to helium 2 trip s to ground decay */
	ip626.ipHe23sGrnd = ipLineEnergy(1.457,"HeI3",0);

	/* photoionization for HeI triplet 23s */
	he.nhei3 = ipContEnergy(0.3506,"HeI3");

	/* 666 error! remove following two, replace with above */
	he.nhei1 = nhe1Com.nhe1[0];
	he.nheii = nh.ipHn[ipHELIUM][ipH1s];

	/* pointer to energy defining effect x-ray column */
	xry.ipxry = ipoint(73.5);

	/* double photoionization
	 * 666 error! most of these have been removed */
	ip2gam.ip2ePhoto = ipoint(7.3);

	/* pointer to Hminus edge at 0.754eV */
	iphminCom.iphmin = ipContEnergy(0.05544,"H-  ");

	/* pointer to CaII XYZ at 8700A
	 * ipxyz = ipoint(0.1)
	 * */
	hmi.iheh1 = ipoint(1.6);
	hmi.iheh2 = ipoint(2.3);

	/* pointer to carbon k-shell ionization */
	opac.ipCKshell = ipoint(20.6);

	/* pointer to threshold for pair production */
	opac.ippr = ipoint(7.51155e4) + 1;

	/* pointer to x-ray - gamma ray bound; 100 kev */
	egray.ipEnerGammaRay = ipoint(egray.EnerGammaRay);

	/* pointers to bound compton electron scattering of H, He */
	ircoil.ipRecoilIon = ipoint(194.);
	ircoil.irche = ipoint(260.);

	for( i=0; i < NFEII; i++ )
	{
		ipfe2Com.ipfe2[i] = ipoint(fe2pmp.fe2enr[i]);
	}

	/* pointers to hot and cold iron */
	fekems.ipkcld = ipLineEnergy(6400./13.54,"Fekc",0);
	fekems.ipkhot = ipLineEnergy(6600./13.54,"Fekh",0);
	he3lines.ipHe3l[0] = ipLineEnergy(0.08416,"HeI3",0);

	/* make low energy edges of some cells exact,
	 * index is on c scale
	 * 0.99946 is correct energy of hydrogen in inf mass ryd */
	fiddle(nh.ipHn[ipHYDROGEN][ipH1s]-1,0.99946);
	fiddle(nh.ipHn[ipHYDROGEN][ipH2p]-1,0.24986);
	/* confirm that labels are in correct location */
	assert( strcmp( rfield.chContLabel[nh.ipHn[ipHYDROGEN][ipH1s]-1], "H  1" ) ==0 );
	assert( strcmp( rfield.chContLabel[nh.ipHn[ipHYDROGEN][ipH2p]-1], "H  1" ) ==0 );

	fiddle(nhe1Com.nhe1[0]-1,1.8072);
	fiddle(nh.ipHn[ipHELIUM][ipH1s]-1,4.00000);
	assert( strcmp( rfield.chContLabel[nhe1Com.nhe1[0]-1], "He 1" ) ==0 );
	assert( strcmp( rfield.chContLabel[nh.ipHn[ipHELIUM][ipH1s]-1], "He 2" ) ==0 );

	/* pointer to excited state of O+2
	 * ipo3exc(1) = ipContEnergy(3.855 ,'O3ex' ) */
	fiddle(opac.ipo3exc[0]-1,3.855);

	/* now fix widflx array so that it is correct */
	for( i=1; i<NCELL-1; ++i )
	{
		rfield.widflx[i] = ((rfield.anu[i+1] - rfield.anu[i]) + (rfield.anu[i] - rfield.anu[i-1]))/2.f;
	}

	/* now save current form of array */
	for( i=0; i < NCELL; i++ )
	{
		rfield.AnuOrg[i] = rfield.anu[i];
		rfield.anusqr[i] = (float)sqrt(rfield.AnuOrg[i]);
	}

	/* following order of elements is in roughly decreasing abundance
	 * when ipShells gets a cell for the valence IP it does so through
	 * ipContEnergy, which makes sure that no two ions get the same cell
	 * earliest elements have most precise ip mapping */

	/* set up shell pointers for hydrogen */
	nelem = 0;
	ipZ = 0;

	/* the number of shells */
	nsShellsCom.nsShells[0][nelem] = 1;

	/*pointer to ionization threshold in energy array*/
	Heavy.ipHeavy[0][0] = nh.ipHn[nelem][ipH1s];
	opac.ipElement[0][0][ipZ][nelem] = nh.ipHn[nelem][ipH1s];

	/* upper limit to energy integration */
	opac.ipElement[1][0][ipZ][nelem] = rfield.nupper;

	if( abundances.lgElmtOn[1] )
	{
		/* set up shell pointers for helium */
		nelem = 1;
		ipZ = 0;

		/* the number of shells */
		nsShellsCom.nsShells[0][nelem] = 1;

		/*pointer to ionization threshold in energy array*/
		Heavy.ipHeavy[0][1] = nhe1Com.nhe1[0];
		opac.ipElement[0][0][ipZ][nelem] = nhe1Com.nhe1[0];

		/* upper limit to energy integration */
		opac.ipElement[1][0][ipZ][nelem] = rfield.nupper;

		/* (hydrogenic) helium ion */
		ipZ = 1;
		/* the number of shells */
		nsShellsCom.nsShells[1][nelem] = 1;

		/*pointer to ionization threshold in energy array*/
		Heavy.ipHeavy[1][1] = nh.ipHn[nelem][ipH1s];
		opac.ipElement[0][0][ipZ][nelem] = nh.ipHn[nelem][ipH1s];

		/* upper limit to energy integration */
		opac.ipElement[1][0][ipZ][nelem] = rfield.nupper;
	}

	/* check that ionization potential of neutral carbon valence shell is
	 * positive */
	assert( PH1COM.PH1[2][5][5][0] > 0. );

	/* now fill in all sub-shell ionization array indices for elements heavier than He,
	 * this must be done after previous loop on nh.ipHn since hydrogenic species use
	 * nh.ipHn rather than ipoint in getting array index within continuum array */
	for( i=2; i<LIMELM; ++i )
	{
		if( abundances.lgElmtOn[i])
		{
			/* i is the atomic number on the c scale, 2 for Li */
			ipShells(i);
		}
	}

	/* oxygen
	 * pointers for excited states
	 * IO3 is pointer to O++ exc state, is set above */
	i3727.i2d = ipoint(1.242);
	i3727.i2p = ipoint(1.367);
	opac.ipo1exc[0] = ipContEnergy(0.856,"O1ex");
	opac.ipo1exc[1] = ipoint(2.0);

	/* finish off excited state photoionization
	 * do not use ipContEnergy since only upper limit */
	opac.ipo3exc[1] = ipoint(5.0);

	/* upper level of 4363 */
	opac.ipo3exc3[0] = ipContEnergy(3.646,"O3ex");
	opac.ipo3exc3[1] = ipoint(5.0);

	/* following are various pointers for OI - Lybeta pump problem
	 * these are delta energies for boltzmann factors
	 * 666 error! this is redundant with contents of oxygen line arrays
	 * use them instead
	 * when removing this, make sure all line intensity predictions
	 * also go into oi line arrays */
	ipoi.ipoiex[0] = ipoint(0.7005);
	ipoi.ipoiex[1] = ipoint(0.10791);
	ipoi.ipoiex[2] = ipoint(0.06925);
	ipoi.ipoiex[3] = ipoint(0.01151);
	ipoi.ipoiex[4] = ipoint(0.01999);

	/* >>chng 97 jan 27, move nitrogen after oxygen so that O gets the
	 * most accurate pointers
	 * Nitrogen
	 * in1(1) is thresh for photo from excited state */
	opac.in1[0] = ipContEnergy(0.893,"N1ex");

	/* upper limit */
	opac.in1[1] = ipoint(2.);

	if( (trace.lgTrace && trace.lgConBug) || (trace.lgTrace && trace.lgPointBug) )
	{
		fprintf( ioQQQ, "   ContCreatePointers:%5ldcells possible,%5ld used. N(1R):%4ld N(1.8):%4ld  N(4Ryd):%4ld N(O3)%4ld  N(x-ray):%5ld N(rcoil)%5ld\n", 
		  NCELL, rfield.nupper, nh.ipHn[ipHYDROGEN][ipH1s], nhe1Com.nhe1[0], 
		  nh.ipHn[ipHELIUM][ipH1s], opac.ipo3exc[0], opac.ipCKshell, 
		  ircoil.ipRecoilIon );

		fprintf( ioQQQ, "   ContCreatePointers: HeI trip: %5ld\n", he.nhei3 );

		fprintf( ioQQQ, "   ContCreatePointers: ipEnerGammaRay: %5ld IPPRpari produc%5ld\n", 
		  egray.ipEnerGammaRay, opac.ippr );

		fprintf( ioQQQ, "   ContCreatePointers: H pointers;" );
		for( i=0; i <= 6; i++ )
		{
			fprintf( ioQQQ, "%4ld%4ld", i, nh.ipHn[ipHYDROGEN][i] );
		}
		fprintf( ioQQQ, "\n" );

		fprintf( ioQQQ, "   ContCreatePointers: HeI pnters;" );
		for( i=1; i <= 6; i++ )
		{
			fprintf( ioQQQ, "%4ld%4ld", i, nhe1Com.nhe1[i-1] );
		}
		fprintf( ioQQQ, "\n" );
		fprintf( ioQQQ, "   ContCreatePointers: Oxy pnters;" );

		for( i=1; i <= 8; i++ )
		{
			fprintf( ioQQQ, "%4ld%4ld", i, Heavy.ipHeavy[i-1][7] );
		}
		fprintf( ioQQQ, "\n" );
	}

	/* Magnesium
	 * following is energy for phot of MG+ from exc state producing 2798 */
	opac.ipmgex = ipoint(0.779);

	/* pointers to ir triplet, kh
	 * icaxyz = ipoint( 0.11 )
	 * icakh = ipoint( 0.235 )
	 * lower, upper edges of Ca+ excited term photoionizaiton
	 * ica2ex(1) = ipoint(0.72) */
	opac.ica2ex[0] = ipContEnergy(0.72,"Ca2x");
	opac.ica2ex[1] = ipoint(1.);

	/* set all pointers to ionizaiton edges with labels */
	/* set up factors and pointers for Fe continuum fluorescence */
	pumpfe.ipfe10 = ipoint(2.605);

	/* center of V filter */
	ipvfil.ipVFilter = ipoint(0.164);

	/* following is WL(CM)**2/(8PI) * conv fac for RYD to NU *A21 */
	pumpfe.pfe10 = (float)(2.00e-18/rfield.widflx[pumpfe.ipfe10-1]);

	/* this is 353 pump, f=0.032 */
	pumpfe.pfe11a = (float)(4.54e-19/rfield.widflx[pumpfe.ipfe10-1]);

	/* this is 341.1 f=0.012 */
	pumpfe.pfe11b = (float)(2.75e-19/rfield.widflx[pumpfe.ipfe10-1]);
	pumpfe.pfe14 = (float)(1.15e-18/rfield.widflx[pumpfe.ipfe10-1]);

	/* set up energy pointers for line optical depth arrays
	 * this also increments flux, sets other parameters for lines */

	/* level 1 heavy elements line array */
	for( i=1; i <= nLevel1; i++ )
	{
		TauLines[i].EnergyRyd = 
			(float)(TauLines[i].EnergyWN* WAVNRYD);

		/* put null terminated line label into chLab using line array*/
		chIonLbl(chLab, &TauLines[i]);

		TauLines[i].ipCont = 
			(int)ipLineEnergy(TauLines[i].EnergyRyd, chLab ,0);
		/* for debugging pointer - index on f not c scale, 
		 * this will find all lines that entered a given cell 
		   if( TauLines[i].ipCont==603 )
			fprintf(ioQQQ,"( level1 %s\n", chLab);*/

		if( TauLines[i].gf > 0. )
		{
			/* the gf value was entered
			 * derive the A, call to function is gf, wl (A), g_up */
			TauLines[i].Aul = 
				(float)(eina(TauLines[i].gf,
			  TauLines[i].EnergyWN,TauLines[i].gHi));
		}
		else if( TauLines[i].Aul > 0. )
		{
			/* the Einstion A value was entered
			 * derive the gf, call to function is A, wl (A), g_up */
			TauLines[i].gf = 
				(float)(GetGF(TauLines[i].Aul,
			  TauLines[i].EnergyWN,TauLines[i].gHi));
		}
		else
		{
			fprintf( ioQQQ, " level 1 line does not have valid gf or A\n" );
			fprintf( ioQQQ, " This is ContCreatePointers\n" );
			puts( "[Stop in ContCreatePointers]" );
			cdEXIT(1);
		}

		TauLines[i].damprel = 
			(float)(TauLines[i].Aul*
		  TauLines[i].WLAng*1e-8/PI4);

		/* derive the abs coef, call to function is gf, wl (A), g_low */
		TauLines[i].opacity = 
			(float)(abscf(TauLines[i].gf,
		  TauLines[i].EnergyWN,TauLines[i].gLo));

		/* excitation energy of transition in degrees kelvin */
		TauLines[i].EnergyK = 
			(float)(T1CM)*TauLines[i].EnergyWN;

		/* energy of photon in ergs */
		TauLines[i].EnergyErg = 
			(float)(ERG1CM)*TauLines[i].EnergyWN;

		/*line wavelength must be gt 0 */
		assert( TauLines[i].WLAng > 0 );
	}

	/* this is rest of hydrogenic species, after level 1 but before level 2,
	 * so that line labels remain meaningful, to go back to first time this
	 * block occurs, searh on ipZ=0 */
	for( ipZ=2; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ])
		{
			/* generate label for this ion */
			sprintf( chLab, "%2s%2ld",elementnames.chElementSym[ipZ], ipZ+1 );
			/* Lya itself is the only transition below n=3 - we explicitly do not
			 * want to generate pointers for 2s-1s or 2p-2s */
			HydroLines[ipZ][ipH2p][ipH1s].ipCont = 
				(int)ipLineEnergy(HydroLines[ipZ][ipH2p][ipH1s].EnergyRyd , chLab ,0);
			/* set large negative pointers for 2p-2s and 2s-1s so we blow
			 * if we try to address 2s-2p or 1s-2s */
			HydroLines[ipZ][ipH2s][ipH1s].ipCont = INT_MIN;
			HydroLines[ipZ][ipH2p][ipH2s].ipCont = 1;
			for( ipLo=ipH1s; ipLo <= nHLevels[ipZ]; ipLo++ )
			{
				/* HLevNIonRyd is level energy ryd 
				 * this is pointer to continuum ionization threshold 
				 * also puts label into continuum array */
				nh.ipHn[ipZ][ipLo] = ipContEnergy(hydro.HLevNIonRyd[ipZ][ipLo],chLab);
				assert( nh.ipHn[ipZ][ipLo]> 0 );

				/* define all hydrogenic line pointers */
				for( ipHi=MAX2(3,ipLo+1); ipHi <= nHLevels[ipZ]; ipHi++ )
				{
					HydroLines[ipZ][ipHi][ipLo].ipCont = 
						(int)ipLineEnergy(HydroLines[ipZ][ipHi][ipLo].EnergyRyd , chLab , 
						nh.ipHn[ipZ][ipLo]);

					/* do not let pointer go into next higher continuum */
					/*HydroLines[ipZ][ipHi][ipLo].ipCont = 
						MIN2( j , (nh.ipHn[ipZ][ipLo]-1) );*/
				}
			}
			/* this was also evaluated in ipShells, should have the same answer,
			 * logical error if not */
			assert( nh.ipHn[ipZ][ipH1s] == opac.ipElement[0][0][ipZ][ipZ] );

			/* set pointer to two photon continuum as emission line to 
			 * completely bad value - if ever used should cause code to crash */
			HydroLines[ipZ][ipH2s][ipH1s].ipCont = INT_MIN;
		}
	}
	/* do he iso sequence */
	for( ipZ=2; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ])
		{
			/* generate label for this ion */
			sprintf( chLab, "%2s%2ld",elementnames.chElementSym[ipZ], ipZ );
			/* Lya itself is the only transition below n=3 - we explicitly do not
			 * want to generate pointers for 2s-1s or 2p-2s */
			for( ipLo=0; ipLo < nHeLevels[ipZ]-1; ipLo++ )
			{
				/* define all he-like line pointers */
				for( ipHi=ipLo+1; ipHi < nHeLevels[ipZ] ; ipHi++ )
				{
					HeLines[ipZ][ipHi][ipLo].ipCont = 
						(int)ipLineEnergy(fabs(HeLines[ipZ][ipHi][ipLo].EnergyRyd) , chLab , 
						nh.ipHn[ipZ][ipLo]);
				}
			}
		}
	}
	/* set up energy pointers for line optical depth arrays
	 * this also increments flux, sets other parameters for lines */

	/* co carbon monoxide line array */
	for( i=0; i < nCORotate; i++ )
	{
		C12O16Rotate[i].EnergyRyd = 
			(float)(C12O16Rotate[i].EnergyWN* WAVNRYD);
		C13O16Rotate[i].EnergyRyd = 
			(float)(C13O16Rotate[i].EnergyWN* WAVNRYD);

		/* excitation energy of transition in degrees kelvin */
		C12O16Rotate[i].EnergyK = 
			(float)(T1CM)*C12O16Rotate[i].EnergyWN;
		C13O16Rotate[i].EnergyK = 
			(float)(T1CM)*C13O16Rotate[i].EnergyWN;

		/* energy of photon in ergs */
		C12O16Rotate[i].EnergyErg = 
			(float)(ERG1CM)*C12O16Rotate[i].EnergyWN;
		C13O16Rotate[i].EnergyErg = 
			(float)(ERG1CM)*C13O16Rotate[i].EnergyWN;

		/* put null terminated line label into chLab using line array*/
		chIonLbl(chLab, &C12O16Rotate[i]);
		chIonLbl(chLab, &C13O16Rotate[i]);

		C12O16Rotate[i].ipCont = 
			(int)ipLineEnergy(C12O16Rotate[i].EnergyRyd, "12CO" ,0);
		C13O16Rotate[i].ipCont = 
			(int)ipLineEnergy(C13O16Rotate[i].EnergyRyd, "13CO" ,0);

		if( C12O16Rotate[i].gf > 0. )
		{
			/* the gf value was entered
			 * derive the A, call to function is gf, wl (A), g_up */
			C12O16Rotate[i].Aul = 
				(float)(eina(C12O16Rotate[i].gf,
			  C12O16Rotate[i].EnergyWN,C12O16Rotate[i].gHi));
		}
		else if( C12O16Rotate[i].Aul > 0. )
		{
			/* the Einstion A value was entered
			 * derive the gf, call to function is A, wl (A), g_up */
			C12O16Rotate[i].gf = 
				(float)(GetGF(C12O16Rotate[i].Aul,
			  C12O16Rotate[i].EnergyWN,C12O16Rotate[i].gHi));
		}
		else
		{
			fprintf( ioQQQ, " 12CO line does not have valid gf or A\n" );
			fprintf( ioQQQ, " This is ContCreatePointers\n" );
			puts( "[Stop in ContCreatePointers]" );
			cdEXIT(1);
		}
		if( C13O16Rotate[i].gf > 0. )
		{
			/* the gf value was entered
			 * derive the A, call to function is gf, wl (A), g_up */
			C13O16Rotate[i].Aul = 
				(float)(eina(C13O16Rotate[i].gf,
			  C13O16Rotate[i].EnergyWN,C13O16Rotate[i].gHi));
		}
		else if( C13O16Rotate[i].Aul > 0. )
		{
			/* the Einstion A value was entered
			 * derive the gf, call to function is A, wl (A), g_up */
			C13O16Rotate[i].gf = 
				(float)(GetGF(C13O16Rotate[i].Aul,
			  C13O16Rotate[i].EnergyWN,C13O16Rotate[i].gHi));
		}
		else
		{
			fprintf( ioQQQ, " 13CO line does not have valid gf or A\n" );
			fprintf( ioQQQ, " This is ContCreatePointers\n" );
			puts( "[Stop in ContCreatePointers]" );
			cdEXIT(1);
		}

		/*line wavelength must be gt 0 */
		assert( C12O16Rotate[i].WLAng > 0 );
		assert( C13O16Rotate[i].WLAng > 0 );

		C12O16Rotate[i].damprel = 
			(float)(C12O16Rotate[i].Aul*
		  C12O16Rotate[i].WLAng*1e-8/PI4);
		C13O16Rotate[i].damprel = 
			(float)(C13O16Rotate[i].Aul*
		  C13O16Rotate[i].WLAng*1e-8/PI4);

		/* derive the abs coef, call to function is gf, wl (A), g_low */
		C12O16Rotate[i].opacity = 
			(float)(abscf(C12O16Rotate[i].gf,
		  C12O16Rotate[i].EnergyWN,C12O16Rotate[i].gLo));
		C13O16Rotate[i].opacity = 
			(float)(abscf(C13O16Rotate[i].gf,
		  C13O16Rotate[i].EnergyWN,C13O16Rotate[i].gLo));
	}

	/* level 2 heavy element lines */
	for( i=0; i < nWindLine; i++ )
	{
		/* energy in ryd scale from wavenumber */
		TauLine2[i].EnergyRyd = (float)(TauLine2[i].EnergyWN* WAVNRYD);

		/* derive the A, call to function is gf, wl (A), g_up */
		TauLine2[i].Aul = 
			(float)(eina(TauLine2[i].gf,
		  TauLine2[i].EnergyWN,TauLine2[i].gHi));

		/* coef needed for damping constant */
		TauLine2[i].damprel = 
			(float)(TauLine2[i].Aul*
		  TauLine2[i].WLAng*1e-8/PI4);

		/* derive the abs coef, call to function is gf, wl (A), g_low */
		TauLine2[i].opacity = 
			(float)(abscf(TauLine2[i].gf,
		  TauLine2[i].EnergyWN,TauLine2[i].gLo));

		/* excitation energy of transition in degrees kelvin */
		TauLine2[i].EnergyK = 
			(float)(T1CM*TauLine2[i].EnergyWN);

		/* energy of photon in ergs */
		TauLine2[i].EnergyErg = 
			(float)(ERG1CM*TauLine2[i].EnergyWN);

		/* put null terminated line label into chLab using line array*/
		chIonLbl(chLab, &TauLine2[i]);

		/* get pointer to energy in continuum mesh */
		TauLine2[i].ipCont = (int)ipLineEnergy(TauLine2[i].EnergyRyd , chLab,0 );
		/*if( TauLine2[i].ipCont==751 )
			fprintf(ioQQQ,"( level2 %s\n", chLab);*/
	}

	/* Verner's FeII lines - do first so following labels will over write this
	 * only call if large feii atom is turned on */
	if( FeII.lgFeIION )
	{
		FeIIPoint();
	}

	/* ================================================================================== */
	/*        two photon two-photon 2-nu 2 nu 2 photon 2-photon                           */

	/* >>chng 01 jan 27, add iso-electronic dimension to following vectors */
	/* for induced two photon emission we need mirror image set
	 * of continuum indices for continuum between Lya and helf Lya */
	/* first malloc LIMELM dimension of space */
	if( (hydro.ipSymHy2nu = (long ***)malloc(sizeof(long *)*(unsigned)2 ) )==NULL )
	{
		fprintf( ioQQQ, 
			" ContCreatePointers could not malloc hydro.ipHy2nu 0\n" );
		puts( "[Stop in ContCreatePointers]" );
		cdEXIT(1);
	}
	if( (hydro.AsHy2nu = (float ***)malloc(sizeof(float *)*(unsigned)2 ) )==NULL )
	{
		fprintf( ioQQQ, 
			" ContCreatePointers could not malloc hydro.AsHy2nu 0\n" );
		puts( "[Stop in ContCreatePointers]" );
		cdEXIT(1);
	}
	/* now loop over the two iso-sequences with two photon continua */
	for( ipISO=0; ipISO<2; ++ipISO )
	{
		if( (hydro.ipSymHy2nu[ipISO] = (long **)malloc(sizeof(long *)*(unsigned)LIMELM ) )==NULL )
		{
			fprintf( ioQQQ, 
				" ContCreatePointers could not malloc hydro.ipHy2nu 1\n" );
			puts( "[Stop in ContCreatePointers]" );
			cdEXIT(1);
		}
		if( (hydro.AsHy2nu[ipISO] = (float **)malloc(sizeof(float *)*(unsigned)LIMELM ) )==NULL )
		{
			fprintf( ioQQQ, 
				" ContCreatePointers could not malloc hydro.AsHy2nu 1\n" );
			puts( "[Stop in ContCreatePointers]" );
			cdEXIT(1);
		}
	}

	/* this loop is only for the hydrogenic sequence */
	ipISO = ipHYDROGEN;
	for( ipZ=0; ipZ<LIMELM; ++ipZ )
	{
		if( abundances.lgElmtOn[ipZ] )
		{
			double eLa = 0.75*(1.+ipZ)*(1.+ipZ);
			double charge4 = (1.+ipZ)*(1.+ipZ)*(1.+ipZ)*(1.+ipZ);

			/* half the energy of La */
			hydro.ipHalfLya[ipISO][ipZ] = ipoint(eLa / 2.);
			if( (hydro.ipSymHy2nu[ipISO][ipZ] = (long *)malloc(sizeof(long)*(unsigned)hydro.ipHalfLya[ipISO][ipZ] ) )==NULL )
			{
				fprintf( ioQQQ, 
					" ContCreatePointers could not mallocs hydro.ipSymHy2nu \n" );
				puts( "[Stop in ContCreatePointers]" );
				cdEXIT(1);
			}

			for( i=0; i < hydro.ipHalfLya[ipISO][ipZ]; i++ )
			{
				/* energy is symmetric energy, the other side of half lya */
				energy = eLa - rfield.anu[i];
				/* find index for this symmetric energy */
				hydro.ipSymHy2nu[ipISO][ipZ][i] = ipoint(energy);
				/* >>chng 00 dec 15, do not include Lya itself in the blue end of the integral,
				 * because resulting rates are far too large */
				hydro.ipSymHy2nu[ipISO][ipZ][i] = MIN2( HydroLines[ipZ][ipH2p][ipH1s].ipCont-1, hydro.ipSymHy2nu[ipISO][ipZ][i]);
			}

			/* hydrogen two photon emission 
			 * ipCont-1 is the cell holding Lya itself, and we do not want
			 * to include that in the following sum */
			limit = HydroLines[ipZ][ipH2p][ipH1s].ipCont-1;
			if( (hydro.AsHy2nu[ipISO][ipZ] = (float *)malloc(sizeof(float)*(unsigned)limit ) )==NULL )
			{
				fprintf( ioQQQ, 
					" ContCreatePointers could not mallocs hydro.AsHy2nu \n" );
				puts( "[Stop in ContCreatePointers]" );
				cdEXIT(1);
			}

			for( i=0; i < limit; i++ )
			{
				/* the factor of widflx is not a mistake - when used for induced two-photon
				 * emission the sum will only go up to half of Lya, so only one factor
				 * of widflx will ever appear in the sum */
				/* the energy argument to eva2nu is because that routine works for H itself,
				 * and this reposes the energy scale to one relative to H */
				/* this AsHy2nu function will integrate up to the spontaneous rate if the
				 * integral goes from 0 to half of La, since it includes both of the produced photons,
				 * factor of two forces integral from 0 to half La, to be Aul */
				xnew = eva2nu(0.75*rfield.AnuOrg[i]/eLa)*2.;
				hydro.AsHy2nu[ipISO][ipZ][i] = (float)(xnew*rfield.widflx[i]*charge4);
			}
		}
	}

	/* this loop is only for the hydrogenic sequence */
	ipISO = ipHELIUM;
	for( ipZ=1; ipZ<LIMELM; ++ipZ )
	{
		if( abundances.lgElmtOn[ipZ] )
		{
			double eLa =HeLines[ipZ][ipHe2s1S][ipHe1s1S].EnergyRyd;

			double charge4 = (1.+ipZ)*(1.+ipZ)*(1.+ipZ)*(1.+ipZ);
			/* half the energy of La */
			hydro.ipHalfLya[ipISO][ipZ] = ipoint(eLa / 2.);
			if( (hydro.ipSymHy2nu[ipISO][ipZ] = (long *)malloc(sizeof(long)*(unsigned)hydro.ipHalfLya[ipISO][ipZ] ) )==NULL )
			{
				fprintf( ioQQQ, 
					" ContCreatePointers could not mallocs hydro.ipSymHy2nu \n" );
				puts( "[Stop in ContCreatePointers]" );
				cdEXIT(1);
			}

			for( i=0; i < hydro.ipHalfLya[ipISO][ipZ]; i++ )
			{
				/* energy is symmetric energy, the other side of half lya */
				energy = eLa - rfield.anu[i];
				/* find index for this symmetric energy */
				hydro.ipSymHy2nu[ipISO][ipZ][i] = ipoint(energy);
				/* >>chng 00 dec 15, do not include Lya itself in the blue end of the integral,
				 * because resulting rates are far too large */
				hydro.ipSymHy2nu[ipISO][ipZ][i] = MIN2( HeLines[ipZ][ipHe2s1S][ipHe1s1S].ipCont-1, hydro.ipSymHy2nu[ipISO][ipZ][i]);
			}

			/* hydrogen two photon emission 
			 * ipCont-1 is the cell holding Lya itself, and we do not want
			 * to include that in the following sum */
			limit = HeLines[ipZ][ipHe2s1S][ipHe1s1S].ipCont-1;
			if( (hydro.AsHy2nu[ipISO][ipZ] = (float *)malloc(sizeof(float)*(unsigned)limit ) )==NULL )
			{
				fprintf( ioQQQ, 
					" ContCreatePointers could not mallocs hydro.AsHy2nu \n" );
				puts( "[Stop in ContCreatePointers]" );
				cdEXIT(1);
			}

			for( i=0; i < limit; i++ )
			{
				/* the factor of widflx is not a mistake - when used for induced two-photon
				 * emission the sum will only go up to half of Lya, so only one factor
				 * of widflx will ever appear in the sum */
				/* the energy argument to eva2nu is because that routine works for H itself,
				 * and this reposes the energy scale to one relative to H */
				/* this AsHy2nu function will integrate up to the spontaneous rate if the
				 * integral goes from 0 to half of La, since it includes both of the produced photons,
				 * factor of two forces integral from 0 to half La, to be Aul */
				/* >>chng 00 aug 22, added fac */
				/* ratio of hei to h two-photon A's */
				double fac = 51./8.23;
				xnew = eva2nu(0.75*rfield.AnuOrg[i]/eLa)*2.*fac;
				hydro.AsHy2nu[ipISO][ipZ][i] = (float)(xnew*rfield.widflx[i]*charge4);
			}
		}
	}

	/* HeI two photon emission */
	for( i=0; i < iphe1lCom.iphe1l[0][1]; i++ )
	{
		/* >>chng 00 aug 22, added fac */
		/* ratio of hei to h two-photon A's */
		double fac = 51./8.23;
		em2nu.he12nu[i] = (float)(ehe12p(rfield.AnuOrg[i])*rfield.widflx[i]*fac );
	}

	/* ================================================================================== */

	/* option to print out whole thing with "trace lines" command */
	if( trace.lgTrLine )
	{
		fprintf( ioQQQ, "       WL(Ang)   E(RYD)   IP   gl  gu      gf       A        damp     abs K\n" );
		for( i=1; i <= nLevel1; i++ )
		{
			strcpy( chLab, chLineLbl(&TauLines[i]) );
			iWL_Ang = (long)TauLines[i].WLAng;
			if( iWL_Ang > 1000000 )
			{
				iWL_Ang /= 10000;
			}
			else if( iWL_Ang > 10000 )
			{
				iWL_Ang /= 1000;
			}

			fprintf( ioQQQ, " %10.10s%5ld%10.3e %4i%4ld%4ld%10.2e%10.2e%10.2e%10.2e\n", 
			  chLab, iWL_Ang, RYDLAM/TauLines[i].WLAng, 
			  TauLines[i].ipCont, (long)(TauLines[i].gLo), 
			  (long)(TauLines[i].gHi), TauLines[i].gf, 
			  TauLines[i].Aul, TauLines[i].damprel, 
			  TauLines[i].opacity );
		}

		/* C12O16 lines */
		for( i=0; i < nCORotate; i++ )
		{
			strcpy( chLab, chLineLbl(&C12O16Rotate[i]) );

			iWL_Ang = (long)C12O16Rotate[i].WLAng;

			if( iWL_Ang > 1000000 )
			{
				iWL_Ang /= 10000;
			}
			else if( iWL_Ang > 10000 )
			{
				iWL_Ang /= 1000;
			}
			fprintf( ioQQQ, " %10.10s%5ld%10.3e %4i%4ld%4ld%10.2e%10.2e%10.2e%10.2e\n", 
			  chLab, iWL_Ang, RYDLAM/C12O16Rotate[i].WLAng, 
			  C12O16Rotate[i].ipCont, (long)(C12O16Rotate[i].gLo), 
			  (long)(C12O16Rotate[i].gHi), C12O16Rotate[i].gf, 
			  C12O16Rotate[i].Aul, C12O16Rotate[i].damprel, 
			  C12O16Rotate[i].opacity );
		}

		/* C13O16 lines */
		for( i=0; i < nCORotate; i++ )
		{
			strcpy( chLab, chLineLbl(&C13O16Rotate[i]) );

			iWL_Ang = (long)C13O16Rotate[i].WLAng;

			if( iWL_Ang > 1000000 )
			{
				iWL_Ang /= 10000;
			}
			else if( iWL_Ang > 10000 )
			{
				iWL_Ang /= 1000;
			}
			fprintf( ioQQQ, " %10.10s%5ld%10.3e %4i%4ld%4ld%10.2e%10.2e%10.2e%10.2e\n", 
			  chLab, iWL_Ang, RYDLAM/C13O16Rotate[i].WLAng, 
			  C13O16Rotate[i].ipCont, (long)(C13O16Rotate[i].gLo), 
			  (long)(C13O16Rotate[i].gHi), C13O16Rotate[i].gf, 
			  C13O16Rotate[i].Aul, C13O16Rotate[i].damprel, 
			  C13O16Rotate[i].opacity );
		}

		for( i=0; i < nWindLine; i++ )
		{
			strcpy( chLab, chLineLbl(&TauLine2[i]) );

			iWL_Ang = (long)TauLine2[i].WLAng;

			if( iWL_Ang > 1000000 )
			{
				iWL_Ang /= 10000;
			}
			else if( iWL_Ang > 10000 )
			{
				iWL_Ang /= 1000;
			}
			fprintf( ioQQQ, " %10.10s%5ld%10.3e %4i%4ld%4ld%10.2e%10.2e%10.2e%10.2e\n", 
			  chLab, iWL_Ang, RYDLAM/TauLine2[i].WLAng, 
			  TauLine2[i].ipCont, (long)(TauLine2[i].gLo), 
			  (long)(TauLine2[i].gHi), TauLine2[i].gf, 
			  TauLine2[i].Aul, TauLine2[i].damprel, 
			  TauLine2[i].opacity );
		}
	}

	/* this is an option to kill fine structure line optical depths */
	if( !fston.lgFstOn )
	{
		for( i=1; i <= nLevel1; i++ )
		{
			if( TauLines[i].EnergyWN < 10000. )
			{
				TauLines[i].opacity = 0.;
			}
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->ContCreatePointers()\n", debug_fp );
#	endif
	return;
}

/*fiddle adjust energy bounds of cell with index ipnt so that lower energy
 * matchs ionization edges exactly, called by createpoint */
/* ipnt is index on c scale */
static void fiddle(long int ipnt, 
  double exact)
{
	float Ehi, 
	  Elo,
	  OldEner ;

#	ifdef DEBUG_FUN
	fputs( "<+>fiddle()\n", debug_fp );
#	endif

	/* make low edge of cell exact for photo integrals */
	assert( ipnt >= 0 );
	assert( ipnt < NCELL-1 );

	/* upper edge of higher cell*/
	Ehi = rfield.anu[ipnt] + rfield.widflx[ipnt]*0.5f;
	/* lower edge of lower cell */
	Elo = rfield.anu[ipnt-1] - rfield.widflx[ipnt-1]*0.5f;

	OldEner = rfield.anu[ipnt];

	/* centroid of desired lower energy and upper edge */
	rfield.anu[ipnt] = (float)((Ehi + exact)/2.);
	/* same for lower */
	rfield.anu[ipnt-1] = (float)((exact + Elo)/2.);

	rfield.widflx[ipnt] = (float)(Ehi - exact);
	rfield.widflx[ipnt-1] = (float)(exact - Elo);

	/* bring upper cell down a bit too, since we dont want large change in widflx */
	rfield.anu[ipnt+1] -= (OldEner - rfield.anu[ipnt])/2.f;

	/* sanity check */
	assert( rfield.widflx[ipnt-1] > 0. );
	assert( rfield.widflx[ipnt] > 0. );

#	ifdef DEBUG_FUN
	fputs( " <->fiddle()\n", debug_fp );
#	endif
	return;
}

/*fill define the continuum energy grid over a specified range, called by ContCreatePointers */
static void fill(
  /* lower bounds to this energy range */
  double fenlo, 
  /* upper bounds to this continuum range */
  double fenhi, 
  /* relative energy resolution */
  double resolv, 
  /* starting index within frequency grid */
  long int *n0, 
  /* which energy band this is */
  long int *ipnt)
{
	long int i, 
	  nbin;
	float widtot;
	double aaa , bbb;

#	ifdef DEBUG_FUN
	fputs( "<+>fill()\n", debug_fp );
#	endif

	if( *ipnt > 0 && fabs(1.-fenlo/fillnu.filbnd[*ipnt]) > 1e-4 )
	{
		fprintf( ioQQQ, " FILL improper bounds.\n" );
		fprintf( ioQQQ, " ipnt=%3ld fenlo=%11.4e filbnd(ipnt)=%11.4e\n", 
		  *ipnt, fenlo, fillnu.filbnd[*ipnt] );
		puts( "[Stop in fill]" );
		cdEXIT(1);
	}

	if( *ipnt >= NFILL )
	{
		fprintf( ioQQQ, " The limit to the number of FILL ranges is%4ld\n", 
		  NFILL );
		puts( "[Stop in fill]" );
		cdEXIT(1);
	}

	fillnu.ifill0[*ipnt] = *n0 - 1;
	fillnu.filbnd[*ipnt] = (float)fenlo;
	fillnu.filbnd[*ipnt+1] = (float)fenhi;
	nbin = (long int)(log(10.)*log10(fenhi/fenlo)/resolv + 1);
	fillnu.fildel[*ipnt] = (float)(log10(fenhi/fenlo)/nbin);

	if( fillnu.fildel[*ipnt] < 0.01 )
	{
		fillnu.filres[*ipnt] = (float)(log(10.)*fillnu.fildel[*ipnt]);
	}
	else
	{
		fillnu.filres[*ipnt] = (float)((pow(10.,2.*fillnu.fildel[*ipnt]) - 1.)/2./
			pow(10.,fillnu.fildel[*ipnt]));
	}

	if( *n0 + nbin > NCELL )
	{
		fprintf( ioQQQ, " Fill would need%4ld cells to get to an energy of%10.3e\n", 
		  *n0 + nbin, fenhi );
		fprintf( ioQQQ, " Increase NCELL in all parameter statements.  The current value is%6ld\n", 
		  NCELL );
		puts( "[Stop in fill]" );
		cdEXIT(1);
	}

	widtot = 0.;
	for( i=0; i < nbin; i++ )
	{
		bbb = fillnu.fildel[*ipnt]*((float)(i) + 0.5) ;
		aaa = pow( 10. , bbb );

		rfield.anu[i+fillnu.ifill0[*ipnt]] = (float)(fenlo*aaa);

		rfield.widflx[i+fillnu.ifill0[*ipnt]] = rfield.anu[i+fillnu.ifill0[*ipnt]]*
		  fillnu.filres[*ipnt];

		widtot += rfield.widflx[i+fillnu.ifill0[*ipnt]];
	}

	*n0 += nbin;
	if( trace.lgTrace && (trace.lgConBug || trace.lgPtrace) )
	{
		fprintf( ioQQQ, 
			" FILL range%2ld from%10.3e to%10.3eR in%4ld cell; ener res=%10.3e WIDTOT=%10.3e\n", 
		  *ipnt, 
		  rfield.anu[fillnu.ifill0[*ipnt]] - rfield.widflx[fillnu.ifill0[*ipnt]]/2., 
		  rfield.anu[fillnu.ifill0[*ipnt]+nbin-1] + rfield.widflx[fillnu.ifill0[*ipnt]+nbin-1]/2., 
		  nbin, 
		  fillnu.filres[*ipnt], 
		  widtot );

		fprintf( ioQQQ, " The requested range was%10.3e%10.3e The requested resolution was%10.3e\n", 
		  fenlo, fenhi, resolv );
	}

	/* nrange is number of ranges  */
	*ipnt += 1;
	fillnu.nrange = MAX2(fillnu.nrange,*ipnt);

#	ifdef DEBUG_FUN
	fputs( " <->fill()\n", debug_fp );
#	endif
	return;
}

/*ChckFill perform sanity check confirming that the energy array has been properly filled */
static void ChckFill(void)
{
	int lgFail;
	long int i, 
	  ipnt;
	double energy;

#	ifdef DEBUG_FUN
	fputs( "<+>ChckFill()\n", debug_fp );
#	endif

	lgFail = FALSE;
	for( i=0; i < fillnu.nrange; i++ )
	{
		/* test middle of energy bound */
		energy = (fillnu.filbnd[i] + fillnu.filbnd[i+1])/2.;
		ipnt = ipoint(energy);
		if( energy < rfield.anu[ipnt-1] - rfield.widflx[ipnt-1]*0.5 )
		{
			fprintf( ioQQQ, " ChckFill middle test low fail\n" );
			lgFail = TRUE;
		}

		else if( energy > rfield.anu[ipnt-1] + rfield.widflx[ipnt-1]*0.5 )
		{
			fprintf( ioQQQ, " ChckFill middle test high fail\n" );
			lgFail = TRUE;
		}

		/* test near low bound */
		energy = fillnu.filbnd[i]*0.99 + fillnu.filbnd[i+1]*0.01;
		ipnt = ipoint(energy);
		if( energy < rfield.anu[ipnt-1] - rfield.widflx[ipnt-1]*0.5 )
		{
			fprintf( ioQQQ, " ChckFill low test low fail\n" );
			lgFail = TRUE;
		}

		else if( energy > rfield.anu[ipnt-1] + rfield.widflx[ipnt-1]* 0.5 )
		{
			fprintf( ioQQQ, " ChckFill low test high fail\n" );
			lgFail = TRUE;
		}

		/* test near high bound */
		energy = fillnu.filbnd[i]*0.01 + fillnu.filbnd[i+1]*0.99;
		ipnt = ipoint(energy);
		if( energy < rfield.anu[ipnt-1] - rfield.widflx[ipnt-1]*
		  0.5 )
		{
			fprintf( ioQQQ, " ChckFill high test low fail\n" );
			lgFail = TRUE;
		}
		else if( energy > rfield.anu[ipnt-1] + rfield.widflx[ipnt-1]*
		  0.5 )
		{
			fprintf( ioQQQ, " ChckFill high test high fail\n" );
			lgFail = TRUE;
		}
	}

	if( lgFail )
	{
		puts( "[Stop in ChckFill]" );
		cdEXIT(1);
	}

}

/*eina convert a gf into an Einstein A */
static double eina(double gf, 
	  double enercm, 
	  double gup)
{
	double eina_v;

#	ifdef DEBUG_FUN
	fputs( "<+>eina()\n", debug_fp );
#	endif

	/* derive the transition prob, given the following
	 * call to function is gf, ener in cm^-1, g_up
	 * gf is product of g and oscilator strength
	 * eina = ( gf / 1.499e-8 ) / (wl/1e4)**2 / gup  */
	eina_v = (gf/gup)/1.4992e-8/POW2(1e4/enercm);


#	ifdef DEBUG_FUN
	fputs( " <->eina()\n", debug_fp );
#	endif
	return( eina_v );
}
