/*ConvEdenIoniz called by ConvTempIonz, calls ConvIoniz solving for eden */
/*esum sum free electron density over all species, sets variable erredn.EdenTrue
 *called by ConvEdenIoniz which actually controls the electron density updates */
#include <string.h>
#include "cddefines.h"
#include "tfidle.h"
#include "phycon.h"
#include "trace.h"
#include "hmi.h"
#include "ionfracs.h"
#include "grainvar.h"
#include "hevmolec.h"
#include "converge.h"
/* limit to how many loops we will do */
#define	LOOPMAX	20
/*esum sum free electron density over all species, sets variable erredn.EdenTrue
 *called by ConvEdenIoniz which actually controls the electron density updates */
static void esum(void);

void ConvEdenIoniz(void)
{
	long int LoopEden ,
		/* this will be LOOPMAX at first, then doubled if no oscillations occur*/
		LoopLimit ,
		/* the number of failures passed up here from convIoniz */
		nConvIonizFails;

	static double EdenOld;
	double 
		/* upper and lower limits to the range of the change we want to consider */
		EdenUpperLimit,
		EdenLowerLimit ,
		factor ,
		PreviousChange ;

#	ifdef DEBUG_FUN
	fputs( "<+>ConvEdenIoniz()\n", debug_fp );
#	endif

	/* this routine is called by ConvTempIonz, it calls ConvIonize
	 * and changes the electron density until it converges */

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "  \n" );
		fprintf( ioQQQ, " ConvEdenIoniz entered \n" );
	}

	LoopEden = 0;
	nConvIonizFails = 0;

	/* this will be increased by 2x if, at the end, no oscillations have occurred */
	LoopLimit = LOOPMAX;

	/* will be set true if sign of change, ever changes */
	conv.lgEdenOscl = FALSE;

	/* will be used to look for oscillations in the electron density */
	PreviousChange = 0.;

	strcpy( conv.chConvIoniz, " NONE!!!!!" );

	/* this is ionization/electron density convergence loop
	 * keep calling ionize until lgIonDone is true */
	do 
	{
		/* this flag will be set false below if electron density not within tolerance 
		 * after ionization is recomputed */
		conv.lgConvEden = TRUE;

		/* remember the old electron density for possible printout */
		EdenOld = phycon.eden;

		/* converge the current level of ioinization */
		ConvIoniz();

		/* ESUM sets variable EdenTrue to true elec den but does not change eden */
		esum();

		if( !conv.lgConvIoniz )
		{
			++nConvIonizFails;
		}

		/* following block is to not let electron density change by
		 * too much
		 * phycon.EdenError is allowed error
		 * the default value of EdenError is 0.01 in zerologic
		 * is changed with the SET EdenError command
		 * EdenOld was incoming value of eden
		 * EdenTrue is correct value from esum for current ionization
		 * new eden will be set using these, trying to prevent oscillations */

		/* is error larger than the tolerance we are trying to beat? */
		if( 
			(float)fabs(phycon.eden-phycon.EdenTrue)/MIN2((float)phycon.eden,phycon.EdenTrue) >= 
			phycon.EdenError )
		{
			/* diference is assumed and true electron density is larger
			 * than tolerance, we are not yet converged */
			conv.lgConvEden = FALSE;
			strcpy( conv.chConvIoniz, "Ne big chg" );

			/* SEARCH is TRUE if this is only search for first solution */
			if( conv.lgSearch )
			{
				phycon.eden = (float)pow(10.,
					(log10(phycon.eden) + log10(phycon.EdenTrue))/ 2.);
			}
			else
			{

				/* was the sign of the change in the electron density changing,
				 * or has it ever changed? */
				if( conv.lgConvEden || PreviousChange*(phycon.EdenTrue-phycon.eden) < 0. )
				{
					/* remember that oscillations are happening  */
					conv.lgEdenOscl = TRUE;

					/* changes in electron density are changing sign - be careful,
					 * factor multiplies the error tolerance on the electron density,
					 * largest change allowed is related to this */
					factor = 0.5;
				}
				else
				{
					/* stable so far . .. */
					/* factor = 2.;*/
					/* had been 2, change to 1 stopped oscillations from developing in 
					 * loc blr grid */
					factor = 1.;
				}

				/* use larger factor if not oscillating and change in eden is large */
				if( !conv.lgEdenOscl && 
					fabs( (phycon.EdenTrue-phycon.eden)/phycon.eden) > 3.*phycon.EdenError)
				{
					/* use 3x larger factor */
					factor *= 3.;
				}

				/* now remember this change for the next time through the loop  */
				PreviousChange = phycon.EdenTrue - phycon.eden ;

				/* is the correct electron density - is it too different? */
				EdenUpperLimit = phycon.eden * (1. + phycon.EdenError*factor);
				EdenLowerLimit = phycon.eden / (1. + phycon.EdenError*factor);

				/* get the new electron density */
				if( phycon.EdenTrue > (float)EdenUpperLimit )
				{
					/* this branch, proposed eden too big */
					phycon.eden = (float)EdenUpperLimit;
				}
				else if( phycon.EdenTrue < (float)EdenLowerLimit )
				{
					/* proposed eden too small */
					phycon.eden = (float)EdenLowerLimit;
				}
				else
				{
					/* eden was within fac of the correct value, use it */
					phycon.eden = (float)phycon.EdenTrue;
				}

				/* this is set by 'set eden' command */
				if( phycon.EdenSet > 0. )
				{
					phycon.eden = phycon.EdenSet;
				}

				if( trace.lgTrace && trace.lgNeBug )
				{
					fprintf( ioQQQ, 
						"     ConvEdenIoniz, chg ne to%10.3e EdenOld%10.3e ratio%10.3e EdenTrue=%10.3e\n", 
					  phycon.eden, EdenOld, phycon.eden/EdenOld,  phycon.EdenTrue );
				}
			}
			/* we now have the proposed new electron density */

			/* update density and temperature related variables */
			tfidle();
		}
		/* >>chng 00 oct 21, did not have this branch before so did not make small changes to
		 * electron density (smaller than size that determined eden not converged */
		/* this branch electron density was pretty close, we just need to make a small update */
		else
		{
			/* eden was within fac of the correct value, use it */
			factor = phycon.eden;
			phycon.eden = (float)phycon.EdenTrue;
			/* this is set by 'set eden' command */
			if( phycon.EdenSet > 0. )
			{
				phycon.eden = phycon.EdenSet;
			}

			/* we have changed the density slightly, so now must reevaluate ionization with this new value */
			/* converge the current level of ioinization
			 * but only do this if change was significant */
			if( fabs(factor-phycon.eden)/phycon.eden  > phycon.EdenError/10. )
			{
				ConvIoniz();
			}

			/* ESUM sets variable EdenTrue to true elec den but does not change eden */
			esum();

			{
				/*@-redef@*/
				/* the old and new electron density should agree - print to confirm */
				enum {DEBUG=FALSE};
				/*@+redef@*/
				if( DEBUG )
				{
					fprintf(ioQQQ,"eden change test %.2e %.2e\n", phycon.eden ,phycon.EdenTrue);
				}
			}
		}
		{
			/*@-redef@*/
			/* debug print statement for change in electron density */
			enum {DEBUG=FALSE};
			/*@+redef@*/
			if( DEBUG )
			{
				fprintf(ioQQQ,"edendebugg %li\t%.2e\t%.2e\t%.2e\t%.2e\n", 
					nzone,phycon.eden ,EdenOld, (phycon.eden-EdenOld)/phycon.eden, phycon.EdenTrue);
			}
		}
		if( trace.lgTrace && trace.lgNeBug )
		{
			fprintf( ioQQQ, 
				"     ConvEdenIoniz, chg ne to%10.3e EdenOld%10.3e ratio%10.3e EdenTrue=%10.3e converge=%c\n", 
			  phycon.eden, EdenOld, phycon.eden/EdenOld,  phycon.EdenTrue ,TorF( conv.lgConvEden));
			/* update density and temperature related variables */
			tfidle();
		}

		if( trace.lgTrConvg>2 )
		{
			fprintf( ioQQQ, "     ConvEdenIoniz%4ld new ne=%12.4e true=%12.4e", 
			  LoopEden, phycon.eden ,  phycon.EdenTrue );

			/* this is flag says whether or not eden/eden has converged */
			if( conv.lgConvEden )
			{
				fprintf( ioQQQ, " converged, eden rel error is %g ", 
					(phycon.EdenTrue-phycon.eden)/phycon.EdenTrue );
			}
			else
			{
				fprintf( ioQQQ, " NOCONV corr:%7.3f prop:%7.3f ", 
				   (phycon.EdenTrue-EdenOld)/EdenOld , 
					(phycon.eden-EdenOld)/EdenOld );
			}
			if( conv.lgEdenOscl )
				fprintf( ioQQQ, " EDEN OSCIL \n");
		}

		/* loop until converged, or we give up */
		++LoopEden;

		/* if last iteration through here and not converged, and no oscillations, 
		 * and no ionization failures ,
		 * then increase limit by 2x */
		if( LoopEden == (LOOPMAX-1) && !conv.lgEdenOscl && nConvIonizFails==0)
			/* double the limit, but only one time, and only if no oscillations */
			LoopLimit *= 2;

	}	while( !conv.lgConvEden && LoopEden < LoopLimit );

	if( trace.lgTrConvg )
	{
		fprintf( ioQQQ, "  \n" );
		fprintf( ioQQQ, " ConvEdenIoniz exits, lgConvEden=%1c \n" , TorF(conv.lgConvEden ) );
	}

#	ifdef DEBUG_FUN
	fputs( " <->ConvEdenIoniz()\n", debug_fp );
#	endif
	return;
}

/*esum sum free electron density over all species, sets variable erredn.EdenTrue
 *called by ConvEdenIoniz which actually controls the electron density updates */
static void esum(void)
{
	long int n, 
	  ion, 
	  nelem;

	double csum, 
	  edmole, 
	  edsave[LIMELM], 
	  osum;

#	ifdef DEBUG_FUN
	fputs( "<+>esum()\n", debug_fp );
#	endif

	/* EdenExtra is normally zero, set with EDEN command, to add extra e-
	 * EdenSet is normally zero, set with SET EDEN command, to set e- */
	phycon.EdenTrue = (float)(xIonFracs[2][0] + xIonFracs[2][1] + 
		2.*xIonFracs[3][1] + phycon.EdenExtra);

	/* add on molecules */
	edmole = phycon.EdenTrue;

	phycon.EdenTrue += hmi.h2plus + hevmolec.hevmol[IPCHP-1] + hevmolec.hevmol[IPOHP-1] + 
	  hevmolec.hevmol[IPCOP-1] + hevmolec.hevmol[IPH2OP-1] + hevmolec.hevmol[IPO2P-1] + 
	  hevmolec.hevmol[IPC2P-1] + hevmolec.hevmol[IPH3OP-1] + hevmolec.hevmol[IPCH2P-1] + 
	  hmi.h3plus;

	/* remember electron density from molecules */
	edmole = phycon.EdenTrue - edmole;

	/* pgrains */
	/* >>chng 00 dec 19, include electrons on grains in total sum */
	phycon.EdenTrue += (float)gv.TotalEden;

	/* sum over all metals */
	phycon.EdenMet = 0.;
	/* free free sum should not include hydrogenic since already done
	 * indpenedently in coolr*/
	phycon.EdenFFSum = 0.;
	for( nelem=2; nelem < LIMELM; nelem++ )
	{
		edsave[nelem] = 0.;
		for( ion=2; ion <= nelem; ion++ )
		{
			/* >>chng 96 oct 27, save all contributors to electron density */
			edsave[nelem] += (float)(ion-1)*xIonFracs[ion][nelem];
			phycon.EdenFFSum += POW2( (float)(ion-1) )*xIonFracs[ion][nelem];
		}
		/* add on hydrogenic specides since now counted in free free sum */
		edsave[nelem] += (float)(nelem)*xIonFracs[nelem+1][nelem];
		phycon.EdenMet += (float)edsave[nelem];
	}

	phycon.EdenTrue += phycon.EdenMet;

	/* this variable is set with the set eden command, 
	 * is supposed to override physical electron density */
	if( phycon.EdenSet > 0. )
	{
		phycon.EdenTrue = phycon.EdenSet;
	}

	/* get all electrons not due to hydrogen */
	phycon.EdenNotH = phycon.EdenTrue - xIonFracs[2][0];

	if( phycon.EdenTrue < 0. )
	{
		fprintf(ioQQQ,"ESUM finds EdenTrue negative before H-, =%e\n", phycon.EdenTrue);
		ShowMe();
		puts( "[Stop in esum]" );
		cdEXIT(1);
	}

	/* >>chng 97 mar 18, some extreme models have H- density very large during
	 * 666 Error! cap below should be elsewhere, this sub should produce TrueEden
	 *search phase when far from solution.  do not let eden go neg */
#if 0
	if( search.lgSearch )
	{
		phycon.EdenTrue -= (float)MIN2(phycon.EdenTrue*0.05,hmi.hminus);
	}
	else
	{
		phycon.EdenTrue -= (float)MIN2(phycon.EdenTrue/100.,hmi.hminus);
	}
#	endif
	/* >>chng 01 jan 08, following logic added to stop H- from forcing
	 * negative electron density during very first search, when H- is off */
	/* correct for hminus, and flag error if we go negative */
	if( hmi.hminus > phycon.EdenTrue )
	{
		phycon.EdenTrue -= (float)MIN2(phycon.EdenTrue*0.05,hmi.hminus);
	}
	else
	{
		phycon.EdenTrue -= hmi.hminus;
	}

 	if( trace.lgTrace )
	{
		csum = 0.;
		for( ion=1; ion <= 6; ion++ )
		{
			csum += (float)(ion)*xIonFracs[ion+1][5];
		}

		osum = 0.;
		for( ion=1; ion <= 8; ion++ )
		{
			osum += (float)(ion)*xIonFracs[ion+1][7];
		}

		fprintf( ioQQQ, 
			"     ESUM: zone %li True:%11.4e old:%11.4e hcorr:%.2e ff sum:%10.3e H:%8.2e He:%8.2e\n",
		  nzone,
		  phycon.EdenTrue, 
		  phycon.eden, 
		  phycon.EdenHCorr,
		  phycon.EdenFFSum, 
		  xIonFracs[2][0], 
		  xIonFracs[2][1] + 2.*xIonFracs[3][1]);

		fprintf( ioQQQ, 
			"     ESUM: cont C:%8.2e O:%8.2e met:%8.2e mol:%8.2e\n", 
		  csum, osum, phycon.EdenMet, edmole );

		if( trace.lgNeBug )
		{
			fprintf( ioQQQ, "     ESUM: heav" );
			for(n=2; n < LIMELM; n++)
			{
				fprintf( ioQQQ, " %8.1e", edsave[n] );
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	if( phycon.EdenTrue <= 0. )
	{
		fprintf( ioQQQ, "ESUM finds non-positive electron density.\n" );
		fprintf( ioQQQ, 
			"     ESUM: EdenTrue to%12.4e fm%12.4e ff sum;%10.3e Ne(H):%10.2e Ne(He):%10.2e Ne(C):\n", 
		  phycon.EdenTrue, 
		  phycon.eden, 
		  phycon.EdenFFSum, 
		  xIonFracs[2][0], 
		  xIonFracs[2][1] + 2.*xIonFracs[3][1] );
		ShowMe();
		puts( "[Stop in esum]" );
		cdEXIT(1);
	}

	if(  gv.TotalEden/phycon.EdenTrue > 0.01 )
	{
		fprintf(ioQQQ," esum finds grains have %g percent of the electrons.\n",
			gv.TotalEden/phycon.EdenTrue*100.);
	}

	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			fprintf(ioQQQ,"esumdebugg\t%li\t%.2e\t%.2e\n",
				nzone,
			  phycon.eden, phycon.EdenTrue);
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->esum()\n", debug_fp );
#	endif
	return;
}

