/*CoolNitr evaluate total cooling due to nitrogen */
#include "cddefines.h"
#include "coolheavy.h"
#include "taulines.h"
#include "phycon.h"
#include "poplevls.h"
#include "ionfracs.h"
#include "ligbar.h"
#include "cooling.h"
#include "level2.h"
#include "atomseqberyllium.h"
#include "putcs.h"
#include "level3.h"
#include "pop3.h"
#include "atomseqboron.h"
#include "embesq.h"

void CoolNitr()
{
	float p2;
	double a21, 
	  a31, 
	  a32, 
	  cs, 
	  cs2s2p, 
	  cs2s3p, 
	  cs21, 
	  cs31, 
	  cs32, 
	  p3;
	
#	ifdef DEBUG_FUN
	fputs( "<+>CoolNitr()\n", debug_fp );
#	endif

	/* >>chng 00 dec 11, add made-up collision strengths 
	 * These have made-up collision strengths  */
#	if 0
	PutCS(1.,&TauLines[ipT671]);
	level2(&TauLines[ipT671]);

	PutCS(1.,&TauLines[ipT315]);
	level2(&TauLines[ipT315]);

	PutCS(1.,&TauLines[ipT333]);
	level2(&TauLines[ipT333]);

	PutCS(1.,&TauLines[ipT324]);
	level2(&TauLines[ipT324]);

	PutCS(1.,&TauLines[ipT374g]);
	level2(&TauLines[ipT374g]);

	PutCS(1.,&TauLines[ipT374x]);
	level2(&TauLines[ipT374x]);
#	endif

	/* Nitrogen I 5200, CS and A from 
	 * trans prob from 
	 * >>refer	N1	AS	Butler, K., & Zeippen, C.J. 1984, A&A, 141, 274
	 * collision strengths from 
	 * >>refer	N1	CS	Dopita, M.A., Mason, D.J., Robb, W.D. 1976, ApJ, 207, 102
	 *
	 * N.B. following CS also appears in nitrogen ionization balance */
	cs21 = 1.32e-4*phycon.te/(phycon.te10*phycon.te01);
	cs31 = 3.60e-5*phycon.te/phycon.te10*phycon.te02;
	cs32 = 5.23e-4*phycon.te70*phycon.te10/phycon.te02;
	/* following does not have photoionization from excited state
	 * since already counted in photo balance equation
	 * 666 error! use level3 */
	a21 = 1.28e-5;
	a31 = 5.31e-3;
	a32 = 6.81e-2;
	p3 = pop3(4.,10.,6.,cs21,cs31,cs32,a21,a31,a32,2.769e4,1.38e4,
	  &p2,xIonFracs[1][6],0.);

	CoolHeavy.c5200 = p2*a21*3.83e-12;
	coladd("N  1",5200,CoolHeavy.c5200);
	cooling.dCooldT += CoolHeavy.c5200*(2.769e4*cooling.tsq1 + cooling.halfte);

	CoolHeavy.c10400 = p3*a32*1.91e-12;
	coladd("N  1",10400,CoolHeavy.c10400);

	CoolHeavy.c3466 = p3*a31*5.74e-12;
	coladd("N  1",3466,CoolHeavy.c3466);
	cooling.dCooldT += (CoolHeavy.c10400 + CoolHeavy.c3466)*(4.15e4*
	  cooling.tsq1 + cooling.halfte);

	/* N I 1200, CS from trans procb */
	PutCS(4.1,&TauLines[ipT1200]);
	level2(&TauLines[ipT1200]);

	/* N II 1084, CS from trans prob */
	PutCS(5.5,&TauLines[ipT1085]);
	level2(&TauLines[ipT1085]);

	/* coll data from 
	 * >>refer	N2	CS	Stafford, R.P., Bell, K.L, Hibbert, A. & Wijesundera, W.P.,
	 * >>rerercon 1994, MNRAS 268, 816,
	 * at 10,000K (v weak T dep)
	 * >>chng 00 dec 11, to Lennon & Burke
	 * >>refer	N2	CS	Lennon, D.J., & Burke, V.M., 1994, A&AS, 103, 273-277
	 * transit prob from 
	 * >>refer	N2	AS	Nussbaumer, H., & Rusca, C. 1979, A&A, 72, 129
	 * 
	 * >>chng 00 dec 11, to Lennon & Burke CS */
	/* POP3(                G1,G2,G3,   O12,  O13,  O23,     A21,A31,A32,E12,E23,P2,ABUND,GAM2) */
	CoolHeavy.c5755 = pop3(  9.,5.,1.,  2.64,0.293,0.834,   3.65e-3,0.0316,
	  1.17,21955.,24982.,&p2,xIonFracs[2][6],0.)*1.17*3.46e-12;
	CoolHeavy.c6584 = p2*3.65e-3*3.03e-12;
	cooling.dCooldT += CoolHeavy.c6584*(2.2e4*cooling.tsq1 - cooling.halfte);
	coladd("N  2",5755,CoolHeavy.c5755);
	coladd("N  2",6584,CoolHeavy.c6584);

	/* N II fine structure lines, */
	/* >>chng 00 dec 11, to Lennon & Burke CS */
	PutCS(0.40,&TauLines[ipT205]);
	PutCS(1.13,&TauLines[ipT122]);
	PutCS(0.265,&TauDummy);
	level3(&TauLines[ipT205],&TauLines[ipT122],&TauDummy);

	/* N II 2140, data 
	 * >>refer Stafford, R.P., Bell, K.L, Hibbert, A. & Wijesundera, W.P. 1994,
	 * >>rerercon MNRAS, 268, 816
	 * >>refer	N2	CS	Lennon, D.J., & Burke, V.M., 1994, A&AS, 103, 273-277
	 * A from 
	 * >>refer	N2	AS	Brage, T., Hibbert, A., Leckrone, D.S. 1997, ApJ, 478, 423 */
	PutCS(1.15,&TauLines[ipT2140]);
	level2(&TauLines[ipT2140]);

	/* N III 989.8, CS from 
	 * >>refer	N3	CS	Blum, R.D., & Pradhan, A.K. 1992, ApJS 80, 425 */
	PutCS(7.12,&TauLines[ipT990]);
	level2(&TauLines[ipT990]);

	/* 57 micron N III, A=
	 * >>refer	N3	AS	Froese Fischer, C. 1983, J.Phys. B, 16, 157
	 * collision strength from 
	 * >>refer	N3	CS	Blum, R.D., & Pradhan, A.K. 1992, ApJS 80, 425 */
	cs = MIN2(1.90,0.2291*phycon.te10*phycon.te10);
	PutCS(cs,&TauLines[ipT57]);
	/*level2(&TauLines[ipT57]);*/
	/*AtomSeqBoron compute cooling from 5-level boron sequence model atom */
	/* >>refer n3 collision Blum, R.D., & Pradhan, A.K., 1992, ApJS 80, 425*/
	AtomSeqBoron(&TauLines[ipT57], 
	  &TauLines[ipN3_1749], 
	  &TauLines[ipN3_1747], 
	  &TauLines[ipN3_1754], 
	  &TauLines[ipN3_1752], 
	  &TauLines[ipN3_1751], 
	  0.201 , 1.088 , 0.668 , 2.044 , "N  3");

	/* N IV 1486, collisions within 3P just guess
	 * cs to ground from 
	 * >>refer	N4	CS	Ramsbottom, C.A., Berrington, K.A., Hibbert, A., Bell, K.L. 1994,
	 * >>refercon Physica Scripta, 50, 246 */
	if( phycon.te > 1.584e4 )
	{
		cs = 21.346/(phycon.te10*phycon.te10*phycon.te10*phycon.te02);
	}
	else
	{
		cs = 75.221/(phycon.sqrte/phycon.te03/phycon.te02);
	}
	cs = MAX2(0.01,cs);
	PutCS(cs,&TauLines[ipT1486]);
	/* BESEQ(CS23,CS24,CS34,tarray,A41) */
	AtomSeqBeryllium(.9,.9,3.0,&TauLines[ipT1486],.0115);
	embesq.em1486 = (float)(PopLevls.PopLevels[3]*0.0115*1.34e-11);

	/* N IV 765, CS from 
	 * >>refer	N4	CS	Ramsbottom, C.A., Berrington, K.A., Hibbert, A., Bell, K.L. 1994,
	 * >>refercon Physica Scripta, 50, 246 */
	cs = MIN2(4.0,1.864*phycon.te03*phycon.te03);
	PutCS(cs,&TauLines[ipT765]);
	level2(&TauLines[ipT765]);

	/* N V 1240
	 * >>refer	N5	CS	Cochrane, D.M., & McWhirter, R.W.P. 1983, PhyS, 28, 25 */
	ligbar(7,&TauLines[ipT1239],&TauLines[ipT209],&cs2s2p,&cs2s3p);
	PutCS(cs2s2p,&TauLines[ipT1239]);
	PutCS(cs2s2p*0.5,&TauLines[ipT1243]);
	PutCS(1.0,&TauDummy);
	level3(&TauLines[ipT1243],&TauDummy,&TauLines[ipT1239]);
	PutCS(cs2s3p,&TauLines[ipT209]);
	level2(&TauLines[ipT209]);

#	ifdef DEBUG_FUN
	fputs( " <->CoolNitr()\n", debug_fp );
#	endif
	return;
}

