/*DoPunch produce punch output during calculation */
/* chTime is 'MIDL' during calculation, 'LAST' at the end */
#include <string.h>
#include "cddefines.h"
#include "cddrive.h"
#include "physconst.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "nhe1lvl.h"
#include "secondaries.h"
#include "nhe3lvl.h"
#include "grainvar.h"
#include "linesave.h"
#include "pnunit.h"
#include "chlinelbl.h"
#include "mappar.h"
#include "colden.h"
#include "itercnt.h"
#include "hevmolec.h"
#include "converge.h"
#include "heating.h"
#include "sphere.h"
#include "called.h"
#include "abundances.h"
#include "compton.h"
#include "opacity.h"
#include "rfield.h"
#include "phycon.h"
#include "timesc.h"
#include "radius.h"
#include "pop371.h"
#include "assertresults.h"
#include "h.h"
#include "he.h"
#include "heat.h"
#include "nh.h"
#include "phe1lv.h"
#include "heopfr.h"
#include "maxcor.h"
#include "wind.h"
#include "hmi.h"
#include "levery.h"
#include "ipvfil.h"
#include "punchskip.h"
#include "heatcool.h"
#include "hcaseb.h"
#include "ionfracs.h"
#include "pressure.h"
#include "he3rate.h"
#include "he1rate.h"
#include "herec.h"
#include "he3gams.h"
#include "nhe1.h"
#include "nsshells.h"
#include "ph1com.h"
#include "elementnames.h"
#include "shell.h"
#include "plwidth.h"
#include "hphoto.h"
#include "ipoint.h"
#include "punlin.h"
#include "pfeii.h"
#include "puncool.h"
#include "pungamma.h"
#include "pgaunt.h"
#include "punheat.h"
#include "gammas.h"
#include "pllabels.h"
#include "dopunchlinedata.h"
#include "result.h"
#include "punspec.h"
#include "chargtran.h"
#include "punt.h"
#include "pmeani.h"
#include "plankf.h"
#include "readar.h"
#include "rdinit.h"
#include "punresult.h"
#include "dopunch.h"

static void PunchLineStuff(FILE * io,char *chJob , float xLimit);

/*plinin produce the 'punch lines intensity' output */
static void plinin(FILE * io);
static void pun1Line( EmLine * t , FILE * io , float xLimit , long index );

void DoPunch(
	char *chTime) /* chTime is null terminated 4 char string, either "MIDL" or "LAST" */
{
	char chEner[LIMELM][7];
	int lgOK, 
	  lgTlkSav;
	FILE * isav;
	long int
	  i, 
	  i1, 
	  ion, 
	  ipConMax, 
	  ipLinMax, 
	  ips, 
	  j, 
	  jj, 
	  limit, 
	  n, 
	  nd, 
	  nelem;
	double ConMax, 
	  RateInter, 
	  av, 
	  conem, 
	  damp, 
	  eps, 
	  flxatt, 
	  flxcor, 
	  flxin, 
	  flxref, 
	  flxtrn, 
	  fout, 
	  fref, 
	  fsum, 
	  opConSum, 
	  opLinSum, 
	  pop, 
	  refac, 
	  stage, 
	  sum, 
	  texc, 
	  xLinMax;

#	ifdef DEBUG_FUN
	fputs( "<+>DoPunch()\n", debug_fp );
#	endif

	/* 
	 * the "last" option on punch command, to punch on last iteration,
	 * is parsed at the top of the loop in only one place.  
	 * no further action is needed at all for punch last to work
	 * ok throughout this routine 
	 */

	/* 
	 * each branch can have a test whether chTime is or is not "LAST"
	 * if( strcmp(chTime,"LAST") == 0 )
	 * if "LAST" then this is last call to routine after calc complete
	 * punch only if "LAST" when results at end of cal are needed
	 * test for .not."LAST" is for every zone result, where you do not
	 * want to punch last zone twice
	 */

	/* punch results on unit ipPnunit  */
	if( pnunit.npunch < 1 )
	{ 
#		ifdef DEBUG_FUN
		fputs( " <->DoPunch()\n", debug_fp );
#		endif
		return;
	}

	for( i=0; i < pnunit.npunch; i++ )
	{
		/* this global variable to remember where in the punch stack we are */
		pnunit.ipConPun = i;
		/* 
		 * lgPunLstIter set true if 'last' key occured on punch command
		 * normally is false.  This will skip punching if last set and
		 * this is not last iteration
		 */

		if( IterCnt.lgLastIt || (!pnunit.lgPunLstIter[i]) )
		{

			if( strcmp(pnunit.chPunch[i],"ABUN") == 0 )
			{
				/* punch abundances vs depth */
				if( strcmp(chTime,"LAST") != 0 )
				{
					for( j=1; j <= LIMELM; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "%6.2f", 
						  log10(xIonFracs[0][j-1]) );
					}
					fprintf( pnunit.ipPnunit[i], "\n" );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"AGES") == 0 )
			{
				/* punch timescales vs depth */
				if( strcmp(chTime,"LAST") != 0 )
				{
					fprintf( pnunit.ipPnunit[i], "%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
					  /* depth, cm */
					  radius.depth,
					  /* cooling timescale */
					  phycon.pden*BOLTZMANN*1.5*phycon.te/ heat.htot, 
					  /* H2 formation timescale */
					  timesc.AgeH2MoleDest, 
					  /* CO formation timescale */
					  timesc.AgeCOMoleDest, 
					  /* H recombination timescale */
					  1./(phycon.eden*2.90e-10/(phycon.te70*phycon.te10/phycon.te03)) );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"ASSE") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch the assert output */
					/*lint -e534 ignore return value of following  - nothing to do with it */
					lgCheckAsserts(pnunit.ipPnunit[i]);
					/*lint +e534 ignore return value of following  - nothing to do with it */
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CHAR") == 0 )
			{
				/* charge exchange rate coefficients, entered with
				 * punch charge transfer command.  Queries Jim Kingdon's
				 * CT fits and routines to get H - He and higher CT rates
				 * rates are evaluated at the current temperature, which can
				 * be specified with the constant temperature commmand */
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* first group is charge transfer recombination,
					 * the process A+x + H => A+x-1 + H+ */
					fprintf( pnunit.ipPnunit[i], "recombination rates, atomic number\n");
					for( j=1; j < LIMELM; j++ )
					{
						/* first number is atomic number, so add 1 to get onto physical scale */
						/* >>chng 00 may 09, caught by Jon Slavin */
						fprintf( pnunit.ipPnunit[i], "%3ld", j+1 );
						/* only do first 6 unless element very light, then do less */
						limit = MIN2(6,j);
						for( jj=0; jj < limit; jj++ )
						{
							fprintf( pnunit.ipPnunit[i], "%10.2e", 
							  HCTRecom(jj,j) );
						}
						fprintf( pnunit.ipPnunit[i], "\n" );
					}

					/* second group is charge transfer ionization,
					 * the process A+x + H+ => A+x+1 + H */
					fprintf( pnunit.ipPnunit[i], "\nionization rates, atomic number\n");
					for( j=1; j < LIMELM; j++ )
					{
						limit = MIN2(6,j);
						fprintf( pnunit.ipPnunit[i], "%3ld", j );
						for( jj=0; jj < limit; jj++ )
						{
							fprintf( pnunit.ipPnunit[i], "%10.2e", 
							  HCTIon(jj,j) );
						}
						fprintf( pnunit.ipPnunit[i], "\n" );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"COOL") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
					PunCool(pnunit.ipPnunit[i]);
			}

			else if( strcmp(pnunit.chPunch[i],"COMP") == 0 )
			{
				/* compton energy exchange coefficients */
				if( strcmp(chTime,"LAST") != 0 )
				{
					for( jj=0; jj<rfield.nflux; jj = jj + PunchSkip.ncPunchSkip)
					{
						fprintf( pnunit.ipPnunit[i], "%10.2e%10.2e%10.2e\n", 
						  rfield.anu[jj], Compton.comup[jj]/rfield.widflx[jj], 
						  Compton.comdn[jj]/rfield.widflx[jj] );
					}
				}
			}

			/* punch continuum commands */
			else if( strcmp(pnunit.chPunch[i],"CON ") == 0 )
			{
				/* this branch is the usual "punch continuum" case */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					if( sphere.lgSphere )
					{
						refac = 0.;
					}
					else
					{
						refac = 1.;
					}

					for( j=0; j<rfield.nflux ; j = j+PunchSkip.ncPunchSkip)
					{
						/* four continua predicted here;
						 * incident, attenuated incident, emitted,
						 * then attenuated incident + emitted, last reflected
						 * reflected continuum is stored relative to inner radius
						 * others are stored for this radius */

						/* NB this code also used in punch emitted,
						 * transmitted continuum commands */

						/* the incident continuum */
						flxin = rfield.FluxSave[j]*rfield.anu2[j]*
						  EN1RYD/rfield.widflx[j];

						/* the reflected continuum */
						flxref = (rfield.anu2[j]*(rfield.ConRefIncid[j]+rfield.ConRefDiff[j])/rfield.widflx[j] +
							rfield.anu[j]*PLWidth.PunchLWidth*rfield.reflin[j])*refac*EN1RYD;

						/* the attenuated incident continuum */
						flxatt = rfield.flux[j]*rfield.anu2[j]*EN1RYD/
						  rfield.widflx[j]*radius.r1r0sq;

						/* the emitted continuum */
						conem = ((rfield.ConOutNoInter[j] + rfield.ConOutInter[j])/
						  rfield.widflx[j]*rfield.anu2[j] + PLWidth.PunchLWidth*
						  rfield.outlin[j]*rfield.anu[j])*radius.r1r0sq*
						  EN1RYD*sphere.covgeo;

						/* sum of emitted and transmitted continua */
						flxtrn = conem + flxatt;

						/* photon energy in appropriate energy or wavelength units */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", AnuUnit(rfield.AnuOrg[j]) );
						/* indicent continuum */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", flxin ); 
						/* trans cont */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", flxatt ); 
						/* DiffOut cont */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", conem ); 
						/* net trans cont */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", flxtrn ); 
						/* reflc cont */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", flxref ); 
						/* total cont */
						fprintf( pnunit.ipPnunit[i],"%.3e\t", flxref + flxtrn ); 
						fprintf( pnunit.ipPnunit[i], "%4.4s\t%4.4s\n", 
						/* line	label */
						  rfield.chLineLabel[j] ,
						/* cont label*/
						  rfield.chContLabel[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONC") == 0 )
			{
				/* punch incident continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* incident continuum */
					for( j=0; j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						flxin = rfield.FluxSave[j]*rfield.anu2[j]*
						  EN1RYD/rfield.widflx[j];
						/* >>chng 96 oct 22, format of anu to 11.5 to resolve grid near 1 ryd */
						fprintf( pnunit.ipPnunit[i], "%12.4e %10.3e\n", 
						  AnuUnit(rfield.AnuOrg[j]), flxin );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONG") == 0 )
			{
				/* punch emitted grain continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* tried to remember emission from the two grain types in rtdiffuse */
					for( j=0; j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						fsum = gv.GraphiteEmission[j]*rfield.anu2[j]*
						  EN1RYD/rfield.widflx[j] *radius.r1r0sq*sphere.covgeo;

						fout = gv.SilicateEmission[j]*rfield.anu2[j]*
						  EN1RYD/rfield.widflx[j] *radius.r1r0sq*sphere.covgeo;

						/* >>chng 96 oct 22, format of anu to 11.5 to resolve grid near 1 ryd */
						fprintf( pnunit.ipPnunit[i], "%12.4e %10.3e %10.3e\n", 
						  AnuUnit(rfield.AnuOrg[j]) , fsum , fout );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONR") == 0 )
			{
				/* punch reflected continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					if( sphere.lgSphere )
					{
						fprintf( pnunit.ipPnunit[i], " Reflected continuum not predicted when SPHERE is set.\n" );
						fprintf( ioQQQ , 
							"\n\n>>>>>>>>>>>>>\n Reflected continuum not predicted when SPHERE is set.\n" );
						puts( "[Stop in DoPunch]" );
						cdEXIT(1);
					}

					for( j=0; j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						/* reflected continuum */
						flxref = rfield.anu2[j]*(rfield.ConRefIncid[j]+rfield.ConRefDiff[j])/
						  rfield.widflx[j]*EN1RYD;
						/* reflected lines */
						fref = rfield.anu[j]*PLWidth.PunchLWidth*
						  rfield.reflin[j]*EN1RYD;
						/* ratio of reflected to incident continuum, the albedo */
						if( rfield.FluxSave[j] > 1e-25 )
						{
							av = rfield.ConRefIncid[j]/rfield.FluxSave[j];
						}
						else
						{
							av = 0.;
						}
						/* >>chng 96 oct 22, format of anu to 12.5 to resolve grid near 1 ryd */
						fprintf( pnunit.ipPnunit[i], "%12.5e%12.4e%12.4e%12.4e%12.4e %4.4s\n", 
						  AnuUnit(rfield.AnuOrg[j]), flxref, fref, flxref + fref, 
						  av, rfield.chContLabel[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONB") == 0 )
			{
				/* punch continuum bins binning */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") != 0 )
				{
					for( j=0; j<rfield.nupper; j = j + PunchSkip.ncPunchSkip)
					{
						fprintf( pnunit.ipPnunit[i], "%14.5e %14.5e %14.5e\n", 
						  AnuUnit(rfield.AnuOrg[j]), rfield.anu[j], rfield.widflx[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"COND") == 0 )
			{
				/* punch diffuse continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* this option to punch diffuse continuum */
					for( j=0; j<rfield.nflux; j = j+PunchSkip.ncPunchSkip)
					{
						/* >>chng 96 oct 22, format of anu to 12.5 to resolve grid near 1 ryd */
						fprintf( pnunit.ipPnunit[i], "%12.5e %12.5e %12.5e\n", 
						  AnuUnit(rfield.AnuOrg[j]), 
						  rfield.ConLocInter[j]*rfield.anu2[j]*EN1RYD/rfield.widflx[j] ,
						  rfield.ConLocNoInter[j]*rfield.anu2[j]*EN1RYD/rfield.widflx[j]);
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONE") == 0 )
			{
				/* punch emitted continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch emitted continuum */
					if( sphere.lgSphere )
					{
						refac = 0.;
					}
					else
					{
						refac = 1.;
					}

					for( j=0; j<rfield.nflux;j = j +PunchSkip.ncPunchSkip)
					{
						/* this is the reflected component */
						flxref = (rfield.anu2[j]*(rfield.ConRefIncid[j]+rfield.ConRefDiff[j])/
						  rfield.widflx[j] + rfield.anu[j]*PLWidth.PunchLWidth*
						  rfield.reflin[j])*refac*EN1RYD;

						/* this is the total emission in the outward direction */
						conem = (rfield.ConOutNoInter[j] + 
							rfield.ConOutInter[j])/
						  rfield.widflx[j]*rfield.anu2[j] + PLWidth.PunchLWidth*
						  rfield.outlin[j]*rfield.anu[j];
						conem *= radius.r1r0sq*EN1RYD*sphere.covgeo;

						/* output: photon energy, reflected, outward, total emission
						 *  >>chng 96 oct 22, format of anu to 11.5 to resolve grid near 1 ryd */
						fprintf( pnunit.ipPnunit[i], "%11.5e%11.3e%11.3e%11.3e %4.4s %4.4s \n", 
						  AnuUnit(rfield.AnuOrg[j]), flxref, conem, flxref + conem, 
						  rfield.chLineLabel[j], rfield.chContLabel[j]
						   );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONF") == 0 )
			{
				/* FeII continuuum, check that FeII is turned on */
				if( FeII.lgFeIION && ( strcmp(chTime,"LAST") == 0 ))
				{
					for( j=0; j < nFeIIConBins; j++ )
					{
						/* [x][0] is center wavelength, [1] and [2] are upper and
						 * lower bounds in Angstroms.  
						 * these are set in FeIIZero */
						/* emission from large FeII atom, integrated over band */
						fprintf( pnunit.ipPnunit[i], "%.2f\t%e \n", 
						(FeII_Cont[j][1]+FeII_Cont[j][2])/2.,
						FeII_Cont[j][0] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONi") == 0 )
			{
				/* punch continuum interactions */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				/* continuum interactions */
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* this is option to set lowest energy */
					if( pnunit.punarg[0][i] <= 0. )
					{
						i1 = 1;
					}
					else if( pnunit.punarg[0][i] < 100. )
					{
						i1 = ipoint(pnunit.punarg[0][i]);
					}
					else
					{
						i1 = (long int)pnunit.punarg[0][i];
					}

					fref = 0.;
					fout = 0.;
					fsum = 0.;
					sum = 0.;
					flxin = 0.;

					for( j=i1-1; j < rfield.nflux; j++ )
					{
						fref += rfield.flux[j]*opac.opac[j];
						fout += rfield.otslin[j]*opac.opac[j];
						fsum += rfield.otscon[j]*opac.opac[j];
						sum += (rfield.ConOutInter[j]+rfield.ConOutRecInter[j])*opac.opac[j];
						flxin += rfield.outlin[j]*opac.opac[j];
					}
					fprintf( pnunit.ipPnunit[i], "%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
					  fref, fout, fsum, sum, flxin );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONI") == 0 )
			{
				/* punch ionizing continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				if( strcmp(chTime,"LAST") == 0 )
				{
					/* this flag will remember whether we have ever printed anything */
					int lgPrt=FALSE;

					/* punch ionizing continuum command
					 * this is option to set lowest energy */
					if( pnunit.punarg[0][i] <= 0. )
					{
						i1 = 1;
					}
					else if( pnunit.punarg[0][i] < 100. )
					{
						i1 = ipoint(pnunit.punarg[0][i]);
					}
					else
					{
						i1 = (long int)pnunit.punarg[0][i];
					}

					sum = 0.;
					for( j=i1-1; j < rfield.nflux; j++ )
					{
						flxcor = rfield.flux[j] + rfield.otslin[j] + 
						  rfield.otscon[j] + rfield.ConOutInter[j]+rfield.ConOutRecInter[j] + 
						  rfield.outlin[j];
						sum += flxcor*opac.opac[j];
					}

					if( sum > 0. )
					{
						sum = 1./sum;
					}
					else
					{
						sum = 1.;
					}

					fsum = 0.;

					for( j=i1-1; j<rfield.nflux ; ++j)
					{
						flxcor = rfield.flux[j] + rfield.otslin[j] + 
						  rfield.otscon[j] + rfield.ConOutInter[j]+rfield.ConOutRecInter[j] + 
						  rfield.outlin[j];
						fsum += flxcor*opac.opac[j];

						/* punched quantities are freq, flux, flux*cross sec,
						 * fraction of total, integral fraction of total */
						RateInter = flxcor*opac.opac[j]*sum;

						/* punage(i,2) is lowest interaction rate to consider, def=0.01 (1 percnt) */
						if( (RateInter >= pnunit.punarg[1][i]) && (flxcor > SMALLFLOAT) )
						{
							lgPrt = TRUE;
							/* >>chng 96 oct 22, format of anu to 11.5 to resolve grid near 1 ryd */
							fprintf( pnunit.ipPnunit[i], 
								"%11.5e\t%10.2e\t%10.2e\t%5.2f\t%5.2f\t%5.2f\t%5.2f\t%10.2e\t%10.2e\t%4.4s\t%4.4s\n", 
							  AnuUnit(rfield.AnuOrg[j]), 
							  flxcor, 
							  flxcor*opac.opac[j], 
							  rfield.flux[j]/flxcor, 
							  rfield.otslin[j]/flxcor, 
							  rfield.otscon[j]/flxcor, 
							  (rfield.ConOutInter[j]+rfield.ConOutRecInter[j] + rfield.outlin[j])/flxcor, 
							  RateInter, 
							  fsum*sum, 
							  rfield.chLineLabel[j], 
							  rfield.chContLabel[j] );
						}
					}
					if( !lgPrt )
					{
						/* entered logical block but did not print anything */
						fprintf(ioQQQ,
							" possible insanity, CONI did not print, sum fsum were %.2e %.2e\n",sum,fsum);
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CORA") == 0 )
			{
				/* punch raw continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				if( strcmp(chTime,"LAST") == 0 )
				{
					/* this option to punch all raw ionizing continuum */
					for( j=0;j<rfield.nflux;j = j + PunchSkip.ncPunchSkip)
					{
						fprintf( pnunit.ipPnunit[i], 
							"%.5e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%.3e\t%4.4s\t%4.4s\n", 
						  AnuUnit(rfield.AnuOrg[j]), 
						  rfield.flux[j], 
						  rfield.otslin[j], 
						  rfield.otscon[j], 
						  rfield.ConRefIncid[j],
						  rfield.ConRefDiff[j], 
						  rfield.ConOutInter[j],
						  rfield.ConOutRecInter[j], 
						  rfield.outlin[j], 
						  rfield.ConOutNoInter[j] ,
						  rfield.ThrowOut[j] ,
						  rfield.chLineLabel[j], 
						  rfield.chContLabel[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONo") == 0 )
			{
				/* punch outward local continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				if( strcmp(chTime,"LAST") == 0 )
				{
					/* option to punch out outward continuum here */
					for( j=0;j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						fprintf( pnunit.ipPnunit[i], "%11.5e%10.2e%10.2e%10.2e%10.2e\n", 
						  AnuUnit(rfield.AnuOrg[j]), 
						  rfield.SavOutCon[j], 
						  rfield.ConOutInter[j] + rfield.outlin[j], 
						  rfield.SavOutCon[j]* opac.opac[j]*rfield.anu[j], 
						  (rfield.flux[j] + rfield.otscon[j] + rfield.otslin[j] + 
						  rfield.ConOutInter[j] + rfield.ConOutInter[j])*opac.opac[j]*
						  rfield.anu[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONO") == 0 )
			{
				/* punch outward continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				if( strcmp(chTime,"LAST") == 0 )
				{
					/* option to punch out outward continuum */
					for( j=0; j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						fprintf( pnunit.ipPnunit[i], "%11.5e%10.2e%10.2e%10.2e%10.2e\n", 
						  AnuUnit(rfield.AnuOrg[j]), 
						  rfield.flux[j]*rfield.anu2[j]* EN1RYD/rfield.widflx[j]*radius.r1r0sq, 
						  rfield.ConOutInter[j]/rfield.widflx[j]*rfield.anu2[j]*radius.r1r0sq*EN1RYD, 
						  PLWidth.PunchLWidth* rfield.outlin[j]*rfield.anu[j]*radius.r1r0sq*EN1RYD, 
						  (rfield.ConOutInter[j]/rfield.widflx[j]*
						  rfield.anu2[j] + PLWidth.PunchLWidth*rfield.outlin[j]*
						  rfield.anu[j])*radius.r1r0sq*EN1RYD );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"CONT") == 0 )
			{
				/* punch transmitted continuum */
				/* set pointer for possible change in units of energy in continuum
				 * AnuUnit will give anu in whatever units were set with punch units */

				if( strcmp(chTime,"LAST") == 0 )
				{
					/* this option to punch transmitted continuum */
					for( j=0;j<rfield.nflux; j = j + PunchSkip.ncPunchSkip)
					{
						/* attenuated incident continuum
						 * >>chng 97 jul 10, remove PunchLWidth from this one only since
						 * we must conserve energy even in lines */
						flxatt = rfield.flux[j]*rfield.anu2[j]*EN1RYD/
						  rfield.widflx[j]*radius.r1r0sq;

						/*conem = (rfield.ConOutNoInter[j] + rfield.ConOutInter[j]+rfield.outlin[j])*
						  rfield.anu2[j] ;
						conem *= radius.r1r0sq*EN1RYD*sphere.covgeo;*/
						/* >>chng 00 jan 03, above did not include all contributors.  
						 * Pasted in below from usual
						 * punch continuum command */
						conem = ((rfield.ConOutNoInter[j] + rfield.ConOutInter[j])/
						  rfield.widflx[j]*rfield.anu2[j] + PLWidth.PunchLWidth*
						  rfield.outlin[j]*rfield.anu[j])*radius.r1r0sq*
						  EN1RYD*sphere.covgeo;

						flxtrn = conem + flxatt;

						/* use AnuOrg here instead of anu since probably
						 * going to be used by table read
						 * and we want good anu array for sanity check*/
						fprintf( pnunit.ipPnunit[i],"%10.3e\t", AnuUnit(rfield.AnuOrg[j]) );
						fprintf( pnunit.ipPnunit[i],"%10.3e\n", flxtrn );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"DUSO") == 0 )
			{
				/* punch grain opacity */
				if( strcmp(chTime,"LAST") == 0 )
				{
					for( j=1; j <= rfield.nflux; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "%12.4e%10.2e%10.2e%10.2e\n", 
						  rfield.anu[j-1], gv.dstab[j-1] + gv.dstsc[j-1], 
						  gv.dstab[j-1], gv.dstsc[j-1] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"DUSP") == 0 )
			{
				/* punch grain physical conditions */
				for( nd=0; nd < gv.nBin; nd++ )
				{
					fprintf( pnunit.ipPnunit[i], "%.2e\t", gv.bin[nd]->tedust );
				}

				for( nd=0; nd < gv.nBin; nd++ )
				{
					fprintf( pnunit.ipPnunit[i], "%.2e\t", gv.bin[nd]->dstpot*EVRYD );
				}

				for( nd=0; nd < gv.nBin; nd++ )
				{
					fprintf( pnunit.ipPnunit[i], "%.2e\t", gv.bin[nd]->DustDftVel );
				}

				fprintf( pnunit.ipPnunit[i], "%.2e\t%.2e\n", 
				  HeatingCom.heating[13][0]/heat.htot, 
				  gv.GasCoolColl/heat.htot );
			}

			else if( strcmp(pnunit.chPunch[i],"DUSQ") == 0 )
			{
				/* punch grain Qs */
				if( strcmp(chTime,"LAST") == 0 )
				{
					for( j=0; j < rfield.nflux; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "%11.4e", 
						  rfield.anu[j] );
						for( nd=0; nd < gv.nBin; nd++ )
						{
							fprintf( pnunit.ipPnunit[i], "%9.1e%9.1e", 
							   gv.bin[nd]->dstab1[j]*4./gv.bin[nd]->IntArea,
							   gv.bin[nd]->dstsc1[j]*4./gv.bin[nd]->IntArea );
						}
						fprintf( pnunit.ipPnunit[i], "\n" );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"ELEM") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* this is the pointer to atomic number */
					jj = (long int)pnunit.punarg[0][i];
					fprintf( pnunit.ipPnunit[i], " %.4e", 
					  radius.depth );

					for( j=1; j <= (jj + 1); j++ )
					{
						fprintf( pnunit.ipPnunit[i], "\t%.2e", 
						  xIonFracs[j][jj-1]/xIonFracs[0][jj-1] );
					}
					fprintf( pnunit.ipPnunit[i], "\n" );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"RECE") == 0 )
			{
				/* punch recombination efficiencies, 
				 * option turned on with the  "punch recombination efficiencies" command
				 * output for the punch recombination coefficients command is actually
				 * produced by a series of routines, as they generate the recombination
				 * coefficients.  these include 
				 * dielsupres, helium, hydrorecom, iibod, and makerecomb*/
				fprintf( pnunit.ipPnunit[i], 
					"%12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n", 
				  hydro.hrec[0][0][ipRecRad], 
				  hydro.hrec[0][0][ipRecNetEsc] ,
				  hydro.hrec[0][2][ipRecRad],
				  hydro.hrec[0][2][ipRecNetEsc],
				  phe1lv.he1rec[1][0], 
				  phe1lv.he1rec[2][0], 
				  heopfr.ophe1f[0] );

			}
			else if( strcmp(pnunit.chPunch[i],"FE2f") == 0 )
			{
				/* set with punch feii fred command, this punches some stuff from
				* fred hamann's feii atom */
				if( strcmp(chTime,"LAST") != 0 )
					pfeii(pnunit.ipPnunit[i]);

			}
			else if( strcmp(pnunit.chPunch[i],"FE2d") == 0 )
			{
				/* punch some departure cefficients for large feii atom */
				if( strcmp(chTime,"LAST") != 0 )
					FeIIPunDepart(pnunit.ipPnunit[i],FALSE);

			}
			else if( strcmp(pnunit.chPunch[i],"FE2D") == 0 )
			{
				/* punch all departure cefficients for large feii atom */
				if( strcmp(chTime,"LAST") != 0 )
					FeIIPunDepart(pnunit.ipPnunit[i],TRUE);

			}
			else if( strcmp(pnunit.chPunch[i],"FE2p") == 0 )
			{
				/* punch some level populations for large feii atom */
				if( strcmp(chTime,"LAST") != 0 )
					FeIIPunPop(pnunit.ipPnunit[i],FALSE);

			}
			else if( strcmp(pnunit.chPunch[i],"FE2P") == 0 )
			{
				/* punch all level populations for large feii atom */
				if( strcmp(chTime,"LAST") != 0 )
					FeIIPunPop(pnunit.ipPnunit[i],TRUE);

			}
			else if( strcmp(pnunit.chPunch[i],"GAMM") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
					PunGamma(pnunit.ipPnunit[i]);

			}
			else if( strcmp(pnunit.chPunch[i],"GAUN") == 0 )
			{
				/* punch gaunt factors */
				if( strcmp(chTime,"LAST") != 0 )
					pgaunt(pnunit.ipPnunit[i]);

			}
			else if( strcmp(pnunit.chPunch[i],"HEAT") == 0 )
			{
				/* punch heating */
				if( strcmp(chTime,"LAST") != 0 )
					PunHeat(pnunit.ipPnunit[i]);

			}

			/*  punch helium */
			/* informatin ou the helium singlets */
			else if( strcmp(pnunit.chPunch[i],"HEL1") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					if( he1rate.smhe2ov1 > 1e-30 )
					{
						/* this is simple estimate of singlet to total he */
						fref = (xIonFracs[2][ipHELIUM]/he1rate.smhe2ov1)/xIonFracs[0][ipHELIUM];
					}
					else
					{
						fref = 0.;
					}

					/* second number is ratio of predicted HeII/AheII to simple
					 *  clhe2ov1 is computed He1/He1 */
					fprintf( pnunit.ipPnunit[i], 
						"he1%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
					  xIonFracs[1][ipHELIUM]/xIonFracs[0][ipHELIUM], 
					  fref, he1rate.clhe2ov1, he1rate.smhe2ov1, 
					  herecCom.reci*phycon.eden, 
					  phe1lv.he1gam[0], 
					  he.hei3/xIonFracs[0][ipHELIUM] );

					/* print out photoionization rates */
					GammaPrt(nhe1Com.nhe1[0],rfield.nflux,opac.iophe1[0],
					  pnunit.ipPnunit[i],phe1lv.he1gam[0],phe1lv.he1gam[0]*
					  0.05);
				}
			}

			/* information about the helium triplets */
			else if( strcmp(pnunit.chPunch[i],"HEL3") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* this is simple estimate of triplet to total he */
					if( he3rate.she2ov3 > 0. )
					{
						fref = (xIonFracs[2][ipHELIUM]/he3rate.she2ov3)/xIonFracs[0][ipHELIUM];
					}
					else
					{
						fref = 0.;
					}
					fprintf( pnunit.ipPnunit[i], "HeTrip%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
					  he.hei3/xIonFracs[0][ipHELIUM], fref, herecCom.rectri*
					  phycon.eden, he3gams.he3gam[0], he3rate.collhe3 );
					/* print out photoionization rates */
					GammaPrt(he.nhei3,nhe1Com.nhe1[0],opac.ioptri,
					  pnunit.ipPnunit[i] , he3gams.he3gam[0] , he3gams.he3gam[0]*0.05);
				}
			}

			/* punch hummer, results needed for Lya transport, to feed into David's routine */
			else if( strcmp(pnunit.chPunch[i],"HUMM") == 0 )
			{
				damp = HydroLines[ipHYDROGEN][ipH2p][ipH1s].damp;
				eps = HydroLines[ipHYDROGEN][ipH2p][ipH1s].Aul/
				  (HydroLines[ipHYDROGEN][ipH2p][ipH1s].ColUL *phycon.eden);
				fprintf( pnunit.ipPnunit[i], 
					" %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n", 
				  radius.depth, 
				  HydroLines[0][ipH2p][ipH1s].TauIn, 
				  hydro.hn[0][ipH1s]*h.hii, 
				  hydro.hn[0][ipH2p]*h.hii, 
				  phycon.te, damp, eps );
			}

			else if( strcmp(pnunit.chPunch[i],"HYDc") == 0 )
			{
				/* punch hydrogen physical conditions */
				fprintf( pnunit.ipPnunit[i], 
					" %.3e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n", 
				  radius.depth, phycon.te, phycon.hden, phycon.eden, 
				  h.hi/phycon.hden, h.hii/phycon.hden, hmi.htwo/phycon.hden, 
				  hmi.h2plus/phycon.hden, hmi.h3plus/phycon.hden, 
				  hmi.hminus/phycon.hden );
			}

			else if( strcmp(pnunit.chPunch[i],"HYDi") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* punch hydrogen ionization
					 * this will be total decays to ground */
					RateInter = 0.;
					stage = hydro.hcolnc[0][0]*phycon.eden*hydro.hn[0][ipH1s];
					fref = HPhoto.hgamnc[0][ipH1s]*hydro.hn[0][ipH1s];
					fout = hydro.hn[0][ipH1s];
					for( ion=ipH2s; ion <= nHLevels[ipHYDROGEN]; ion++ )
					{
						/* this is total decays to ground */
						RateInter += 
							HydroLines[ipHYDROGEN][ion][ipH1s].Aul*
						  (HydroLines[ipHYDROGEN][ion][ipH1s].Pesc + 
						  HydroLines[ipHYDROGEN][ion][ipH1s].Pdest);
						/* total photo from all levels */
						fref += HPhoto.hgamnc[0][ion]*hydro.hn[0][ion];
						/* total col ion from all levels */
						stage += hydro.hcolnc[0][ion]*phycon.eden*
						  hydro.hn[0][ion];
						fout += hydro.hn[0][ion];
					}
					fprintf( pnunit.ipPnunit[i], "hion\t%4ld\t%10.2e\t%10.2e", 
					  nzone, 
					  HPhoto.hgamnc[0][ipH1s], 
					  hcaseb.HRecEffec[0] );
					for(j=0; j < LIMELM; j++)
					{
						fprintf( pnunit.ipPnunit[i], "\t%10.2e", 
						  hcaseb.HRecRad[j] );
					}

					fprintf( pnunit.ipPnunit[i], 
						"\t%10.2e\t%10.2e\t%10.2e\t%10.2e\t%10.2e\t%10.2e\t%10.2e\t%10.2e\n", 
					  h.hii/h.hi, 
					  HPhoto.hgamnc[0][ipH1s]/(phycon.eden*hcaseb.HRecEffec[0]), 
					  hydro.hrec[0][1][ipRecEsc], 
					  RateInter, 
					  fref/MAX2(1e-37,fout), 
					  stage/MAX2(1e-37,fout), 
					  HPhoto.hgamnc[0][ipH1s]*h.hi/(phycon.eden* h.hii) ,
					  Secondaries.csupra);

					GammaPrt(nh.ipHn[0][0],rfield.nflux,opac.ipHOpac[0][ipH1s],
					  pnunit.ipPnunit[i],HPhoto.hgamnc[0][ipH1s],HPhoto.hgamnc[0][ipH1s]*
					  0.05);
				}
			}

			else if( strcmp(pnunit.chPunch[i],"HYDp") == 0 )
			{
				/* punch hydrogen populations */
				fprintf( pnunit.ipPnunit[i], " %10.2e%10.2e%10.2e", 
				  radius.depth, h.hi, h.hii );
				for( j=ipH1s; j <= 6; j++ )
				{
					fprintf( pnunit.ipPnunit[i], "%10.2e", hydro.hn[0][j] );
				}
				fprintf( pnunit.ipPnunit[i], "%10.2e%10.2e\n", 
				  hydro.hn[0][ipH2s], hydro.hn[0][ipH2p] );
			}

			else if( strcmp(pnunit.chPunch[i],"IONI") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch mean ionization distribution */
					pmeani( 'i', FALSE , pnunit.ipPnunit[i] );
				}
			}

			else if( strcmp(pnunit.chPunch[i]," IP ") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch valence shell ip's */
					for( nelem=0; nelem < LIMELM; nelem++ )
					{
						for( n=0; n <= 15; n += 15 )
						{
							if( n + 1 <= nelem+1 )
							{
								fprintf( pnunit.ipPnunit[i], 
								  " \n" );
								limit = MIN2(n+15,nelem+1);
								fprintf( pnunit.ipPnunit[i], 
								  "%2.2s", elementnames.chElementSym[nelem]);
								for( ion=n + 1; ion <= limit; ion++ )
								{
									fprintf( pnunit.ipPnunit[i], 
									  "%4ld", ion );
								}
								fprintf( pnunit.ipPnunit[i], 
								  "\n" );
							}

							if( n + 1 <= nelem+1 )
							{
								for( ips=0; ips < nsShellsCom.nsShells[n][MIN2(n+15,nelem+1)-1]; ips++ )
								{
									limit = MIN2(n+15,nelem+1);
									for( j=n; j < limit; j++ )
									{
										sum = PH1COM.PH1[ips][nelem+1-j][nelem][0];
										if( sum < 2. )
										{
											strcpy( chEner[j], "      " );
										}
										else if( sum < 10. )
										{
											sprintf( chEner[j], "%6.3f", sum );
										}
										else if( sum < 100. )
										{
											sprintf( chEner[j], "%6.2f", sum );
										}
										else if( sum < 1000. )
										{
											sprintf( chEner[j], "%6.1f", sum );
										}
										else
										{
											sprintf( chEner[j], "%6ld",  (long)(sum) );
										}
									}

									limit = MIN2(n+15,nelem+1);
									fprintf( pnunit.ipPnunit[i], 
									  "%2.2s", Shell.chShell[ips]
									   );

									for( j=n; j < limit; j++ )
									{
										fprintf( pnunit.ipPnunit[i], 
										  "%6.6s", chEner[j] );
									}

									fprintf( pnunit.ipPnunit[i], 
									  "\n" );
								}
							}
						}
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LINC") == 0 )
			{
				/* punch line cumulative */
				/* lgOK not used, placdholder */
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* not used for anything here, but should not pass baloney*/
					lgOK = TRUE;
					punlin(pnunit.ipPnunit[i],"PUNC",lgOK); 
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LINL") == 0 )
			{
				/* punch line labels, then stop */
				pllabels(pnunit.ipPnunit[i]);
				puts( "[Stop in DoPunch]" );
				fprintf( ioQQQ, " Cloudy ends:\n" );
				cdEXIT(1);
			}

			else if( strcmp(pnunit.chPunch[i],"LINO") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch line optical depths */
					PunchLineStuff(pnunit.ipPnunit[i],"optical" , pnunit.punarg[0][i]);
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LINP") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					static int lgFirst=TRUE;
					/* punch line populations, need to do this twice if very first
					 * call since first call to PunchLineStuff generates atomic parameters
					 * rather than level pops */
					PunchLineStuff(pnunit.ipPnunit[i],"populat" , pnunit.punarg[0][i]);
					if( lgFirst )
					{
						lgFirst = FALSE;
						PunchLineStuff(pnunit.ipPnunit[i],"populat" , pnunit.punarg[0][i]);
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LIND") == 0 )
			{
				/* punch line data, then stop */
				DoPunchLineData(pnunit.ipPnunit[i]);
				puts( "[Normal stop in DoPunch]" );
				fprintf( ioQQQ, " Cloudy ends:\n" );
				cdEXIT(1);
			}

			else if( strcmp(pnunit.chPunch[i],"LINS") == 0 )
			{
				/* punch line structure */
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* not acutally used, but should not pass baloney*/
					lgOK = TRUE;
					punlin(pnunit.ipPnunit[i],"PUNS",lgOK);
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LINA") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					/* punch out all lines with energies */
					for( j=0; j < LineSave.nsum; j++ )
					{
						/* xLineEnergy is sentinel saying to add to continuum only if positive */
						if( LineSv[j].xLineEnergy > 0. && 
							LineSv[j].sumlin > 0. )
						{
							fprintf( pnunit.ipPnunit[i], "%12.5e\t%8.3f\t%4.4s %li%c%c\t%c\n", 
							  AnuUnit(LineSv[j].xLineEnergy), 
							  log10(MAX2(SMALLFLOAT,LineSv[j].sumlin) ) + radius.pirsq, 
							  LineSv[j].chALab ,
							  LineSv[j].lin,
							  LineSv[j].chUnits,
							  LineSv[j].chShift,
							  LineSv[j].chSumTyp);
						}
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"LINI") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 && 
					(nzone/levery.LinEvery)*levery.LinEvery != nzone )
				{
					/* this is the last zone
					 * punch line intensities - but do not do last zone twice */
					plinin(pnunit.ipPnunit[i]);
				}
				else if( strcmp(chTime,"LAST") != 0 )
				{
					/* following so we only punch first zone if LinEvery reset */
					if( (levery.lgLinEvery && nzone == 1) || 
					  (nzone/levery.LinEvery)*levery.LinEvery == 
					  nzone )
					{
						/* this is middle of calculation
						 * punch line intensities */
						plinin(pnunit.ipPnunit[i]);
					}
				}
			}

			/* punch Lya - some details about Lya */
			else if( strcmp(pnunit.chPunch[i],"LYMA") == 0 )
			{
				pop = (hydro.hn[0][ipH2p]/6.)/(hydro.hn[0][ipH1s]/2.);
				if( pop > 0. )
				{
					texc = -1.183e5/log(pop);
				}
				else
				{
					texc = 0.;
				}
				fprintf( pnunit.ipPnunit[i], 
					"%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n", 
					radius.depth,
				  HydroLines[0][ipH2p][ipH1s].TauIn, 
				  pop, 
				  texc, 
				  phycon.te, 
				  texc/phycon.te ,
				  HydroLines[0][ipH2p][ipH1s].Pdest, 
				  opac.opac[HydroLines[0][ipH2p][ipH1s].ipCont-1],
				  opac.albedo[HydroLines[0][ipH2p][ipH1s].ipCont-1] );
			}

			else if( strcmp(pnunit.chPunch[i],"MAP ") == 0 )
			{
				/* do the map now if we are at the zone, or if this
				 * is the LAST call to this routine and map not done yet */
				if(  (MapPar.lgMapDone==FALSE ) &&
					(nzone == MapPar.MapZone  ||  strcmp(chTime,"LAST") == 0 ) )
				{
					lgTlkSav = called.lgTalk;
					called.lgTalk = TRUE;
					isav = ioQQQ;
					ioQQQ = pnunit.ipPnunit[i];
					punt(" map");
					ioQQQ = isav;
					called.lgTalk = lgTlkSav;
				}
			}

			else if( strcmp(pnunit.chPunch[i],"MOLE") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* molecules, especially for pdr */
					fprintf( pnunit.ipPnunit[i], 
						"%.3e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n", 
					  radius.depth, 
					  hmi.htwo/phycon.hden, 
					  hmi.htwo_star/phycon.hden, 
					  hevmolec.hevmol[IPCO-1]/
					  phycon.hden, 
					  hevmolec.hevmol[IPH2O-1]/phycon.hden, 
					  hevmolec.hevmol[IPOH-1]/phycon.hden, 
					  hevmolec.hevmol[IPCH-1]/phycon.hden, 
					  hevmolec.hevmol[IPOTWO-1]/phycon.hden );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"OPAC") == 0 )
			{
				/* punch opacity - routine will parse which type of opacity punch to do */
				if( strcmp(chTime,"LAST") == 0 )
					PunOpac(pnunit.ipPnunit[i],i);
			}

			/* punch optical depths command */
			else if( strcmp(pnunit.chPunch[i],"OPTI") == 0 )
			{
				if( strcmp(chTime,"LAST") == 0 )
				{
					for( j=0; j < rfield.nflux; j++ )
					{
						fprintf( pnunit.ipPnunit[i], 
							"%12.4e\t%.3e\t%12.4e\t%.3e\n", 
						  AnuUnit(rfield.AnuOrg[j]), 
						  opac.TauAbsFace[j]+opac.TauScatFace[j], 
						  opac.TauAbsFace[j], 
						  opac.TauScatFace[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i]," OTS") == 0 )
			{
				ConMax = 0.;
				xLinMax = 0.;
				opConSum = 0.;
				opLinSum = 0.;
				ipLinMax = 1;
				ipConMax = 1;

				for( j=0; j < rfield.nflux; j++ )
				{
					opConSum += rfield.otscon[j]*opac.opac[j];
					opLinSum += rfield.otslin[j]*opac.opac[j];
					if( rfield.otslin[j]*opac.opac[j] > xLinMax )
					{
						xLinMax = rfield.otslin[j]*opac.opac[j];
						ipLinMax = j+1;
					}
					if( rfield.otscon[j]*opac.opac[j] > ConMax )
					{
						ConMax = rfield.otscon[j]*opac.opac[j];
						ipConMax = j+1;
					}
				}
				fprintf( pnunit.ipPnunit[i], 
					"tot con lin=%10.2e%10.2e lin=%4.4s%11.4e%10.2e con=%4.4s%11.4e%10.2e\n", 
				  opConSum, opLinSum, rfield.chLineLabel[ipLinMax-1]
				  , rfield.anu[ipLinMax-1], xLinMax, rfield.chContLabel[ipConMax-1]
				  , rfield.anu[ipConMax-1], ConMax );
			}

			else if( strcmp(pnunit.chPunch[i],"OVER") == 0 )
			{
				/* overview of model results,
				 * depth, te, hden, eden, ion fracs H, He, c, O */
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* print the depth */
					fprintf( pnunit.ipPnunit[i], "%.3e\t", radius.depth );

					/* temperature, heating */
					fprintf( pnunit.ipPnunit[i], "%.3f\t%.3f", 
					  log10(phycon.te), log10(heat.htot) ); 

					/* hydrogen and electron densities */
					fprintf( pnunit.ipPnunit[i], "\t%.3f\t%.3f", 
					  log10(phycon.hden), log10(phycon.eden) );
					  
					/* ionization fractions of hydrogen */
					fprintf( pnunit.ipPnunit[i], "\t%.3f\t%.3f", 
					  -log10(MAX2(5e-10,h.hi/phycon.hden)), 
					  -log10(MAX2(5e-10,h.hii/phycon.hden)) );

					/* ionization fractions of helium */
					for( j=1; j <= 3; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "\t%.3f", 
						  -log10(MAX2(5e-10,xIonFracs[j][1]/
						  xIonFracs[0][ipHELIUM])) );
					}

					/* ionization fractions of carbon */
					for( j=1; j <= 4; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "\t%.3f", 
						  -log10(MAX2(5e-10,xIonFracs[j][5]/
						  xIonFracs[0][ipCARBON])) );
					}

					/* ionization fractions of carbon */
					for( j=1; j <= 6; j++ )
					{
						fprintf( pnunit.ipPnunit[i], "\t%6.3f", 
						  -log10(MAX2(5e-10,xIonFracs[j][7]/
						  xIonFracs[0][ipOXYGEN])) );
					}
					fprintf( pnunit.ipPnunit[i], "\n" );
				}
			}

			else if( strcmp(pnunit.chPunch[i]," PDR") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					av = opac.TauTotalGeo[0][ipvfil.ipVFilter-1]*1.08574;
					fprintf( pnunit.ipPnunit[i], 
						"%.3e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n", 
					  radius.depth, 
					  coldenCom.colden[IPCOLUMNDENSITY-1], 
					  av, 
					  phycon.te, 
					  h.hi/phycon.hden, 
					  hmi.htwo/phycon.hden, 
					  hmi.htwo_star/phycon.hden, 
					  xIonFracs[1][5]/xIonFracs[0][5], 
					  hevmolec.hevmol[IPCO-1]/xIonFracs[0][5], 
					  hevmolec.hevmol[IPH2O-1]/xIonFracs[0][7] );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"PHYS") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					/* punch physical conditions */
					fprintf( pnunit.ipPnunit[i], "%14.6e%12.4e%11.3e%11.3e%11.3e%11.3e\n", 
					  radius.depth, phycon.te, phycon.hden, 
					  phycon.eden, heat.htot, wind.AccelTot );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"PRES") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					fprintf( pnunit.ipPnunit[i], 
						" %10.2e%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e %10.2e %10.2e %c\n", 
					  radius.depth, pressure.pnow, pressure.PressureInit + 
					  pressure.PresInteg, pressure.PressureInit, pressure.GasPres, 
					  pressure.RamPres, pressure.RadPres, 
					  /* subtract continuum rad pres which has already been added on */
					  pressure.PresInteg - pressure.pinzon, 
					  wind.windv/1e5,
					  TorF(conv.lgConvPres) );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"RADI") == 0 )
			{
				if( strcmp(chTime,"LAST") != 0 )
				{
					fprintf( pnunit.ipPnunit[i], " %4ld%12.4e%12.4e%12.4e\n", 
					  nzone, radius.Radius, radius.depth, 
					  radius.drad );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"RESU") == 0 )
			{
				/*  punch results of the calculation */
				if( strcmp(chTime,"LAST") == 0 )
					result(pnunit.ipPnunit[i]);
			}

			else if( strcmp(pnunit.chPunch[i],"SOUS") == 0 )
			{
				/* full spectrum of continuum source function at 1 depth
				 *  command was "punch source spectrum" */
				if( strcmp(chTime,"LAST") != 0 )
				{
					limit = MIN2(maxcor.ipMaxBolt,rfield.nflux);
					for( j=0; j < limit; j++ )
					{
						fprintf( pnunit.ipPnunit[i], 
							"%12.4e%12.4e%12.4e%12.4e%12.4e\n", 
						  rfield.anu[j], 
						  (rfield.ConLocInter[j]+rfield.ConLocNoInter[j])/rfield.widflx[j], 
						  opac.opac[j], 
						  (rfield.ConLocInter[j]+rfield.ConLocNoInter[j])/rfield.widflx[j]/MAX2(1e-35,opac.opac[j]), 
						  (rfield.ConLocInter[j]+rfield.ConLocNoInter[j])/rfield.widflx[j]/MAX2(1e-35,opac.opac[j])/plankf(j) );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"SOUD") == 0 )
			{
				/* parts of continuum source function vs depth
				 * command was source depth */
				j = nh.ipHn[0][ipH1s] + 2;
				fprintf( pnunit.ipPnunit[i], 
					"%12.4e%12.4e%12.4e%12.4e\n", 
				  opac.TauAbsFace[j-1], 
				  (rfield.ConLocInter[j-1]+rfield.ConLocNoInter[j-1])/rfield.widflx[j-1]/MAX2(1e-35,opac.opac[j-1]), 
				  rfield.otscon[nh.ipHn[0][ipH1s]-1], 
				  rfield.otscon[nh.ipHn[0][0]-1]/opac.opac[nh.ipHn[0][ipH1s]-1] );
			}

			/* this is punch special option */
			else if( strcmp(pnunit.chPunch[i],"SPEC") == 0 )
			{
				PunSpec(pnunit.ipPnunit[i],chTime);
			}

			else if( strcmp(pnunit.chPunch[i],"TEGR") == 0 )
			{
				/* punch history of heating and cooling */
				if( strcmp(chTime,"LAST") != 0 )
				{
					fprintf( pnunit.ipPnunit[i], " " );
					for( j=0; j < NGRID; j++ )
					{
						fprintf( pnunit.ipPnunit[i], 
							"%4ld%11.3e%11.3e%11.3e\n", 
						  HeatCool.nZonGrid[j], 
						  HeatCool.TeGrid[j], 
						  HeatCool.HtGrid[j], 
						  HeatCool.ClGrid[j] );
					}
				}
			}

			else if( strcmp(pnunit.chPunch[i],"TEMP") == 0 )
			{
				/* temperature and its derivative */
				fprintf( pnunit.ipPnunit[i], " %4ld%12.4e%12.4e\n", 
				  nzone, phycon.te, (phycon.te - phycon.tlast)/
				  radius.drad );
			}

			else if( strcmp(pnunit.chPunch[i],"TPRE") == 0 )
			{
				/* temperature and its predictors, turned on with punch tprid */
				fprintf( pnunit.ipPnunit[i], "%5ld %11.4e %11.4e %11.4e %g\n", 
				  nzone, phycon.TeInit, phycon.TeProp, phycon.te, 
				  (phycon.TeProp- phycon.te)/phycon.te );
			}

			else if( strcmp(pnunit.chPunch[i],"VERN") == 0 )
			{
				if( (strcmp(chTime,"LAST") == 0) && FeII.lgFeIION)
				{
					/*FeIILines("pun",norm.ScaleNormLine/LineSv[norm.ipNormWavL].sumlin);*/
					/*FeIIPunchLines(FeII.ioVerner);*/
					FeIIPunchLines( pnunit.ipPnunit[i] );
				}
			}

			else if( strcmp(pnunit.chPunch[i],"WIND") == 0 )
			{
				/* wind velocity, radiative accel, and ratio total to e- accel */
				fprintf( pnunit.ipPnunit[i], "%12.4e%12.4e%12.4e%12.4e%12.4e\n", 
				  radius.Radius, radius.depth, wind.windv, wind.AccelTot, 
				  wind.fmul );
			}

			else
			{
				/* this is insanity, internal flag set in getpunch not seen here */
				fprintf( ioQQQ, " DoPunch does not recognize flag %4.4s set by GETPUNCH.  This is impossible.\n", 
				  pnunit.chPunch[i] );
				ShowMe();
				puts( "[Stop in DoPunch]" );
				cdEXIT(1);
			}
			/* at start of loop, broke to here if not last iteration,
			 * and punch last was set
			 * throw special line if punching every iteration */
			if( strcmp(chTime,"LAST") == 0 && !IterCnt.lgLastIt )
			{
				fprintf( pnunit.ipPnunit[i], " ###########################\n" );
			}
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->DoPunch()\n", debug_fp );
#	endif
	return;
}

/*plinin produce the 'punch lines intensity' output */
static void plinin(FILE * io)
{
	char chCard[81];
	int lgEOF;
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>plinin()\n", debug_fp );
#	endif

	/* used to punch out all the emission line intensities
	 * first initialize the line image reader */

	rdinit();

	fprintf( io, "**********************************************************************************************************************************\n" );
	
	lgEOF = FALSE;
	while( !lgEOF )
	{
		readar(chCard,&lgEOF);
		if( !lgEOF )
		{
				fprintf( io, "%s\n", chCard );
		}
	}

	/* first print any cautions or warnings */
	cdWarnings( io);
	cdCautions( io);
	fprintf( io, "zone=%5ld\n", nzone );
	fprintf( io, "**********************************************************************************************************************************\n" );
	fprintf( io, "begin emission lines\n" );

	/* >>chng 96 jul 03, only punch zero intensities */
	PunResult(io,"    ",0,0.,'o',"Start");
	for( i=0; i < LineSave.nsum; i++ )
	{
		if( LineSv[i].sumlin > 0. )
		{
			PunResult(io,(char*)LineSv[i].chALab,LineSv[i].lin,
			  LineSv[i].sumlin, LineSv[i].chSumTyp, "Line ");
		}
	}

	PunResult(io,"    ",0,0.,'o',"Flush");

	fprintf( io, "     \n" );
	fprintf( io, "**********************************************************************************************************************************\n" );


#	ifdef DEBUG_FUN
	fputs( " <->plinin()\n", debug_fp );
#	endif
	return;
}

static int lgPopsFirst , lgOptical ;
/* punch optical depths or source functions for all transferred lines */
static void PunchLineStuff(FILE * io,char *chJob , float xLimit )
{

	long int ipZ , ipLo , ipHi , i;
	long index = 0;
	static int lgFirst=TRUE;

#	ifdef DEBUG_FUN
	fputs( "<+>PunchLineStuff()\n", debug_fp );
#	endif

	/*find out which job this is and set a flag to use later */
	if( strcmp( &*chJob , "optical" ) == 0 )
	{
		/* punch line optical depths */
		lgOptical = TRUE;
		lgPopsFirst = FALSE;
	}
	else if( strcmp( &*chJob , "populat" ) == 0 )
	{
		lgOptical = FALSE;
		/* level population information */
		if( lgFirst )
		{
			lgPopsFirst = TRUE;
			fprintf(io,"index\tAn.ion\tgLo\tgUp\tE(wn)\tgf\n");
			lgFirst = FALSE;
		}
		else
		{
			lgPopsFirst = FALSE;
		}
	}
	else
	{
		fprintf( ioQQQ, " insane job in PunchLineStuff =%s\n", 
		  &*chJob );
		puts( "[Stop in PunchLineStuff]" );
		cdEXIT(1);exit(1);
	}

	/* loop over all lines, calling put1Line to create info (routine located below) */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ]  )
		{
			for( ipHi=ipH2s; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				for( ipLo=ipH1s; ipLo <= (ipHi - 1); ipLo++ )
				{
					++index;
					pun1Line( &HydroLines[ipZ][ipHi][ipLo] , io , xLimit  , index);
				}
			}
		}
	}

	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		if( ipZ < 2 || abundances.lgElmtOn[ipZ] )
		{
			/* arrays are dim'd ipHe2p1P */
			for( ipLo=ipHe1s1S; ipLo < nHeLevels[ipZ]-1; ipLo++ )
			{
				/* set ENTIRE array to impossible values, in case of bad pointer */
				for( ipHi=ipLo+1; ipHi < nHeLevels[ipZ]; ipHi++ )
				{
					++index;
					pun1Line( &HeLines[ipZ][ipHi][ipLo] , io, xLimit  , index);
				}
			}
		}
	}

	/* index from 1 to skip over dummy line */
	for( i=1; i < nLevel1; i++ )
	{
		++index;
		pun1Line( &TauLines[i] , io , xLimit  , index);
	}

	for( i=0; i < nWindLine; i++ )
	{
		if( TauLine2[i].nelem != TauLine2[i].IonStg )
		{
			++index;
			pun1Line( &TauLine2[i] , io , xLimit  , index);
		}
	}

	/* C12O16 */
	for( i=0; i < nCORotate; i++ )
	{
		++index;
		pun1Line( &C12O16Rotate[i] , io , xLimit  , index);
	}

	/* C13O16 */
	for( i=0; i < nCORotate; i++ )
	{
		++index;
		pun1Line( &C13O16Rotate[i] , io , xLimit  , index);
	}
	fprintf(io, "##################################\n"); 

#	ifdef DEBUG_FUN
	fputs( " <->PunchLineStuff()\n", debug_fp );
#	endif
	return;
}

static void pun1Line( EmLine * t , FILE * io , float xLimit  , long index )
{

	if( lgOptical )
	{
		/* optical depths, no special first time, only print them */
		if( t->TauIn >= xLimit )
		{
			fprintf( io , "%2.2s%2.2s\t%.5e\t%.3f\n", 
			  elementnames.chElementSym[t->nelem-1] ,
			  elementnames.chIonStage[t->IonStg-1]  ,
			  AnuUnit(t->EnergyRyd), 
			  log10(MAX2(SMALLFLOAT,t->TauIn))
			  );
		}
	}
	else if( lgPopsFirst )
	{
		char chLbl[11];
		/* first call to line populations, print atomic parameters and indices */
		strcpy( chLbl, chLineLbl(t) );
		fprintf(io, "%li\t%s\t" , index , chLbl );
		/* stat weights */
		fprintf(io, "%.0f\t%.0f\t", 
			t->gLo ,t->gHi);
		/* energy difference, gf */
		fprintf(io, "%.2f\t%.3e\t", 
			t->EnergyWN ,t->gf);
		fprintf(io, "\n");
	}
	else
	{
		/* not first call, so do level populations and indices defined above */
		if( t->PopHi > xLimit )
		{
			fprintf(io,"%li\t%.2e\t%.2e\n", index,
				t->PopLo , 
				t->PopHi );
		}
	}
}
