/*He-like main routine to call HeLevel and determine model he-like species atom level balance */
void HeLike(void);

/* create he-like series */
void HeCreate(void);

void testhelike(void);

/* Function assign assigns, in order of increasing energy, 	*/
/* quantum numbers n,l, and s to each element of vector he_levels.	
 * nmax is the upper quantum number, there will be many l levels within it.
 * ndim is dimension of vector, used to confirm that size not exceeded */ 
void he_assign( long nmax , long ndim );

/*	A structure is defined here, for use in 	*/
/*	multiple subroutines, which contains the 	*/
/*	quantum numbers n,l, and s for a given energy level.	*/ 

typedef struct {
	long n;
	long s;
	long l;
} Elevels;/* Main_Config.h */ 

EXTERN Elevels *he_levels;	/* A vector of structures each containing n,l, and s */
 
/* Passed to function loop is the index of a lower level.	*/
/* Since the levels are indexed in increasing energy order,	*/
/* the upper level of a transition must have a higher index.	*/
void he_1trans( long ipZ , Elevels he_upper , Elevels he_lower , double *Enerwn , double *Aul );

/* 
 *  Inclusive -- i.e. n = 1,2,...,MAX_N 
 *  MAX_N -- Quantum Number n 
 * 
 *  maximum principle quantum number considered in calculations (inclusive) 
 * (long int) 
*/ 
#define      MAX_N           2L 
 
/* Effective charge of H+ ion */ 
/* (long int) */ 
#define      Z               1L 
 
/* Effective charge of He+ ion */ 
/* (long int) */ 
#define      Z_HELIUM        2L 
 
/* Effective temperature of hydrogen and helium */ 
/* (double) */ 
#define      TEMPERATURE     10000.0 
 
/* ndim -- Array Dimension 
 * 
 *  Sum of levels (n,l) for 1 <= n <= MAX_N is given by 
 * 
 *         MAX_N 
 *         --- 
 * NDIM =  >  n' = ( (MAX_N)^2 + MAX_N ) * 1/2 
 *         --- 
 *         n'=1 
 * 
 * Now subtract one (1) since Array's are (indexed) like 
 * 0,1,2,3,..., NDIM-1 
 * DANGER WILL ROBINSON -- Divide (even) integer by 2 below 
 * 
 *  number of states (n,l) between n=1 and n=MAX_N (inclusive) 
 */ 
#define NDIM            ((((MAX_N)*(MAX_N))+MAX_N)/2) 
#define NUMLEVELS       ((((MAX_N)*(MAX_N))+MAX_N)/2) 
 
/* a small number */ 
#define SMALLNUM        +1.0e-8 
 
/* whether to use case A,B, or C */ 
#define CASE_ABC         'B'                           /* 'A'  ,   'B'  or 'C' 
*/ 
 
/* the number density of free electrons (cm^3)-1 */ 
#define NELECTRON       1.0                           /* (double) */ 
 
/* the number density of free Hydrogen+ Ions (protons) (cm^3)-1 */ 
#define NPROTON         1.0                           /* (double) */ 
 
/* I can't remember what this was */ 
#define SDIM            ((MAX_N)*(MAX_N)) 
 
/* 
    if non-zero use Brocklehurst for recombination co-efficients 
*/ 
#define BROCKLEHURST            1 
 
#define TEMPERATURE_HELIUM      TEMPERATURE 
#define MAX_N_HELIUM            MAX_N 
#define NUMLEVELS_HELIUM        NUMLEVELS 
#define HOANG_BINH 
 
#define XACC                    1e-16 
 double correct(
		Elevels Li,
		Elevels Lf,
		long Z1);

double he_energy(
		double nistar, 
		double nfstar,
		long Z1,
		long s,
		long li
		);

		double forbidden(
		Elevels Li,
		long Z1
		);

/* declar of func, the last arg, changed from double(*)() to above, 
 * seemed to fix flags that were raised */ 
 double ritoa(
		long int li,
		long int lf,
		long int Z1,
		double k,
		double RI 
		);

#if 0
 /* compute energy diffference in wn and Aul for given line
  * return is 0 for success, 1 for failure */
void he_1trans(  
			  /* charge on the C scale, 1 is helium */
			  long ipZ , 
			  /* structure giving n, l, s of upper level */
			  Elevels he_upper , 
			  /* structure giving n, l, s of lower level */
			  Elevels he_lower , 
			  /* energy difference in wavenumber */
			  double *Enerwn , 
			  /* transition probability */
			  double *Aul );
#endif
/* 
   scqdri.h 
*/ 
 

/* all of these are initialized in zero */
EXTERN struct t_helike 
{

	/* radiative recombination */
	float ***herec /*hrec[LIMELM][LMHLVL+1][3]*/;

	/* line boltzmann factors */
	double ***helbolt /*hrec[LIMELM][LMHLVL+1][3]*/;

	/* collisional ionization */
	float **hecolnc/*hcolnc[LIMELM][LMHLVL+1]*/;

	/* photo ionization */
	float **hegamnc/*hcolnc[LIMELM][LMHLVL+1]*/;

	/* ionization fractions */
	float HeIonFrac[LIMELM];

	/* ionization fractions */
	float HeIonSimple[LIMELM];

	/* ionization potentials */
	double HeLevNIonRyd[LIMELM];

}	helike;
