/*HydroLevelPop solve for ionization balance level populations of model hydrogen atom */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "hcolrt.h"
#include "secondaries.h"
#include "hphoto.h"
#include "chargtran.h"
#include "trace.h"
#include "phycon.h"
#include "hcaseb.h"
#include "typmatrx.h"
#include "hratedest.h"
#include "linpack.h"
#include "matin1.h"
#include "veclib.h"
#include "hydrogenic.h"

void HydroLevelPop(long int ipZ ,
	double **SaveZ/*[nHLevels[ipZ]+2][nHLevels+2]*/, 
	double *bvec/*[nHLevels[ipZ]+2]*/, 
	double *error/*[nHLevels[ipZ]+2]*/, 
	double *work/*[nHLevels[ipZ]+2]*/, 
	double **z/*[nHLevels[ipZ]+2][nHLevels+2]*/ ,
	long int *ipiv , /* malloc out to [nHLevels[ipZ]+1] */
	double *totcap/* malloc out to [nHLevels[ipZ]+1]*/)
{
	long int n, 
	  ipHi, 
	  ipLo, 
	  j, 
	  job, 
	  level, 
	  nerror1,
	  nerror2;

#	ifdef DEBUG_FUN
	fputs( "<+>HydroLevelPop()\n", debug_fp );
#	endif

	/* check that we were called with valid charge */
	assert( ipZ >= 0);
	assert( ipZ < LIMELM );

	/* 
	 * following electron density has approximate correction for neutrals
	 * corr of hi*1.7e-4 accounts for col ion by HI; Drawin Zs Phys 225, 483.
	 * used EdenHCorr instead
	 * edhi = eden + hi * 1.7e-4
	 */
#if 0
	hcaseb.HRecEffec[ipZ] = 0.;
	for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
	{
		z[nHLevels[ipZ]+1][n] = ((double)(
			hydro.hrec[ipZ][n][ipRecRad]*hydro.hrec[ipZ][n][ipRecNetEsc]) + 
		  HPhoto.rinduc[ipZ][n])/
		  hcolrt.hlte[ipZ][n] + 
		  hydro.hcolnc[ipZ][n]* phycon.EdenHCorr;

		/* hlte(n,ipZ) is only LTE pop when mult by Ne Nh */
		totcap[n] = z[nHLevels[ipZ]+1][n]*hcolrt.hlte[ipZ][n];
		hcaseb.HRecEffec[ipZ] += (float)totcap[n];
	}
#	endif

	hcaseb.HRecEffec[ipZ] = 0.;
	for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
	{
		z[nHLevels[ipZ]+1][n] = ((double)(
			hydro.hrec[ipZ][n][ipRecRad]*hydro.hrec[ipZ][n][ipRecNetEsc]) + 
		  HPhoto.rinduc[ipZ][n]) *phycon.eden +
		  hydro.hcolnc[ipZ][n]*
		  phycon.EdenHCorr*hcolrt.hlte[ipZ][n]*phycon.eden;

		/* hlte(n,ipZ) is only LTE pop when mult by Ne Nh */
		totcap[n] = z[nHLevels[ipZ]+1][n]/phycon.eden;
		hcaseb.HRecEffec[ipZ] += (float)totcap[n];
	}

	/* add on charge transfer recombination into ground */
	z[nHLevels[ipZ]+1][ipH1s] += ChargTran.CTHrec[MIN2(ipZ+1,2)-1][1];

	totcap[ipH1s] = z[nHLevels[ipZ]+1][ipH1s]/phycon.eden;

	/* master balance equation */
	for( level=ipH1s; level <= nHLevels[ipZ]; level++ )
	{
		/* 
		 * all process depopulating level
		 */
		z[level][level] = HPhoto.hgamnc[ipZ][level] + 
		  hydro.hcolnc[ipZ][level]* phycon.EdenHCorr + 
		  Secondaries.csupra;

		/* all processes populating level from below */
		for( ipLo=ipH1s; ipLo <= (level - 1); ipLo++ )
		{
			z[ipLo][level] = 
				-HydroLines[ipZ][level][ipLo].ColUL* phycon.EdenHCorr * 
			  HydroLines[ipZ][level][ipLo].gHi/HydroLines[ipZ][level][ipLo].gLo*
			  hydro.hlbolt[ipZ][ipLo][level]  -
			  HydroLines[ipZ][level][ipLo].pump ;

			/* pumping out of here to lower level */
			fixit();/* for 1s-2s two photon induced processes, pump contains the
			 * 1s - 2s pump rate, while Aul has the 2s-1s induced rate added in,
			 * and so is the total.  This should have a branch testing for 1s-2s
			 * and included the reverse pump in that case.  should not matter since
			 * induced 2s-1s should be slow compated to spontaneous A */
			z[level][level] += HydroLines[ipZ][level][ipLo].pump *
			  HydroLines[ipZ][level][ipLo].gLo/HydroLines[ipZ][level][ipLo].gHi;

			/* collisions out of here to lower level */
			z[level][level] += 
				HydroLines[ipZ][level][ipLo].ColUL * phycon.EdenHCorr;

			/* radiative decays out of here to lower level */
			z[level][level] += 
				HydroLines[ipZ][level][ipLo].Aul*
			  (HydroLines[ipZ][level][ipLo].Pesc + 
			  HydroLines[ipZ][level][ipLo].Pdest);
		}

		/* all processes populating level from above */
		for( ipHi=level + 1; ipHi <= nHLevels[ipZ]; ipHi++ )
		{
			double RadDecay;
			RadDecay =
				HydroLines[ipZ][ipHi][level].Aul*
			  (HydroLines[ipZ][ipHi][level].Pesc + 
			  HydroLines[ipZ][ipHi][level].Pdest);

			z[ipHi][level] = 
				-(RadDecay + 
			  HydroLines[ipZ][ipHi][level].pump*
			  HydroLines[ipZ][ipHi][level].gLo/HydroLines[ipZ][ipHi][level].gHi + 
			  HydroLines[ipZ][ipHi][level].ColUL *phycon.EdenHCorr);

			/* pumping out of here to upper level */
			z[level][level] += HydroLines[ipZ][ipHi][level].pump ;

			/* collisions out of here to upper level */
			z[level][level] += HydroLines[ipZ][ipHi][level].ColUL *
				HydroLines[ipZ][ipHi][level].gHi / HydroLines[ipZ][ipHi][level].gLo *
				hydro.hlbolt[ipZ][level][ipHi]*phycon.EdenHCorr;
		}
	}

	if( Secondaries.Hx12[0][1] > 0. )
	{
		/* now add on supra thermal excitation */
		for( level=ipH2s; level <= nHLevels[ipZ]; level++ )
		{
#			if 0
			/* stuff in min after Hx12 evaluates to 0 for atom, or 1 for atom */
			z[ipH1s][ipH1s] += Secondaries.Hx12[MIN2(ipZ,1)][level];

			z[level][ipH1s] -= 
				(Secondaries.Hx12[MIN2(ipZ,1)][level]*
				HydroLines[ipZ][level][ipH1s].gLo/ HydroLines[ipZ][level][ipH1s].gHi);

			z[ipH1s][level] -= 
				Secondaries.Hx12[MIN2(ipZ,1)][level];

			z[level][level] += Secondaries.Hx12[MIN2(ipZ,1)][level]*
				HydroLines[ipZ][level][ipH1s].gLo/ HydroLines[ipZ][level][ipH1s].gHi;
#			endif
			double RateUp , RateDown;

			RateUp = Secondaries.Hx12[MIN2(ipZ,1)][level];
			RateDown = RateUp * (double)HydroLines[ipZ][level][ipH1s].gLo /
				(double)HydroLines[ipZ][level][ipH1s].gHi;

			/* stuff in min after Hx12 evaluates to 0 for atom, or 1 for ion */
			/* total rate out of lower level */
			z[ipH1s][ipH1s] += RateUp;

			/* rate from the upper level to ground */
			z[level][ipH1s] -= RateDown ;

			/* rate from ground to upper level */
			z[ipH1s][level] -= RateUp ;

			z[level][level] += RateDown;  
		}
	}

	/* charge transfer ionization */
	z[ipH1s][ipH1s] += ChargTran.CTHion[MIN2(ipZ,1)][1];

	/* remember ground state destruction rate */
	if( ipZ == 0 )
	{
		HRateDest.HRateDestGnd[0] = (float)z[ipH1s][ipH1s];
		HRateDest.HRateDestGnd[1] = (float)(HPhoto.hgamnc[ipZ][ipH1s]/
		  z[ipH1s][ipH1s]);
		HRateDest.HRateDestGnd[2] = (float)(hydro.hcolnc[ipZ][ipH1s]*
		  phycon.EdenHCorr/z[ipH1s][ipH1s]);
		HRateDest.HRateDestGnd[4] = (float)(Secondaries.csupra/z[ipH1s][ipH1s]);
		HRateDest.HRateDestGnd[5] = (float)(Secondaries.Hx12[MIN2(ipZ,1)][ipH2p]*
		  9./z[ipH1s][ipH1s]);
		HRateDest.HRateDestGnd[7] = (float)(ChargTran.CTHion[MIN2(ipZ,1)][1]/
		  z[ipH1s][ipH1s]);
	}

	if( (trace.lgTrace && trace.lgHBugFull) && (ipZ == trace.ipZTrace) )
	{
		if( ipZ == 0 )
		{
			fprintf(ioQQQ," grnd dest fracs:");
			for( j=ipH1s; j < 7; j++ )
			{
				fprintf( ioQQQ," ");
				fprintf( ioQQQ,PrintEfmt("%8.1e", HRateDest.HRateDestGnd[j] ));
			}
			fprintf(ioQQQ," hcoldc %e\n", phycon.EdenHCorr );
			fprintf( ioQQQ, "\n" );
		}

		fprintf( ioQQQ, "  pop level     others => (HydroLevelPop)\n" );
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			fprintf( ioQQQ, "       HII%2ld", n );
			for( j=ipH1s; j <= (nHLevels[ipZ] + 1); j++ )
			{
				fprintf( ioQQQ," ");
				/*fprintf( ioQQQ,PrintEfmt("%8.1e", z[j][n] ) );*/
				fprintf( ioQQQ,"%.9e\t", z[j][n] );
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* save matrix */
	for( j=ipH1s; j <= nHLevels[ipZ]; j++ )
	{
		for( n=ipH1s; n <= (nHLevels[ipZ] + 1); n++ )
		{
			SaveZ[n][j] = z[n][j];
		}
	}

	/* which matrix solver? */
	if( strcmp(TypMatrx.chMatrix,"matin1 ") == 0 )
	{
		/* this is the usual matrix inversion method, slightly faster */
		/*matin1();*/
		nerror1 = 0;
		if( nerror1 != 0 )
		{
			fprintf( ioQQQ, " hydrogen matrix error, stop.\n" );
			puts( "[Stop in HydroLevelPop]" );
			cdEXIT(1);
		}
	}

	/* this is the default */
	else if( strcmp(TypMatrx.chMatrix,"linpack") == 0 )
	{
		/*double amat[LMHLVL+2][LMHLVL+2];*/
		double *amat;
	  /* nHLevels[ipZ]+1 is right dimension of matrix */
#		ifdef AMAT
#			undef AMAT
#		endif
#		define AMAT(I_,J_)	(*(amat+(I_)*(nHLevels[ipZ]+1)+(J_)))
		/* malloc space for the  1-d array */
		if( (amat=(double*)malloc( (sizeof(double)*(unsigned)((nHLevels[ipZ]+1)*(nHLevels[ipZ]+1)) ))) == NULL )
		{
			fprintf( ioQQQ, " HydroLevelPop malloc amat error\n" );
			puts( "[Stop in HydroLevelPop]" );
			cdEXIT(1);
		}

		/* this one may be more robust */
		for( j=ipH1s; j <= nHLevels[ipZ]; j++ )
		{
			for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
			{
				/*amat[n][j] = z[n][j];*/
				AMAT(n,j) = z[n][j];
			}
			bvec[j] = z[nHLevels[ipZ]+1][j];
		}
		/*DGETRF(nHLevels[ipZ]+1,nHLevels+1,
		  (double*)amat,LMHLVL+2,ipiv,&nerror1);
		DGETRS('N',nHLevels[ipZ]+1,1,(double*)amat,LMHLVL+2,ipiv,
		  bvec,LMHLVL+1,&nerror2);*/
		DGETRF(nHLevels[ipZ]+1,nHLevels[ipZ]+1,
		  amat,(nHLevels[ipZ]+1),ipiv,&nerror1);
		DGETRS('N',nHLevels[ipZ]+1,1,amat,(nHLevels[ipZ]+1),ipiv,
		  bvec,(nHLevels[ipZ]+1),&nerror2);
		if( nerror1 != 0 || nerror2 != 0 )
		{
			fprintf( ioQQQ, " HydroLevelPop dgetrs error\n" );
			puts( "[Stop in HydroLevelPop]" );
			cdEXIT(1);
		}
		/* now put results back into z so rest of code treates only
		 * one case - as if matin1 had been used */
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			z[nHLevels[ipZ]+1][n] = bvec[n];
		}
		free( amat);
	}
	else if( strcmp(TypMatrx.chMatrix,"veclib ") == 0 )
	{
		double /*amat[LMHLVL+2][LMHLVL+2], */ rcond;
		/* Jason found this one on the Exemplar, distributed source just stops */
		fprintf( ioQQQ, " this has not been checked since H atom conv\n" );
		for( j=ipH1s; j <= nHLevels[ipZ]; j++ )
		{
			/*for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
			{
				amat[n][j] = z[n][j];
			}*/
			bvec[j] = z[nHLevels[ipZ]+1][j];
		}
		job = 0;
		rcond = 0.;
		/*dgeco((double*)amat,nHLevels[ipZ]+2,nHLevels+1,ipiv,rcond, work);
		dgesl((double*)amat,nHLevels[ipZ]+2,nHLevels+1,ipiv,bvec, job);*/
		dgeco((double*)z,nHLevels[ipZ]+2,nHLevels[ipZ]+1,ipiv,rcond, work);
		dgesl((double*)z,nHLevels[ipZ]+2,nHLevels[ipZ]+1,ipiv,bvec, job);
		/* now put results back into z so rest of code treates only
		 * one case - as if matin1 had been used */
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			z[nHLevels[ipZ]+1][n] = bvec[n];
		}
		puts( "[Stop in HydroLevelPop]" );
		cdEXIT(1);
	}
	else
	{
		fprintf( ioQQQ, " chMatrix type insane in HydroLevelPop, was%7.7s\n", 
		  TypMatrx.chMatrix );
		puts( "[Stop in HydroLevelPop]" );
		cdEXIT(1);
	}
	/* end of branches for which matrix solution, now check if valid */

	/* check whether solution is valid */
	for( level=ipH1s; level <= nHLevels[ipZ]; level++ )
	{
		double BigSoln= 0.;
		error[level] = 0.;
		for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
		{
			/* remember the largest value of the soln matrix to div by below */
			if( fabs(SaveZ[n][level] ) > BigSoln )
				BigSoln = fabs(SaveZ[n][level]);

			error[level] += SaveZ[n][level]*z[nHLevels[ipZ]+1][n];
		}

		if( BigSoln > 0. )
		{
			error[level] = (error[level] - SaveZ[nHLevels[ipZ]+1][level])/ BigSoln;
		}
		else
		{
			error[level] = 0.;
		}
	}

	/* convert from departure coef in Z(I) to level pop rel to HII */

	/* put departure coefficients and level populations into master array */
	for( ipLo=ipH1s; ipLo <= nHLevels[ipZ]; ipLo++ )
	{
		hydro.hn[ipZ][ipLo] = (float)z[nHLevels[ipZ]+1][ipLo];
		if( hcolrt.hlte[ipZ][ipLo] > 0. )
		{
			hydro.hbn[ipZ][ipLo] = 
				(hydro.hn[ipZ][ipLo]/
				(hcolrt.hlte[ipZ][ipLo]* phycon.eden) );
		}
		else
		{
			hydro.hbn[ipZ][ipLo] = 0.;
		}
#		if 0
		hydro.hbn[ipZ][ipLo] = z[nHLevels[ipZ]+1][ipLo];

		/* hn(n,ipZ) are pop per level rel to HII; n.e., N(n)/N(HII) */
		hydro.hn[ipZ][ipLo] = 
			(float)(hydro.hbn[ipZ][ipLo]*hcolrt.hlte[ipZ][ipLo]*
		  phycon.eden);
#		endif
	}

#	ifdef DEBUG_FUN
	fputs( " <->HydroLevelPop()\n", debug_fp );
#	endif
	return;
}

