/*HydroOTS evaluate model hydrogen atom ots rates */
#include "cddefines.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "ionrange.h"
#include "opacity.h"
#include "pbowen.h"
#include "abundances.h"
#include "converge.h"
#include "phycon.h"
#include "ionfracs.h"
#include "trace.h"
#include "nh.h"
#include "rfield.h"
#include "heots.h"
/* following two needed for some debug statements */
#include "radius.h"
#include "rt.h"
/* the Bowen HeII yield */
#define BOWEN 0.3f

void HydroOTS(void)
{
	long int ipHi, 
	  ipLo, 
	  ipZ, 
	  n;
	float bwnfac ,
	  ots660 ,
	  otsnew;

#	ifdef DEBUG_FUN
	fputs( "<+>HydroOTS()\n", debug_fp );
#	endif

	/**************************************************************************
	 *
	 * the bowen HeII - OIII fluorescense problem
	 *
	 **************************************************************************/

	ipZ = 1;
	if( abundances.lgElmtOn[ipZ] )
	{
		/* conversion per unit atom to OIII, at end of sub we divide by it,
		 * to fix lines back to proper esc/dest probs */
		bwnfac = BOWEN * (1.f - HydroLines[ipZ][ipH2p][ipH1s].Pesc );

		/* escaping part that will be added to the outward beam in metdif,
		 * this uses ipLnEsc instead of ipLnDesp */
		heots.esc660 = HydroLines[ipZ][ipH2p][ipH1s].Aul*
		  HydroLines[ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ+2][ipZ]*
		  HydroLines[ipZ][ipH2p][ipH1s].Pesc *BOWEN*2.0f * 0.8f ;

		/* the last factor accounts for fact that two photons are produced,
		 * and the branching ratio */
		ots660 = HydroLines[ipZ][ipH2p][ipH1s].Aul*
		  HydroLines[ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ+2][ipZ]*
		  HydroLines[ipZ][ipH2p][ipH1s].Pdest *BOWEN*2.0f * 0.8f ;

		/* now add this to the ots field */
		RTAddOTSLin(ots660 , pbowen.ip660 );

		/* decrease the destruction prob by the amount we will add elsewhere,
		 * ok since dest probs updated on every iteration*/
		HydroLines[ipZ][ipH2p][ipH1s].Pdest *= bwnfac;
	}

	else
	{
		bwnfac = 1.;
	}

	/* make ots fields due to hydrogen lines and continua */

	/* loop over all elements */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		/* do if this stage exists */
		if( (IonRange.IonHigh[ipZ] == ipZ+1)  )
		{

			/**************************************************************************
			 *
			 * generate hydrogenic line ots rates 
			 *
			 **************************************************************************/

			/* now loop over all possible levels, but cannot include two photon
			 * since there is no pointer to this continuum */
			for( ipHi=ipH2p; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				for( ipLo=ipH1s; ipLo < ipHi; ipLo++ )
				{
					/* ots rates, the destp prob was set in hydropesc */
					HydroLines[ipZ][ipHi][ipLo].ots = 
						HydroLines[ipZ][ipHi][ipLo].Aul*
						HydroLines[ipZ][ipHi][ipLo].PopHi*xIonFracs[ipZ+2][ipZ]*
						HydroLines[ipZ][ipHi][ipLo].Pdest;

					assert( HydroLines[ipZ][ipHi][ipLo].ots >= 0. );
					/* option to kill lyalpha ots rates 
					if( ipZ==0 && ipHi==ipH2p && ipLo==ipH1s )
					{
						broken();
						HydroLines[ipZ][ipHi][ipLo].ots = 0.;
					}*/

					/* finally dump the ots rate into the stack */
					RTAddOTSLin(HydroLines[ipZ][ipHi][ipLo].ots,
						HydroLines[ipZ][ipHi][ipLo].ipCont );
				}
			}
			{
				/* debugging code for Lya problems */
				/*@-redef@*/
				enum {DEBUG=FALSE};
				/*@+redef@*/
				if( DEBUG )
				{
					if( ipZ==7  && nzone > 44 ) 
					{
#						if 0
					  fprintf(ioQQQ,"%li blr89bugg hlyaots\t%g\t%g\t%g\t%g\t%g\t%g\t%g\n",
							conv.nTotalIoniz, 
							HydroLines[ipZ][ipH2p][ipH1s].TauIn ,
							HydroLines[ipZ][ipH2p][ipH1s].TauTot,
							HydroLines[ipZ][ipH2p][ipH1s].Pdest,
							HydroLines[ipZ][ipH2p][ipH1s].ots , 
							opac.opac[HydroLines[ipZ][ipH2p][ipH1s].ipCont-1] ,
							rfield.otslinNew[HydroLines[ipZ][ipH2p][ipH1s].ipCont-1] ,
							HydroLines[ipZ][ipH2p][ipH1s].PopHi*xIonFracs[ipZ+2][ipZ]
							);
#						endif
						fprintf(ioQQQ,"%li hlyaots\t%g\t%g\t%g\n",
							nzone, xIonFracs[ipZ+2][ipZ] ,
							xIonFracs[12][ipSILICON],
							xIonFracs[16][ipIRON]);
					}
				}
			}

			/**************************************************************************
			 *
			 * ots recombination bound-free b-f continua continuum
			 *
			 **************************************************************************/

			/* put in OTS continuum */
			for( n=ipH1s; n <= nHLevels[ipZ]; n++ )
			{
				otsnew = (float)(hydro.hrec[ipZ][n][ipRecRad]*
					(1. - hydro.hrec[ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ+2][ipZ]);
				assert( otsnew >= 0. );

				/*otsnew = (rfield.otscon[nh.ipHn[ipZ][n]-1]*
					(1.f - damp) + 
					damp*hydro.hrec[ipZ][n][ipRecRad]*
					(1.f - hydro.hrec[ipZ][n][ipRecEsc])*phycon.eden*
					xIonFracs[ipZ+2][ipZ]/
					opac.opac[nh.ipHn[ipZ][n]-1])/2.f;*/

				/* continuum energy index used in this routine is decremented by one there */
				RTAddOTSCon(otsnew,nh.ipHn[ipZ][n]);
				/* debugging code for rec continua */
				{
					/*@-redef@*/
					enum {DEBUG=FALSE};
					/*@+redef@*/
					if( DEBUG )
					{
						if( nzone > 446 && ipZ==0 && n==3) 
						{
							fprintf(ioQQQ,"hotsdebugg %li hn2=%.2e h con ots\t%g\t%g\t%g\t%g\t%g\t%g\n",
								nzone, radius.depth ,
								HydroLines[0][2][0].PopHi*xIonFracs[2][0],
								otsnew,
								hydro.hrec[ipZ][n][ipRecEsc] , 
								opac.opac[nh.ipHn[ipZ][n]-1] ,
								rfield.otsconNew[nh.ipHn[ipZ][n]-1] ,
								opac.e2TauAbs[nh.ipHn[ipZ][n]-1] );
						}
					}
				}
			}
		}
	}
	/* more debugging code for rec continua */
	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			ipZ = 0;
			fprintf(ioQQQ,"hotsdebugg %li \t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\t%.2e\n",
				nzone, 
				rfield.otsconNew[nh.ipHn[ipZ][0]-1],
				rfield.otsconNew[nh.ipHn[ipZ][1]-1],
				rfield.otsconNew[nh.ipHn[ipZ][3]-1],
				rfield.otsconNew[nh.ipHn[ipZ][4]-1],
				rfield.otsconNew[nh.ipHn[ipZ][5]-1],
				rfield.otsconNew[nh.ipHn[ipZ][6]-1],
				       opac.opac[nh.ipHn[ipZ][6]-1]);
		}
	}

	ipZ = 1;
	if( abundances.lgElmtOn[ipZ] && bwnfac > 0. )
	{
		/* increase the destruction prob by the amount we wdecreased it above */
		HydroLines[ipZ][ipH2p][ipH1s].Pdest /= bwnfac;
	}

	if( trace.lgTrace )
	{
		fprintf(ioQQQ,"     HydroOTS Pdest %.2e ots rate %.2e in otslinNew %.2e con opac %.2e\n",
			HydroLines[0][ipH2p][ipH1s].Pdest , HydroLines[0][ipH2p][ipH1s].ots , 
			rfield.otslinNew[HydroLines[0][ipH2p][ipH1s].ipCont-1]  ,
			opac.opac[HydroLines[0][ipH2p][ipH1s].ipCont-1]
			);
	}

#	ifdef DEBUG_FUN
	fputs( " <->HydroOTS()\n", debug_fp );
#	endif
	return;
}

