/*iWavLen convert real wavelength in A into integer form used in printout */
#include <limits.h>
#include "cddefines.h"
#include "taulines.h"
#include "iwavlen.h"

long iWavLen(
	/* the incoming structure we will analyse */
	EmLine * t ,
	/* units, 'A' for Angstroms, 'm' for microns */
	char *chUnits , 
	/* the shift for the decimal point */
	char *chShift )
{
	long int iWavLen_v;

#	ifdef DEBUG_FUN
	fputs( "<+>iWavLen()\n", debug_fp );
#	endif

	/*
	 * this routine converts floating wavelengths into intergers used in output
	 * micron > 100, give micons
	 * micron > 10 in tenths of micron
	 * micron > 1 in hundreths of microns
	 * Ang < 10, in hundreths of Ang
	 * Ang < 100 in tenths of ang 
	 */

	*chUnits = 'x';
	*chShift = 'x';

	if( t->WLAng >= (float)LONG_MAX )
	{
		/* set to zero if larger than a long int */
		iWavLen_v = 0;
	}
	else if ( t->WLAng == 0 )
	{
		/* special two photon signal */
		iWavLen_v = 0;
		/* default is Angstroms, with no shift */
		*chUnits = ' ';
		*chShift = ' ';
	}
	else
	{
		/* add 0.5 so that integer rounds up */
		iWavLen_v = (long)(t->WLAng + 0.5);
		/* default is Angstroms, with no shift */
		*chUnits = 'A';
		*chShift = ' ';

		/* for wavelenghts longward of 100 microns, give wavelength in microns */
		if( iWavLen_v > 1000000 )
		{
			iWavLen_v /= 10000;
			/* units are precise microns */
			*chUnits = 'm';
			*chShift = ' ';
		}

		/* >>chng 00 sep 12, change printed wavelengths below 100A */
		/* this option is to use the new scale for wavelengths less than 100A */
		else if( lgWavLenNew )
		{
			if( iWavLen_v > 100000 )
			{
				/* wavelengths longer than 10 micron */
				iWavLen_v /= 1000;
				/* units are tenths of microns */
				*chUnits = 'm';
				*chShift = '1';
			}

			else if( iWavLen_v > 10000 )
			{
				/* wavelengths longer than 1 micron */
				iWavLen_v /= 100;
				/* units are hundreths of microns */
				*chUnits = 'm';
				*chShift = '2';
			}

			else if( iWavLen_v < 10 )
			{
				/* add 0.5 so that integer rounds up */
				iWavLen_v = (long)(t->WLAng*100. + 0.5);
				/* units are hundreths of Angstroms */
				*chUnits = 'A';
				*chShift = '2';
			}
			else if( iWavLen_v < 100 )
			{
				/* add 0.5 so that integer rounds up */
				iWavLen_v = (long)(t->WLAng*10. + 0.5);
				/* units are tenths of Angstroms */
				*chUnits = 'A';
				*chShift = '1';
			}
		}
		/* this option is set with set oldwavelength command to use old
		 * scale for wavelengths below 10A */
		else if( !lgWavLenNew )
		{
			if( iWavLen_v > 10000 )
			{
				/* wavelengths longer than 1 micron */
				iWavLen_v /= 1000;
				/* units are tenths of microns */
				*chUnits = 'm';
				*chShift = '1';
			}

			else if( iWavLen_v < 10 )
			{
				/* add 0.5 so that integer rounds up */
				iWavLen_v = (long)(t->WLAng*10. + 0.5);
				/* units are tenths of Angstroms */
				*chUnits = 'A';
				*chShift = '1';
			}
		}
		/* this is the x-ray part, wavelength in A less than 10 */
	}

#	ifdef DEBUG_FUN
	fputs( " <->iWavLen()\n", debug_fp );
#	endif
	return( iWavLen_v );
}

