/*lines_continuum put energetics, H, and He lines into line intensity stack */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "nhe1lvl.h"
#include "linesave.h"
#include "coolheavy.h"
#include "secondaries.h"
#include "hydrogenic.h"
#include "grainvar.h"
#include "opacity.h"
#include "totlum.h"
#include "printit.h"
#include "heots.h"
#include "nh.h"
#include "h.h"
#include "hmi.h"
#include "heat.h"
#include "heating.h"
#include "comneg.h"
#include "heatlmax.h"
#include "hsrate.h"
#include "he1ex.h"
#include "smbeta.h"
#include "phycon.h"
#include "ionfracs.h"
#include "dphoht.h"
#include "compton.h"
#include "phe1lv.h"
#include "heinwd.h"
#include "he3c.h"
#include "he3tau.h"
#include "he3lines.h"
#include "dtrans.h"
#include "sphere.h"
#include "cionhe.h"
#include "codish.h"
#include "hfbnet.h"
#include "rfield.h"
#include "hcaseb.h"
#include "predcont.h"
#include "trace.h"
#include "photrate.h"
#include "ip626.h"
#include "iphe1l.h"
#include "ionrange.h"
#include "linadd.h"
#include "lindst.h"
#include "iwavlen.h"
#include "radius.h"
#include "putline.h"
#include "putextra.h"
#include "getmaxhline.h"
#include "lines.h"

void lines_continuum(void)
{

	long int i,lamba;
	double f1, 
		f2 , 
		bac , 
		flow;

#	ifdef DEBUG_FUN
	fputs( "<+>lines_continuum()\n", debug_fp );
#	endif

	/* some continua */
	i = StuffComment( "continua" );
	linadd( 0., i , "####", 'i',' ',' ');

	/* memory not allocated until ipass >= 0 */
	if( LineSave.ipass > 0 )
		LineSv[LineSave.nsum].sumlin = 0.;

	/***********************************************************************
	 * stuff in Bac ratio - continuum above the BJ 
	 * this is trick, zeroing out saved continuum integrated so far,
	 * and adding the current version, so that the line array gives the 
	 * value in the final continuum 
	 *
	 * reflec continuum is different from others since relative to inner
	 * radius, others for for this radius 
	 *************************************************************************/

	/***************************************************************************
	 * "Bac " , 3646,  this is total continuum at peak of Balmer Jump          *
	 ***************************************************************************/
	/* >>chng 00 dec 02, remove opac.tmn */
	/* >>chng 00 dec 19, remove / radius.GeoDil */
	f1 = (rfield.ConOutInter[nh.ipHn[0][2]-1] + 
		rfield.ConOutNoInter[nh.ipHn[0][2]-1] + 
		rfield.ConRefDiff[nh.ipHn[0][2]-1]/radius.r1r0sq )/
	  rfield.widflx[nh.ipHn[0][2]-1]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-1]*/;

	/* extrapolated continuum below head */
	/* >>chng 00 dec 19, remove / radius.GeoDil */
	f2 = (rfield.ConOutInter[nh.ipHn[0][2]-2]+ 
		rfield.ConOutNoInter[nh.ipHn[0][2]-2] + 
		rfield.ConRefDiff[nh.ipHn[0][2]-2]/radius.r1r0sq )/
	  rfield.widflx[nh.ipHn[0][2]-2]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-2]*/;

	/* convert to nuFnu units */
	bac = (f1 - f2)*0.250*0.250*EN1RYD*radius.r1r0sq;
	/* memory not allocated until ipass >= 0 */
	if( LineSave.ipass > 0 )
		LineSv[LineSave.nsum].sumlin = 0.;

	/* residual flux at head of Balmer continuum, nuFnu */
	linadd(MAX2(0.,bac)/radius.dVeff,3646,"Bac ",'i',' ',' ');

	/*********************************************************************
	 * "cout" , 3646,  this is outward continuum at peak of Balmer Jump  *
	 * equal to total in spherical geometry, half in opt thin open geo   *
	 *********************************************************************/
	/* >>chng 00 dec 02, remove opac.tmn */
	/* >>chng 00 dec 19, remove / radius.GeoDil */
	f1 = (rfield.ConOutInter[nh.ipHn[0][2]-1]+ rfield.ConOutNoInter[nh.ipHn[0][2]-1])/
		rfield.widflx[nh.ipHn[0][2]-1]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-1]*/;

	/* >>chng 00 dec 19, remove / radius.GeoDil */
	f2 = (rfield.ConOutInter[nh.ipHn[0][2]-2]+ rfield.ConOutNoInter[nh.ipHn[0][2]-2])/
		rfield.widflx[nh.ipHn[0][2]-2]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-2]*/;

	/* net balmer jump */
	bac = (f1 - f2)*0.250*0.250*EN1RYD*radius.r1r0sq;

	/* memory not allocated until ipass >= 0 */
	if( LineSave.ipass > 0 )
		LineSv[LineSave.nsum].sumlin = 0.;

	/* residual flux in Balmer continuum, nuFnu */
	linadd(MAX2(0.,bac)/radius.dVeff,3646,"cout",'i',' ',' ');

	/*********************************************************************
	 * "cref" , 3646,  this is reflected continuum at peak of Balmer Jump*
	 * equal to zero in spherical geometry, half of total in op thin opn *
	 *********************************************************************/
	/* >>chng 00 dec 02, remove opac.tmn */
	/* >>chng 00 dec 19, remove / radius.GeoDil */
	f1 = rfield.ConRefDiff[nh.ipHn[0][2]-1]/
		rfield.widflx[nh.ipHn[0][2]-1]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-1]*/;

	f2 = rfield.ConRefDiff[nh.ipHn[0][2]-2]/
		rfield.widflx[nh.ipHn[0][2]-2]/*/radius.GeoDil /opac.tmn[nh.ipHn[0][2]-2]*/;

	/* net balmer jump */
	bac = (f1 - f2)*0.250*0.250*EN1RYD;

	/* memory not allocated until ipass >= 0 */
	if( LineSave.ipass > 0 )
		LineSv[LineSave.nsum].sumlin = 0.;

	/* residual flux in Balmer continuum, nuFnu */
	linadd(MAX2(0.,bac)/radius.dVeff,3646,"cref",'i',' ',' ');

	/*********************************************************************
	 * "thin" , 3646, tot optically thin continuum at peak of Balmer Jump*/
	f1 = rfield.ConLocInter[nh.ipHn[0][2]-1]/rfield.widflx[nh.ipHn[0][2]-1];
	f2 = rfield.ConLocInter[nh.ipHn[0][2]-2]/rfield.widflx[nh.ipHn[0][2]-2];
	bac = (f1 - f2)*0.250*0.250*EN1RYD;

	/* residual flux in Balmer continuum, nuFnu */
	linadd(MAX2(0.,bac),3646,"thin",'i',' ',' ');

	/* incident continuum nu*f_nu at H-beta, at illuminated face of cloud  */
	smbeta.cn4861 /= (float)radius.dVeff;
	linadd(smbeta.cn4861,4860,"Inci",'i',' ',' ');

	/* incident continuum nu*f_nu near Ly-alpha, at illuminated face of cloud */
	smbeta.cn1216 /= (float)radius.dVeff;
	linadd(smbeta.cn1216,1215,"Inci",'i',' ',' ');
	smbeta.cn1216 *= (float)radius.dVeff;
	smbeta.cn4861 *= (float)radius.dVeff;

	if( LineSave.ipass > 0 )
	{
		smbeta.cn4861 = 0.;
		smbeta.cn1216 = 0.;
	}

	/* integrated Balmer continuum emission */
	flow = (hydro.hrec[0][ipH2p][ipRecRad] + hydro.hrec[0][ipH2s][ipRecRad])*
	  hydro.hrec[0][ipH2p][ipRecEsc]*phycon.eden*xIonFracs[2][0]*
	  5.45e-12;
	linadd(flow,0,"Ba C",'i',' ',' ');

	/* >>chng 96 june 13, had been hrec(3,1,ipRecEsc)
	 * Paschen continuum emission */
	flow = hydro.hrec[0][3][ipRecRad]*hydro.hrec[0][3][ipRecEsc]*
	  phycon.eden*xIonFracs[2][0]*3.53e-12;
	linadd(flow,0,"PA C",'i',' ',' ');


	/* He brems emission */
	linadd(CoolHeavy.bhe,0,"HeFF",'c',' ',' ');

	/* He recombination cooling */
	linadd(CoolHeavy.herec,0,"HeFB",'c',' ',' ');

	/* heavy element recombination cooling */
	linadd(CoolHeavy.heavfb,0,"MeFB",'c',' ',' ');

	/* metal brems emission */
	linadd(CoolHeavy.heavff,0,"MeFF",'c',' ',' ');

	/* total brems emission */
	linadd(hydro.FreeFreeCool+CoolHeavy.bhe+CoolHeavy.heavff,0,"ToFF",'i',' ',' ');

	/* predict emitted continuum at series of continuum points */
	/* variables are located in predcont.h, 
	 * NPREDCONT number of continuum points,
	 * next two arrays are defined in zerologic.c
	 * EnrPredCont - energy (Ryd) where we want to predict the continuum,
	 *               these are set in zerologic.c,
	 * ipPreCont -   pointers to energy within continuum array, 
	 *               these are set in ContCreatePointers
	 *
	 * ths array of continuum energies where this will be done is
	 * stored in the array EnrPredCont which lives in predcont.h
	 * and is initialized in zerologic.c
	 *
	 * the entry nFnu will only be printed if the command
	 * print diffuse continuum
	 * is entered - 
	 *
	 * this code should be kept parallel with that in dopunch, where
	 * punch continuum is produced, since two must agree */
	for( i=0; i < NPREDCONT; i++ )
	{
		char chUnits , chShift;
		double ContOutInciFnu , Cont_nInu ;
		double ConRefIncid, 
		  ConOutInter,
		  ConOutNoInter, 
		  ConRefDiff;
		double renorm;

		/* put wavelength in Angstroms into dummy structure, so that we can use iWavLen
		 * to get a proper wavelength with units, continuum energies are stored in EnrPredCont */
		TauDummy.WLAng = (float)(RYDLAM/EnrPredCont[i]);
		lamba = iWavLen(&TauDummy , &chUnits , &chShift );

		/* >>chng 00 dec 02, there were three occurances of /opac.tmn which had the
		 * effect of raising the summed continuum by the local opacity correction factor.
		 * in the case of the Lyman continuum this raised the reported value by orders
		 * of magnitude.  There have been commented out in the following for now. */
		/* reflected toal continuum (dif+incident emitted inward direction) */

		/* >>chng 00 dec 08, implement the "set nFnu [SOURCE_REFLECTED] ... command, PvH */
		/* >>chng 00 dec 19, remove / radius.GeoDil */
		renorm = rfield.anu2[ipPredCont[i]]*EN1RYD/rfield.widflx[ipPredCont[i]]/*/radius.GeoDil*/;
		  
		/* this is the reflected diffuse continuum */
		ConRefDiff = printit.lgDiffuseInward ? rfield.ConRefDiff[ipPredCont[i]]*renorm : 0.;
			/* /opac.tmn[ipPredCont[i]]*/

		/* reflected part of INCIDENT continuum (only incident, not diffuse, which was above) */
		ConRefIncid = printit.lgSourceReflected ? rfield.ConRefIncid[ipPredCont[i]]*renorm : 0.;
			/* /opac.tmn[ipPredCont[i]]*/

		/* the attenuated incident continuum */
		if( printit.lgSourceTransmitted )
		{
			ContOutInciFnu = rfield.flux[ipPredCont[i]]*renorm*radius.r1r0sq ;
		}
		else
		{
			ContOutInciFnu = 0. ;
		}

		/* outward diffuse interactive continuum */
		ConOutInter = printit.lgDiffuseOutward ? rfield.ConOutInter[ipPredCont[i]]*renorm*radius.r1r0sq : 0.;
			/* /opac.tmn[ipPredCont[i]]*/

		/* >>chng 00 dec 02, break this out as separate quantity */
		/* outward diffuse non-interactive continuum */
		ConOutNoInter = printit.lgDiffuseOutward ? rfield.ConOutNoInter[ipPredCont[i]]*renorm*radius.r1r0sq : 0.;
			/* /opac.tmn[ipPredCont[i]]*/

		/* memory has not been allocated until ipass >= 0, so must not access this element,
		 * this element will be used to save the following quantity */
		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum].sumlin = 0.;
		}
		/* total continuum produced by cloud at selected energy points */
		/* >>chng 00 dec 02, did not include ConRefIncid, ConOutNoInter */
		linadd((ConRefDiff+ConRefIncid+ConOutInter+ContOutInciFnu+ConOutNoInter)/radius.dVeff,
			lamba,"nFnu",'i',chUnits , chShift );

		/* emslin saves the per unit vol emissivity of a line, which is normally 
		 * what goes into linadd.  We zero this unit emissivity since it is so situation dependent */
		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum-1].emslin = 0.;
		}

		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum].sumlin = 0.;
		}
		Cont_nInu = rfield.flux[ipPredCont[i]]*renorm*radius.r1r0sq +
			rfield.ConRefIncid[ipPredCont[i]]*renorm;
		/* transmitted and reflected indicent continuum, at selected energy points */
		linadd( Cont_nInu/radius.dVeff,
			lamba,"nInu",'i',chUnits , chShift );

		/* emslin saves the per unit vo emissivity of a line, which isnormally 
		 * what goes into linadd.  We zero this unit emissivity since it is so situation dependent */
		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum-1].emslin = 0.;
		}

		/* memory has not been allocated until ipass >= 0 */
		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum].sumlin = 0.;
		}
		/* total reflected continuum, total inward emission plus
		 * reflected (XXdiffuseXX) total continuum */
		/* >>chng 00 dec 02, did not include ConRefIncid */
		linadd( (ConRefDiff+ConRefIncid)/radius.dVeff,lamba,"InwT",'i',chUnits , chShift);

		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum-1].emslin = 0.;
		}

		/* memory has not been allocated until ipass >= 0 */
		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum].sumlin = 0.;
		}

		/* reflected incident continuum (only incident) */
		linadd(ConRefIncid/radius.dVeff,lamba,"InwC",'i',chUnits , chShift);

		if( LineSave.ipass > 0 )
		{
			LineSv[LineSave.nsum-1].emslin = 0.;
		}
		/*fprintf( ioQQQ,"%li\n",lamba );*/
	}

	/* total grain heating by all sources, lines, collisions, incident continuum */
	linadd(gv.GrainHeatSum,0,"GraT",'i',' ',' ');

	/* grain heating by incident continuum */
	linadd(gv.GrainHeatInc,0,"GraI",'i',' ',' ');

	/* grain heating due to destruction of Ly alpha  */
	linadd(gv.GrainHeatLya,1216,"GraL",'i',' ',' ');

	/* grain heating due to collisions with gas */
	linadd(gv.GrainHeatCollSum,0,"GraC",'i',' ',' ');

	/* grain heating due to diffuse fields, may also have grain emission */
	linadd(gv.GrainHeatDif,0,"GraD",'i',' ',' ');

	/* part of H brems, in x-ray beyond 0.5KeV */
	linadd((hydro.FreeFreeCool+CoolHeavy.bhe)*sexp(5.8e6/phycon.te),0,"FF X"
	  ,'i',' ',' ');

	/* total Compton cooling */
	linadd(CoolHeavy.tccool,0,"ComC",'c',' ',' ');

	/* expansion cooling, only non-zero for wind */
	linadd(CoolHeavy.expans,0,"Expn",'c',' ',' ');

	/* electron - electron brems */
	linadd(CoolHeavy.eebrm,0,"eeff",'c',' ',' ');

	/* H radiative recombination cooling */
	linadd(hydro.HRadRecCool[0],0,"H FB",'i',' ',' ');

	/* net free-bound cooling and heating */
	linadd(MAX2(0.,HFBNet.HFreBndNet[0]),0,"HFBc",'c',' ',' ');
	linadd(MAX2(0.,-HFBNet.HFreBndNet[0]),0,"HFBh",'h',' ',' ');

	/* cooling due to induced rec of hydrogen */
	linadd(HFBNet.crcind[0],0,"Hind",'c',' ',' ');

	/* cooling due to induced rec of fully ionized helium */
	linadd(CoolHeavy.c3ind,0,"3He2",'c',' ',' ');

	/* cyclotron cooling */
	linadd(CoolHeavy.cyntrn,0,"Cycn",'c',' ',' ');

	/* changed from info to cooling june 25 95 to pick this up in primal.in
	 * collisionally excited La cooling */
	linadd(MAX2(0.,hydro.coola[0]),1216,"Cool",'i',' ',' ');

	/* collisionally de-excited La heating */
	linadd(MAX2(0.,-hydro.coola[0]),1216,"Heat",'i',' ',' ');

	/* cooling due to n>2 Lyman lines */
	linadd(MAX2(0.,hydro.restly[0]),960,"Crst",'i',' ',' ');

	/* heating due to n>2 Lyman lines */
	linadd(MAX2(0.,-hydro.restly[0]),960,"Hrst",'i',' ',' ');

	/* cooling due to n>3 Balmer lines */
	linadd(MAX2(0.,hydro.barest[0]),4861,"Crst",'i',' ',' ');

	/* heating due to n>3 Balmer lines */
	linadd(MAX2(0.,-hydro.barest[0]),4861,"Hrst",'i',' ',' ');

	/* cooling due to higher Paschen lines */
	linadd(MAX2(0.,hydro.hrest[0]),0,"Crst",'i',' ',' ');

	/* heating due to higher Paschen lines */
	linadd(MAX2(0.,-hydro.hrest[0]),0,"Hrst",'i',' ',' ');

#	ifdef DEBUG_FUN
	fputs( " <->LineSet1()\n", debug_fp );
#	endif
	return;
}
