/*lines_general put general information and energetics into line intensity stack */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "nhe1lvl.h"
#include "linesave.h"
#include "coolheavy.h"
#include "secondaries.h"
#include "hydrogenic.h"
#include "grainvar.h"
#include "cooling.h"
#include "opacity.h"
#include "totlum.h"
#include "heots.h"
#include "nh.h"
#include "h.h"
#include "hmi.h"
#include "heat.h"
#include "heating.h"
#include "comneg.h"
#include "heatlmax.h"
#include "hsrate.h"
#include "he1ex.h"
#include "he.h"
#include "smbeta.h"
#include "phycon.h"
#include "ionfracs.h"
#include "dphoht.h"
#include "compton.h"
#include "phe1lv.h"
#include "heinwd.h"
#include "he3c.h"
#include "he3tau.h"
#include "he3lines.h"
#include "dtrans.h"
#include "sphere.h"
#include "cionhe.h"
#include "codish.h"
#include "hfbnet.h"
#include "rfield.h"
#include "hcaseb.h"
#include "predcont.h"
#include "trace.h"
#include "photrate.h"
#include "ip626.h"
#include "iphe1l.h"
#include "ionrange.h"
#include "linadd.h"
#include "lindst.h"
#include "iwavlen.h"
#include "radius.h"
#include "putline.h"
#include "putextra.h"
#include "getmaxhline.h"
#include "lines.h"

void lines_general(void)
{
	long int i, 
	  ipHi, 
	  ipLo, 
	  ipZ, 
	  ipnt, 
	  nd;
	double 
	  dhtot, 
	  EmisFac ,/* used for hydrogenic emissivity or intensity */
	  hbetac, 
	  HeatMetal ,
	  ee511, 
	  hlalph;

#	ifdef DEBUG_FUN
	fputs( "<+>lines_general()\n", debug_fp );
#	endif

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "   lines_general called\n" );
	}

	/* start of general properties */
	i = StuffComment( "general properties" );
	linadd( 0., i , "####", 'i',' ',' ');

	/* do we want intensity/luminosity or emissivity */
	if( hydro.lgHydEmiss )
	{
		/* option to print emissivity, not intensity */
		EmisFac = 1./phycon.eden;
	}
	else
	{
		EmisFac = xIonFracs[2][0];
	}
	/* total H-beta from multi-level atom */
	ipZ = 0;
	ipHi = 4;
	ipLo = ipH2p;
	hbetac = 
		(HydroLines[ipZ][ipHi][ipH2s].Aul * 
		HydroLines[ipZ][ipHi][ipH2s].Pesc +
		HydroLines[ipZ][ipHi][ipH2p].Aul *
		HydroLines[ipZ][ipHi][ipH2p].Pesc ) *
		HydroLines[ipZ][ipHi][ipLo].PopHi*
		HydroLines[ipZ][ipHi][ipLo].EnergyErg*
		EmisFac ;
	/* total H-beta from multi-level atom */
	/*hbetac = HydroLines[0][4][ipH2s].xIntensity + */
		/*HydroLines[0][4][ipH2p].xIntensity;*/

	/* these lines added to outlin in metdif - following must be false 
	 * this passes array index for line energy in continuum mesh - in rest
	 * of code this is set by a previous call to PntForLine, this index
	 * is on the f not c scale */
	lindst(hbetac,4861,"TOTL",HydroLines[0][4][2].ipCont,
	  'i',FALSE, ' ', ' ' );

	/* total Ly-a from multi-level atom */
	ipHi = ipH2p;
	ipLo = ipH1s;
	hlalph = 
		HydroLines[ipZ][ipHi][ipLo].Aul* 
		HydroLines[ipZ][ipHi][ipLo].PopHi*
		HydroLines[ipZ][ipHi][ipLo].Pesc*
		HydroLines[ipZ][ipHi][ipLo].EnergyErg*
		EmisFac ;
	/*hlalph = HydroLines[0][ipH2p][ipH1s].xIntensity;*/
	linadd(hlalph,1216,"TOTL",'i',' ',' ');

	totlum.totlsv = totlum.totlsv/(float)radius.dVeff*sphere.covgeo;

	/* someday, put in 21 cm, A from Gould, ApJ 423, 522, =2.88426e-15 s-1 */
	
	/* total luminosity in incident continuum */
	linadd(totlum.totlsv,0,"Inci",'i',' ',' ');
	totlum.totlsv = totlum.totlsv*(float)radius.dVeff/sphere.covgeo;

	/* ipass is flag to indicate whether to only set up line array
	 * (ipass=0) or actually evaluate lines intensities (ipass=1) */
	if( LineSave.ipass > 0 )
	{
		totlum.totlsv = 0.;
	}

	/* total heating, all forms, information since individuals added later */
	linadd(heat.htot,0,"TotH",'i',' ',' ');
	/* total cooling, all forms, information since individuals added later */
	linadd(cooling.ctot,0,"TotC",'i',' ',' ');

	/* hydrogen photoionization heating, ground state only */
	linadd(HeatingCom.heating[0][0],0,"BFH1",'h',' ',' ');

	/* net hydrogen photoionization heating less rec cooling, all excited states
	 * normally zero, positive if excited states are net heating */
	linadd(HeatingCom.heating[1][0],0,"BFHx",'h',' ',' ');
	if( heat.htot > 0. )
	{
		if( HeatingCom.heating[22][0]/heat.htot > heatlmax.HeatLineMax )
		{
			heatlmax.HeatLineMax = (float)(HeatingCom.heating[22][0]/heat.htot);
			GetMaxhLine();
		}
	}

	/* heating due to induced lines absorption of continuum */
	linadd(HeatingCom.heating[22][0],0,"Line",'h',' ',' ');

	/* net cooling due to collisional ionization of Heo */
	linadd(MAX2(0.,he1exCOM.he1ex*he.heii),0,"He1i",'c',' ',' ');

	/* this is the heating due to 3-body recombination */
	linadd(MAX2(-he1exCOM.he1ex*he.heii,0.),0,"3He1",'h',' ',' ');

	/* total helium photoionization heating, all stages */
	linadd(HeatingCom.heating[0][1]+HeatingCom.heating[1][1]+HeatingCom.heating[2][1],
	  0,"BFHe",'h',' ',' ');

	HeatMetal = 0.;
	/* some sums that will be printed in the stack */
	for( ipZ=2; ipZ<LIMELM; ++ipZ)
	{
		/* we now have final solution for this element */
		for( i=IonRange.IonLow[ipZ]; i < IonRange.IonHigh[ipZ]; i++ )
		{
			/* total metal photo heating for LINES */
			HeatMetal += HeatingCom.heating[i][ipZ];
		}
	}
	/* total heavy element photoionization heating, all stages */
	linadd(HeatMetal,0,"TotM",'h',' ',' ');

	/* net free-free cooling, nearly cancels with cooling in lte */
	linadd(MAX2(0.,hydro.HFFNet),0,"HFFc",'c',' ',' ');

	/* net free-free heating, nearly cancels with cooling in lte */
	linadd(MAX2(0.,-hydro.HFFNet),0,"HFFh",'h',' ',' ');

	/* H brems (free-free) cooling */
	linadd(hydro.FreeFreeCool,0,"H FF",'i',' ',' ');

	/* total free-free heating */
	linadd(hydro.FreeFreeHeat,0,"FF H",'i',' ',' ');

	/* integrate the volume as a sanity check,
	 * this is compared with true volume in final.  The number can't
	 * actualy be unity since this would overflow on a 32 bit machine */
	linadd( 1.e-10 , 1 , "Unit" , 'i' ,' ',' ');

	/* Compton heating */
	linadd(Compton.cmheat,0,"ComH",'h',' ',' ');

	/* H- heating */
	linadd(MAX2(0.,hmi.hmihet),0,"H- H",'h',' ',' ');

	/* induced H- cooling */
	linadd(MAX2(0.,-hmi.hmihet),0,"H-Hc",'c',' ',' ');

	/* find total grain heating */
	dhtot = 0.;
	for( nd=0; nd < gv.nBin; nd++ )
	{
		/* add heating due to all grain species that are included */
		dhtot += gv.bin[nd]->GasHeatPhotoEl;
	}

	dphoht.TotalDustHeat += (float)(dhtot*radius.dVeff);
	dphoht.dphmax = MAX2((float)(dhtot/heat.htot),dphoht.dphmax);
	dphoht.h2pmax = MAX2(dphoht.h2pmax,(float)(HeatingCom.heating[16][0]/heat.htot));

	/* gas heating by grain photoionization */
	linadd(dhtot,0,"GrGH",'h',' ',' ');

	/* gas heating by thermionic emissions of grains */
	linadd(HeatingCom.heating[25][0],0,"GrTH",'h',' ',' ');

	/* heating/cooling due to charge transfer */
	linadd(MAX2(0.,HeatingCom.heating[24][0]),0,"CT H",'h',' ',' ');
	linadd(MAX2(0.,-HeatingCom.heating[24][0]),0,"CT C",'c',' ',' ');

	dphoht.dclmax = MAX2(dphoht.dclmax,(float)(gv.GasCoolColl/heat.htot));

	/* gas cooling by collisions with grains */
	linadd(gv.GasCoolColl,0,"GrGC",'c',' ',' ');

	/* "extra" heat added to this zone, from HEXTRA command */
	linadd(HeatingCom.heating[20][0],0,"extH",'h',' ',' ');

	/* "extra" cooling added to this zone, from CEXTRA command */
	linadd(CoolHeavy.cextxx,0,"extC",'c',' ',' ');

	/* 511keV annihilation line */
	ee511 = (phycon.hden + 4.*he.ahe)*PhotRate.PhotoRate[0][0][10][0]*2.*8.20e-7;
	PntForLine(2.427e-2,"e-e+",&ipnt);
	lindst(ee511,511,"e-e+",ipnt,'i',TRUE, ' ', ' ' );
	
#	ifdef DEBUG_FUN
	fputs( " <->lines_general()\n", debug_fp );
#	endif
	return;
}

