/*lines_helium put energetics, H, and He lines into line intensity stack */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "nhe1lvl.h"
#include "linesave.h"
#include "coolheavy.h"
#include "secondaries.h"
#include "hydrogenic.h"
#include "grainvar.h"
#include "opacity.h"
#include "abundances.h"
#include "totlum.h"
#include "heots.h"
#include "nh.h"
#include "h.h"
#include "hmi.h"
#include "heat.h"
#include "heating.h"
#include "comneg.h"
#include "heatlmax.h"
#include "hsrate.h"
#include "he1ex.h"
#include "he.h"
#include "smbeta.h"
#include "phycon.h"
#include "ionfracs.h"
#include "dphoht.h"
#include "compton.h"
#include "phe1lv.h"
#include "heinwd.h"
#include "he3c.h"
#include "he3tau.h"
#include "he3lines.h"
#include "dtrans.h"
#include "sphere.h"
#include "cionhe.h"
#include "codish.h"
#include "hfbnet.h"
#include "rfield.h"
#include "hcaseb.h"
#include "predcont.h"
#include "trace.h"
#include "photrate.h"
#include "ip626.h"
#include "iphe1l.h"
#include "ionrange.h"
#include "linadd.h"
#include "lindst.h"
#include "iwavlen.h"
#include "radius.h"
#include "putline.h"
#include "putextra.h"
#include "getmaxhline.h"
#include "lines.h"

void lines_helium(void)
{
	long int i, 
	  ipHi, 
	  ipLo, 
	  ipZ, 
	  ipnt;
	double 
	  cb206, 
	  cb5016, 
	  col, 
	  e228, 
	  e504, 
	  e584, 
	  e626, 
	  e911, 
	  eff, 
	  em, 
	  f10830, 
	  pop, 
	  rec, 
	  totl;

#	ifdef DEBUG_FUN
	fputs( "<+>lines_helium()\n", debug_fp );
#	endif

	/* HeI */
	i = StuffComment( "He iso-sequence" );
	linadd( 0., i , "####", 'i',' ',' ');

	/* He I triplet net collisional ionization cooling */
	linadd(MAX2(0.,cionhe.He3IonCool),0,"He3I",'c',' ',' ');

	/* He I triplet net 3-body heating */
	linadd(MAX2(0.,-cionhe.He3IonCool),0,"He3b",'h',' ',' ');

	/* He I 584, collisional excitation cooling */
	linadd(MAX2(0.,CoolHeavy.c584),584,"HeIC",'c',' ',' ');

	/* He I 584, collisional de-excitation heating */
	linadd(MAX2(0.,-CoolHeavy.c584),584,"HeIH",'h',' ',' ');

	/* total 584 emergent from cloud in inner direction
	 * now compute emergent Helium emergent flux */
	if( sphere.lgSphere )
	{
		e911 = 0.;
		e584 = 0.;
		e626 = 0.;
		e228 = 0.;
		e504 = 0.;
	}
	else
	{
		if( strncmp( dtrans.chDffTrns , "OU" , 2) == 0 )
		{
			eff = 0.5;
		}
		else
		{
			eff = 1.;
		}

		e504 = heots.esc504/2.*3.95e-11*eff;

		/* this is inward looking tau but no fac of two in original defn */
		e584 = heots.esc584/2.*3.41e-11/2.;

		/* this looked to illuminated face and does not need extra factor */
		e626 = heots.esc626*3.18e-11;
		e911 = heots.esc911*2.18e-11*eff;
		e228 = heots.esc228*8.72e-11*eff;
	}

	/* He I rec to ground escaping cloud */
	linadd(e504,504,"He I",'i',' ','A');

	/* He I Ly alpha escaping cloud
	 * call linadd( e584 , 584, 'esc ' ,'i')
	 * these lines added to outlin in metdif - following must be false */
	lindst(e584,584,"esc ",iphe1lCom.iphe1l[0][1],'i',FALSE, 'A', ' ' );

	/* He I 2^3S-ground emission escaping lluminated face of cloud
	 * call linadd( e626 , 626 , 'esc ' ,'i')
	 * these lines added to outlin in metdif - following must be false */
	lindst(e626,626,"esc ",ip626.ipHe23sGrnd,'i',FALSE, 'A', ' ' );

	/* He I 4471, */
	if( phycon.te <= 1e4 )
	{
		em = 2.42e-22/(phycon.te/phycon.te10);
	}
	else
	{
		em = 8.79e-22/phycon.te/phycon.te03/phycon.te01;
	}

	/* >>chng 97 jan 04, label was HeI, changed to Ca B for symmetry with others
	 * He I 4471 recombination only, fit to Brocklehurst '72 */
	rec = phycon.eden*he.heii*em;
	PntForLine(4471.,"He 1",&ipnt);
	lindst(rec,4471,"Ca B",ipnt,'i',TRUE, 'A', ' ' );

	/* He I total 5876, from n-level atom
	 * these lines added to outlin in metdif - following must be false */
	lindst(he3lines.p5876,5876,"TOTL",he3lines.ipHe3l[1],'i',FALSE, 'A', ' ' );

	/* inward part of 5876 */
	linadd(he3lines.p5876*he3tau[IPT5876-1].FracInwd,
	  5876,"Inwd",'i','A',' ');

	/* He I 5876 REC, simple fit to Brocklehurst */
	linadd(phycon.eden*he.heii*4.22e-21/(phycon.te*phycon.te10),
	  5876,"Ca B",'r','A',' ');

	/* He I 6678 REC, simple fit to Brocklehurst */
	rec = phycon.eden*he.heii*1.32e-21/(phycon.te*phycon.te10*phycon.te01);
	PntForLine(6678.,"He 1",&ipnt);
	lindst(rec,6678,"Ca B",ipnt,'i',TRUE, 'A', ' ' );

	/* 10830 from n-level atom
	 * these lines added to outlin in metdif - following must be false */
	lindst(he3lines.p10830,10830,"TOTL",he3lines.ipHe3l[0],'i',FALSE, 'A', ' ' );

	f10830 = 1.02e7*he3tau[IPT10830-1].Pesc/(1.02e7*
	  he3tau[IPT10830-1].Pesc + 2.51e-9*phycon.sqrte*
	  phycon.eden);

	/* He I 10830 produced by radiative recombination */
	rec = he.heii*phycon.eden*5.13e-22/(phycon.te70*phycon.te10*
	  phycon.te01*phycon.te01)*f10830;
	linadd(rec,10830,"reco",'i','A',' ');

	/* following expression from Clegg 1987 */
	pop = he.heii*5.79e-6*(pow(phycon.te/1e4,-1.18))/(1. + 3110./phycon.eden*
	  (pow(phycon.te/1e4,-0.51)));

	col = pop*he3c.c2s2p*1.84e-12*f10830;

	/* collisionally excited 10830 estimated from Clegg 1987 (not model atom) */
	linadd(col,10830,"coll",'i','A',' ');

	/* inward escaping HeI 10830 */
	linadd(heinwd.he10in,10830,"Inwd",'i','A',' ');

	/* He I 3889 from n-level atom
	 * these lines added to outlin in metdif - following must be false */
	lindst(he3lines.p3889,3889,"TOTL",he3lines.ipHe3l[3],'i',FALSE, 'A', ' ' );

	/* He I 7065 from n-level atom
	 * these lines added to outlin in metdif - following must be false */
	lindst(he3lines.p7065,7065,"TOTL",he3lines.ipHe3l[2],'i',FALSE, 'A', ' ' );

	/* total collisional He I triplet line cooling, from n-level atom */
	linadd(MAX2(0.,he3lines.he3clx),0,"CcHE",'c',' ',' ');

	/* total collisional de-exec He I heating, from n-level atom */
	linadd(MAX2(0.,-he3lines.he3clx),0,"ChHE",'h',' ',' ');

	em = phycon.eden*he.heii;
	cb206 = 1.064e-22/phycon.te70/phycon.te10/phycon.te03/phycon.te03*
	  em;

	/* case B He I 2.06 micron - new or old wavelength scaling?  */
	if( lgWavLenNew ) 
	{
		linadd(cb206,206,"Ca B",'i','m','2');
	}
	else
	{
		linadd(cb206,2,"Ca B",'i','m',' ');
	}

	/* He I 2.06 micron from model atom, all physics in */
	totl = he.heii*phe1lv.he12p*t206.Aul*t206.Pesc*
	  9.66e-13;
	PntForLine(20600.,"He 1",&ipnt);
	if( lgWavLenNew ) 
	{
		lindst(totl,206,"TOTL",ipnt,'i',TRUE, 'm', '2' );
	}
	else
	{
		lindst(totl,2,"TOTL",ipnt,'i',TRUE, 'm', ' ' );
	}

	/* Case B He I 5016 */
	cb5016 = 5.43e-23/phycon.te70/phycon.te10*em;
	PntForLine(5016.,"He 1",&ipnt);
	lindst(cb5016,5016,"Ca B",ipnt,'i',TRUE, 'A', ' ' );

	/* He II Lyman continuum */
	linadd(e228,228,"HeII",'i','A',' ');

	/* He II Balmer continuum escaping from cloud */
	linadd(e911,911,"He2C",'i','A',' ');

	/* this is the main printout, where line intensities are entered into the stack */
	for( ipZ=1; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ] )
		{
			/* do not print more than first 50 levels, since my pc does not
			 * have enough memory */
			/* NB NB - low and high must be in this order so that all balmer, paschen,
			 * etc series line up correctly in final printout */
			for( ipLo=0; ipLo < nHeLevels[ipZ]-1; ipLo++ )
			{
				for( ipHi=ipLo+1; ipHi < nHeLevels[ipZ]; ipHi++ )
				{
					HeLines[ipZ][ipHi][ipLo].phots = 
						HeLines[ipZ][ipHi][ipLo].Aul*
						HeLines[ipZ][ipHi][ipLo].PopHi*
						HeLines[ipZ][ipHi][ipLo].Pesc*
						xIonFracs[ipZ+1][ipZ];

					/* now find line intensity */
					/* >>chng 01 jan 15, put cast double to force double evaluation */
					HeLines[ipZ][ipHi][ipLo].xIntensity = 
						(double)HeLines[ipZ][ipHi][ipLo].Aul*
						(double)HeLines[ipZ][ipHi][ipLo].PopHi*
						(double)HeLines[ipZ][ipHi][ipLo].Pesc*
						(double)xIonFracs[ipZ+1][ipZ]*
						(double)HeLines[ipZ][ipHi][ipLo].EnergyErg;
					/* this will enter .xIntensity into the line stack */
					PutLine(&HeLines[ipZ][ipHi][ipLo]);
				}
			}
		}
	}

	/* ====================================================
	 * end helium
	 * ====================================================*/

	if( LineSave.ipass <= 0 )
	{
		/* on first pass set flag saying that case b results are ok,
		 * will be set false if we ever overstep bounds of table */
		CaseBHS.lgHCaseBOK = TRUE;
		CaseBHS.lgHeCaseBOK = TRUE;
	}

	if( trace.lgTrace )
	{
		fprintf( ioQQQ, "   lines_helium returns\n" );
	}

#	ifdef DEBUG_FUN
	fputs( " <->lines_helium()\n", debug_fp );
#	endif
	return;
}

