/*lines_hydro put H-iso seq into line intensity stack */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "physconst.h"
#include "nhe1lvl.h"
#include "linesave.h"
#include "coolheavy.h"
#include "secondaries.h"
#include "hydrogenic.h"
#include "grainvar.h"
#include "opacity.h"
#include "abundances.h"
#include "totlum.h"
#include "heots.h"
#include "nh.h"
#include "h.h"
#include "hmi.h"
#include "heat.h"
#include "heating.h"
#include "comneg.h"
#include "heatlmax.h"
#include "hsrate.h"
#include "he1ex.h"
#include "smbeta.h"
#include "phycon.h"
#include "ionfracs.h"
#include "dphoht.h"
#include "compton.h"
#include "phe1lv.h"
#include "heinwd.h"
#include "he3c.h"
#include "he3tau.h"
#include "he3lines.h"
#include "dtrans.h"
#include "sphere.h"
#include "cionhe.h"
#include "codish.h"
#include "hfbnet.h"
#include "rfield.h"
#include "hcaseb.h"
#include "predcont.h"
#include "trace.h"
#include "photrate.h"
#include "ip626.h"
#include "iphe1l.h"
#include "ionrange.h"
#include "linadd.h"
#include "lindst.h"
#include "iwavlen.h"
#include "radius.h"
#include "putline.h"
#include "putextra.h"
#include "getmaxhline.h"
#include "lines.h"

void lines_hydro(void)
{
	long int i, ipZ, ipHi, ipLo;
	double hbetab, em , EmisFac, pump;

#	ifdef DEBUG_FUN
	fputs( "<+>lines_hydro()\n", debug_fp );
#	endif

	/* next come the H-iso sequence lines */
	i = StuffComment( "H-like iso-seq" );
	linadd( 0., i , "####", 'i',' ',' ');

	/* heating due to pair production */
	linadd(HeatingCom.heating[21][0],0,"pair",'h',' ',' ');

	/* case b intensity of Ly-alpha, no two photon */
	/* do we want intensity/luminosity or emissivity */
	if( hydro.lgHydEmiss )
	{
		/* option to print emissivity, not intensity */
		EmisFac = 1.;
	}
	else
	{
		EmisFac = xIonFracs[2][0]*phycon.eden;
	}

	linadd(hcaseb.HRecRad[0]*EmisFac*1.64e-11,1216,"Ca B",'i',' ',' ');

	/* H-beta computed from Q(H) and specified covering factor */
	smbeta.SimHBeta = smbeta.SimHBeta/(float)radius.dVeff*sphere.covgeo;
	linadd(smbeta.SimHBeta,4861,"Q(H)",'i',' ',' ');

	/* Ly-alpha from Q(H), high-dens lim, specified covering factor */
	linadd(smbeta.SimHBeta*34.19,1216,"Q(H)",'i',' ',' ');
	smbeta.SimHBeta = smbeta.SimHBeta*(float)radius.dVeff/sphere.covgeo;

	if( LineSave.ipass > 0 )
	{
		smbeta.SimHBeta = 0.;
	}

	/* this is main loop creating hydrogenic intensity or emissivity */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		if( hydro.lgHydEmiss )
		{
			/* option set with Hydrogen emissivity command
			 * will produce emissivity not intensity - luminosity */
			for( ipZ=0; ipZ < LIMELM; ipZ++ )
			{
				if( abundances.lgElmtOn[ipZ]  )
				{
					for( ipHi=ipH2s; ipHi <= nHLevels[ipZ]; ipHi++ )
					{
						for( ipLo=ipH1s; ipLo < ipHi; ipLo++ )
						{
							/* this is in real units not emissivit*/
							HydroLines[ipZ][ipHi][ipLo].phots = 
								HydroLines[ipZ][ipHi][ipLo].Aul*
								HydroLines[ipZ][ipHi][ipLo].PopHi*
								HydroLines[ipZ][ipHi][ipLo].Pesc*
								xIonFracs[ipZ+2][ipZ];

							/* now find line intensity  */
							HydroLines[ipZ][ipHi][ipLo].xIntensity = 
								(double)HydroLines[ipZ][ipHi][ipLo].Aul*
								(double)HydroLines[ipZ][ipHi][ipLo].PopHi*
								(double)HydroLines[ipZ][ipHi][ipLo].Pesc*
							    (double)HydroLines[ipZ][ipHi][ipLo].EnergyErg/phycon.eden;

						}
					}
				}
			}
		}

		else
		{
			/* this is the default case - luminosity - intensity */
			for( ipZ=0; ipZ < LIMELM; ipZ++ )
			{
				if( abundances.lgElmtOn[ipZ]  )
				{
					for( ipHi=ipH2s; ipHi <= nHLevels[ipZ]; ipHi++ )
					{
						for( ipLo=ipH1s; ipLo < ipHi; ipLo++ )
						{
							HydroLines[ipZ][ipHi][ipLo].phots = 
								HydroLines[ipZ][ipHi][ipLo].Aul*
								HydroLines[ipZ][ipHi][ipLo].PopHi*
								HydroLines[ipZ][ipHi][ipLo].Pesc*
								xIonFracs[ipZ+2][ipZ];

							/* now find line intensity and line ots rates */
							/* set hydrogen line emission and ots fields */
							/* >>chng 01 jan 15, add double cast to work at low densities */
							HydroLines[ipZ][ipHi][ipLo].xIntensity = 
								(double)HydroLines[ipZ][ipHi][ipLo].Aul*
								(double)HydroLines[ipZ][ipHi][ipLo].PopHi*
								(double)HydroLines[ipZ][ipHi][ipLo].Pesc*
								(double)xIonFracs[ipZ+2][ipZ]*
								(double)HydroLines[ipZ][ipHi][ipLo].EnergyErg;

						}
					}
				}
			}
		}
	}

	ipZ = 0;
	ipLo = 0;
	ipHi = 1;
#	if 0
	fprintf(ioQQQ,"%.2e %.2e %.2e %.2e %.2e %.2e \n",
		hydro.hn[0][ipH2s] , HydroLines[ipZ][ipHi][ipLo].PopHi,
		8.23, HydroLines[ipZ][ipHi][ipLo].Aul*HydroLines[ipZ][ipHi][ipLo].Pesc,
		1.634e-11, HydroLines[ipZ][ipHi][ipLo].EnergyErg );
	fprintf(ioQQQ,"%.2e %.2e \n",
		h.hii*hydro.hn[0][ipH2s]*8.23*1.634e-11, 
		HydroLines[ipZ][ipHi][ipLo].xIntensity );
#	endif

	/* create emissivity or intensity for hydrogenic species,
	 * first combine/bring balmer series together */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		if( IonRange.IonHigh[ipZ] == ipZ + 1 )
		{
			/* bring n=> 2s + 2p together into one line with 2s lower level
			 * must be 2s since it comes first in line stack */
			for( ipHi=3; ipHi <= nHLevels[ipZ]; ipHi++ )
			{
				HydroLines[ipZ][ipHi][ipH2s].xIntensity += 
					HydroLines[ipZ][ipHi][ipH2p].xIntensity;

				/* kill 2p */
				HydroLines[ipZ][ipHi][ipH2p].xIntensity = 0.;
			}
		}
	}

	/* H beta recombination, assuming old case B */
	hbetab = (double)((pow(10.,-20.89 - 0.10612*POW2(phycon.alogte - 4.4)))/
	  phycon.te);
	/* convert to emission per unit then dump into line stack */
	if( !hydro.lgHydEmiss )
	{
		hbetab *= xIonFracs[2][0]*phycon.eden;
	}
	/* this is old case b, had been in code for very long time */
	linadd(hbetab,4861,"CaBo",'i',' ',' ');

	/* 1640 1640 1640 */
	em = 2.03e-20/(phycon.te70*phycon.te10*phycon.te03);
	if( !hydro.lgHydEmiss )
	{
		em *= xIonFracs[3][1]*phycon.eden;
	}

	/* old prediction of He II 1640, case B at low densities */
	linadd(em,1640,"CaBo",'i',' ',' ');

	/* hydrogenic helium */
	/* old prediction of He II 4686, case B */
	em = 2.52e-20/(pow(phycon.te,1.05881));
	if( !hydro.lgHydEmiss )
	{
		em *= xIonFracs[3][1]*phycon.eden;
	}
	linadd(em,4686,"CaBo",'i',' ',' ');

	/* predict case b intensities of hydrogen lines */
	ipZ = 0;
	for( ipZ=0; ipZ<2; ++ipZ )
	{
		for( ipLo=ipH2p; ipLo<MIN2(6,nHLevels[ipZ]); ++ipLo )
		{
			for( ipHi=ipLo+1; ipHi< MIN2(ipLo+4,nHLevels[ipZ]); ++ipHi )
			{
				char chUnits , chShift ;
				/* Put case b predictions into line stack
				 * NB NB NB each Hummer & Storey case b line must be 
				 * explicitly clobbered by hand in routine final if not ok flag is set 
				 * since this indicates that we exceeded bounds of table,
				 * DO NOT want to print lines in that case */

				/* first do case b emssivity of balmer lines */

				/* get HS predictions */
				hbetab = (double)HSRate( ipHi,ipLo , ipZ+1, phycon.te , phycon.eden );
				if( hbetab<=0. )
				{
					/* routine returns -1 if outside bounds of table*/
					if( ipZ==0 )
					{
						/* hydrogen */
						CaseBHS.lgHCaseBOK = FALSE;
					}
					else
					{
						/* helium */
						CaseBHS.lgHeCaseBOK = FALSE;
					}
					hbetab = 0.f;
				}

				/* convert to emission per unit vol then dump into line stack */
				if( !hydro.lgHydEmiss )
				{
					hbetab *= xIonFracs[ipZ+2][ipZ]*phycon.eden;
				}

				/* this is wavelength of interpolated case b from HS tables */
				i = iWavLen(&HydroLines[ipZ][ipHi][ipLo] , &chUnits , &chShift );

				/* add to line stack */
				linadd(hbetab,i,"Ca B",'i' , chUnits , chShift );
			}
		}
	}

	/* this is the main printout, where line intensities are entered into the stack */
	for( ipZ=0; ipZ < LIMELM; ipZ++ )
	{
		if( abundances.lgElmtOn[ipZ] )
		{
			/* do not print more than first 50 levels, since my pc does not
			 * have enough memory */
			/* NB NB - low and high must be in this order so that all balmer, paschen,
			 * etc series line up correctly in final printout */
			for( ipLo=ipH1s; ipLo < MIN2(50,nHLevels[ipZ]-1); ipLo++ )
			{
				/* don't bother with decays to 2p ince we set them to zero above */
				if( ipLo==ipH2p )
					continue;
				for( ipHi=ipLo+1; ipHi <= MIN2(50,nHLevels[ipZ]); ipHi++ )
				{
					/* this will enter .xIntensity into the line stack */
					PutLine(&HydroLines[ipZ][ipHi][ipLo]);
				}
			}
		}
	}

	/* total collisional cooling due to all hydrogen lines */
	linadd(MAX2(0.,hydro.HLineTotCool[0]),912,"Clin",'c',' ',' ');

	/* total collisional heating due to all hydrogen lines */
	linadd(MAX2(0.,-hydro.HLineTotCool[0]),912,"Hlin",'h',' ',' ');

#	if 0
	/* >>chng 00 nov 25, two photon now in main hydro loop */
	/* 2-photon two photon emission from multi-level atom */
	linadd(h.hii*hydro.hn[0][ipH2s]*8.23*1.634e-11,0,"2 NU",'r',' ',' ');
#	endif

	/* la contribution from suprathermal secondaries from ground */
	linadd(Secondaries.x12tot*h.hii*hydro.hn[0][ipH1s]*1.634e-11,1216,"LA X"
	  ,'i',' ',' ');

	/* H-alpha produce by H- mutual neutralization */
	linadd(hmi.HalphaHmin*3.032e-12,6563,"H-CT",'i',' ',' ');

	/* "Ly alpha" produced by induced two photon */
	linadd(hydro.hn[0][ipH2s]*h.hii*HydroLines[0][ipH2s][ipH1s].pump*1.634e-11,1216,
		"Ind2" ,'i',' ',' ');

	/* H-beta produced by continuum pumping in optically thin ld limit */
	/* factor of 0.4836 is ratio of A(4-2)/(A(4-3)+A(4-2))
	 * the IPLNPUMP is the actual pumping rate per atom */
	pump = (double)(HydroLines[0][4][ipH1s].pump*hydro.hn[0][ipH1s]*
	  h.hii*4.09e-12*0.4836);
	linadd(pump,4861,"Pump",'r',' ',' ');

	/* CHION is HEX*HII (in COOLR) and is net col ionz-3 body heat
	 * collision ionization cooling of hydrogen */
	linadd(MAX2(0.,hydro.chion[0]),0,"CION",'c',' ',' ');

	/* this is the heating due to 3-body recombination */
	linadd(MAX2(-hydro.chion[0],0.),0,"3bHt",'h',' ',' ');

	/*fixit(); stark term not now added to total rad decay rate -
			   * so energy not right in this printout - why is stark
				* not added */
	/* Stark broadening contribution to line */
	linadd(h.hii*hydro.hn[0][ipH2p]*0.*hydro.pestrk[2][0]*1.634e-11,
	  1216,"Strk",'i',' ',' ');

	/* Stark broadening contribution to line */
	linadd(h.hii*hydro.hn[0][3]*hydro.pestrk[3][2]*3.025e-12,
	  6563,"Strk",'i',' ',' ');

	/* Stark broadening contribution to line */
	linadd(h.hii*hydro.hn[0][4]*hydro.pestrk[4][2]*4.084e-12,
	  4861,"Strk",'i',' ',' ');

	/* Stark broadening contribution to line */
	linadd(h.hii*hydro.hn[0][4]*hydro.pestrk[4][3]*1.059e-12,
	  18751,"Strk",'i',' ',' ');

	/* pestrk[5,4] is A[4,5]*pest[4,5] 
	 * Stark broadening contribution to line */
	linadd(h.hii*hydro.hn[0][5]*hydro.pestrk[5][4]*4.900e-13,
	  40512,"Strk",'i',' ',' ');

	/* this can fail if RTMake never updates the ots rates, a logic error,
	 * but only assert this during actual calculation (ipass>0), */
	assert( LineSave.ipass  <1 ||
		HydroLines[0][ipH2p][ipH1s].ots>= 0.);
	/* portion of line lost due to absorp by background opacity */
	linadd(HydroLines[0][ipH2p][ipH1s].ots*HydroLines[0][ipH2p][ipH1s].EnergyErg, 1216,"Dest",'i',' ',' ');

	/* portion of line lost due to absorp by background opacity */
	linadd(HydroLines[0][3][ipH2s].ots*HydroLines[0][3][ipH2s].EnergyErg, 6563,"Dest",'i',' ',' ');

	/* portion of line lost due to absorp by background opacity */
	linadd(HydroLines[0][5][4].ots*HydroLines[0][5][4].EnergyErg,40516, "Dest",'i',' ',' ');

	/* portion of line lost due to absorp by background opacity */
	linadd(HydroLines[0][4][ipH2s].ots*HydroLines[0][4][ipH2s].EnergyErg, 4861,"Dest",'i',' ',' ');

	/* portion of line lost due to absorp by background opacity */
	linadd(HydroLines[0][4][3].ots*HydroLines[0][4][3].EnergyErg ,18751, "Dest",'i',' ',' ');

	/* Ly-alpha destroyed by overlap with FeII */
	linadd(hydro.hn[0][ipH2p]*h.hii*HydroLines[0][ipH2p][ipH1s].Aul*
	  hydro.dstfe2lya*HydroLines[0][ipH2p][ipH1s].EnergyErg , 1216 , "Fe 2" , 'i' ,' ',' ');


#	ifdef DEBUG_FUN
	fputs( " <->LineSet1()\n", debug_fp );
#	endif
	return;
}

