/*ParseAtomHeLike parse parameters off the helium command */
#include "cddefines.h"
#include "taulines.h"
#include "hydrogenic.h"
#include "heclon.h"
#include "parse.h"

void ParseAtomHeLike(char *chCard )
{
	long int i, 
		nHe, 
		ipZ;
	int lgEOL;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseAtomHeLike()\n", debug_fp );
#	endif


	if( lgMatch("COLL",chCard) )
	{
		/* turn off helium atom collisions */
		if( lgMatch(" OFF",chCard) )
		{
			heclon.lgHeClON = FALSE;
		}
		else
		{
			heclon.lgHeClON = TRUE;
		}

	}

	else if( lgMatch("LEVE",chCard) )
	{
		/* number of levels for helium iso-sequence */
		/* there are two options here,
		 * when keyword ELEMENT appears, scan off element name and change levels only for
		 * that one.
		 * when there is no ELEMENT then set all he-like levels to same number */

		/* NB - not possible to DECREASE number of levels in HeI itself */

		/* lgHydroMalloc is FALSE at start of calculation, set true when space 
		 * allocated for the hydrogen and helium lines.  Once done we must ignore all 
		 * future changes in the number of levels */
		if( !lgHydroMalloc )
		{
			i = 5;
			nHe = (long int)FFmtRead(chCard,&i,76,&lgEOL);

			if( lgEOL )
			{
				/* must be a number, or a key, either large (or limit) or small
				 * if no number but LARGER or SMALL then set to preset number */
				if( lgMatch("LARG",chCard) )
				{
					/* there is no real limit, but this includes n=3 */
					nHe = 11;
				}

				/* this is small or compact keyword */
				else if( lgMatch("SMAL",chCard) || lgMatch("COMP",chCard) )
				{
					/* a small atom only has n=2 */
					nHe = 5;
				}

				/* this is limit keyword, sets to largest possible value */
				else
				{
					/* punch out if no number */
					NoNumb(chCard);
				}

			}
			else if( nHe < 5 )
			{
				fprintf( ioQQQ, " cannot have fewer than 5 levels, the requested number was %li\n" ,
					nHe  );
				fprintf( ioQQQ, " Sorry.\n" );
				puts( "[Stop in ParseAtomHeLike]" );
				cdEXIT(1);
			}

			/* we now have the desired number of levels, and it has been fully qualified. 
			 * now check whether ELEMENT keyword is on line, if not then set all to
			 * the number, if so then only element in use */
			if( lgMatch("ELEM",chCard) )
			{
				/* check which element is on the line, ipZ is atomic number on C scale */
				GetElem(chCard,&ipZ);
				/* only set number of levels for this one element */
				if( ipZ==1 && nHe < nHeLevels[ipZ] ) 
				{
					fprintf( ioQQQ, " I cannot reduce the number of levels of Helium itself below %li\n" ,
						nHeLevels[ipZ]  );
					fprintf( ioQQQ, " Sorry.\n" );
					puts( "[Stop in ParseAtomHeLike]" );
					cdEXIT(1);
				}
				else
				{
					nHeLevels[ipZ] = nHe;
				}
			}
			else
			{
				/* keyword element did not appear, so set all he-like species to this number,
				 * note that Helium itself is not included in this loop - it is only changed
				 * by explicitly requesting a change */
				for( i=2; i<LIMELM; ++i )
				{
					nHeLevels[i] = nHe;
				}
			}

		}

	}
	else
	{
		fprintf( ioQQQ, " There should have been a keyword - STOP\n" );
		puts( "[Stop in hatom]" );
		cdEXIT(1);
	}


#	ifdef DEBUG_FUN
	fputs( " <->ParseAtomHeLike()\n", debug_fp );
#	endif
	return;
}

