/*ParseIonPar parse the ionization parameter command */
#include <string.h>
#include "cddefines.h"
#include "incidspec.h"
#include "varypar.h"
#include "radius.h"
#include "input.h"
#include "toomancon.h"
#include "parse.h"

void ParseIonPar(long int *nqh, 
  char *chCard,
  float *ar1)
{
	int lgEOL;
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>ParseIonPar()\n", debug_fp );
#	endif

	/*  check not too many continua */
	if( *nqh >= LIMSPC )
	{
		TooManCon();
	}

	/* say that continuum is per unit area, ionization parameter*/
	strcpy( spnorm.chRSpec[*nqh], "SQCM" );
	if( chCard[0] == 'I' )
	{
		/* this is the usual ionization parameter, U */
		strcpy( spnorm.chSpNorm[*nqh], "IONI" );
	}
	else if( chCard[0] == 'X' )
	{
		/* the X-Ray ionization parameter, xi */
		strcpy( spnorm.chSpNorm[*nqh], "IONX" );
	}
	else
	{
		fprintf(ioQQQ," ParseIonPar hit chCard insanity.\n");
		puts( "[Stop in ParseIonPar]" );
		cdEXIT(1);
	}

	/* get the ionization parameter*/
	i = 5;
	IncidSpec.totpow[*nqh] = (float)FFmtRead(chCard,&i,76,&lgEOL);
	if( lgEOL )
	{
		NoNumb(chCard);
	}

	/* vary option */
	if( VaryPar.lgVarOn )
	{
		if( chCard[0] == 'I' )
		{
			/* this is the usual ionization parameter, U */
			strcpy( VaryPar.chVarFmt[VaryPar.nparm], "IONIZATION PARAMETER=%f" );
		}
		else if( chCard[0] == 'X' )
		{
			/* the X-Ray ionization parameter, xi */
			strcpy( VaryPar.chVarFmt[VaryPar.nparm], "XI=%f" );
		}
		else
		{
			fprintf( ioQQQ, " Insanity in detection which ionization parameter.\n" );
			puts( "[Stop in ParseIonPar]" );
			cdEXIT(1);
		}
		/* pointer to where to write */
		VaryPar.nvfpnt[VaryPar.nparm] = input.nRead;
		VaryPar.vparm[0][VaryPar.nparm] = IncidSpec.totpow[*nqh];
		VaryPar.vincr[VaryPar.nparm] = 0.5;
		VaryPar.nvarxt[VaryPar.nparm] = 1;
		++VaryPar.nparm;
	}

	/* set R to large value if U specified. */
	if( radius.Radius == 0. )
	{
		*ar1 = (float)radius.rdfalt;
		radius.Radius = pow(10.,*ar1);
	}
	/* increment nmber of specifications of continuum intensities, */
	++*nqh;

#	ifdef DEBUG_FUN
	fputs( " <->ParseIonPar()\n", debug_fp );
#	endif
	return;
}
