/*PrtLinePres print line radiation pressures for current conditions */
#include <string.h>
#include <float.h>
#include <limits.h>
#include "cddefines.h"
#include "pressure.h"
#include "taulines.h"
#include "chionlbl.h"
#include "iwavlen.h"
#include "widthline.h"
#include "prtlinepres.h"
/* this is limit to number of lines we can store at one time */
#define	NLINE	20
/* faintest pressure we will bother with */
#define	THRESH	0.05

void PrtLinePres(void)
{
	char chLab[NLINE][5];
	long int i, 
	  ip, 
	  iwl[NLINE];
	double RadPres1, 
	  frac[NLINE];

#	ifdef DEBUG_FUN
	fputs( "<+>PrtLinePres()\n", debug_fp );
#	endif

	/* this routine only called if printout of contributors to line
	 * radiation pressure is desired */

	/* compute radiation pressure due to special H and He lines */
	ip = 0;

	for(i=0; i<NLINE; ++i)
	{
		frac[i] = DBL_MAX;
		iwl[i] = LONG_MAX;
	}

	/* this wil be H L alpha */
	strcpy( chLab[ip] , "Lya " );
	frac[ip] = pressure.HLalpha/MAX2(SMALLFLOAT,pressure.RadPres);
	iwl[ip] = 1216;
	++ip;

	if( pressure.RadPres > 1e-30 )
	{
		/* line radiation pressure from large set of level 1 lines */
		for( i=1; i <= nLevel1; i++ )
		{
			if( TauLines[i].PopHi > 1e-30 )
			{
				RadPres1 = 5.551e-2*(powi(TauLines[i].EnergyWN/
				  1.e6,4))*(TauLines[i].PopHi/TauLines[i].gHi)/
				  (TauLines[i].PopLo/TauLines[i].gLo)*
				  WidthLine(&TauLines[i])*2.;
			}
			else
			{
				RadPres1 = 0.;
			}

			if( RadPres1 > pressure.RadPres*THRESH )
			{
				char chUnits , chShift;
				iwl[ip] = iWavLen(&TauLines[i], &chUnits , &chShift);

				/* put null terminated line label into chLab using line array*/
				chIonLbl(chLab[ip], &TauLines[i]);
				frac[ip] = RadPres1/pressure.RadPres;

				ip = MIN2((long)NLINE-1,ip+1);
			}
		}

		for( i=0; i < nWindLine; i++ )
		{
			if( TauLine2[i].PopHi > 1e-30 )
			{
				RadPres1 = 5.551e-2*(powi(TauLine2[i].EnergyWN/
				  1.e6,4))*(TauLine2[i].PopHi/TauLine2[i].gHi)/
				  (TauLine2[i].PopLo/TauLine2[i].gLo)*
				  WidthLine(&TauLine2[i])*2.;
			}
			else
			{
				RadPres1 = 0.;
			}

			if( RadPres1 > pressure.RadPres*THRESH )
			{
				char chUnits , chShift;
				iwl[ip] = iWavLen(&TauLine2[i], &chUnits , &chShift);

				/* put null terminated line label into chLab using line array*/
				chIonLbl(chLab[ip], &TauLine2[i]);
				frac[ip] = RadPres1/pressure.RadPres;

				ip = MIN2((long)NLINE-1,ip+1);
			}
		}

		for( i=0; i < nCORotate; i++ )
		{
			if( C12O16Rotate[i].PopHi > 1e-30 )
			{
				RadPres1 = 5.551e-2*(powi(C12O16Rotate[i].EnergyWN/
				  1.e6,4))*(C12O16Rotate[i].PopHi/C12O16Rotate[i].gHi)/
				  (C12O16Rotate[i].PopLo/C12O16Rotate[i].gLo)*
				  WidthLine(&C12O16Rotate[i])*2.;
			}
			else
			{
				RadPres1 = 0.;
			}

			if( RadPres1 > pressure.RadPres*THRESH )
			{
				char chUnits , chShift;
				iwl[ip] = iWavLen(&C12O16Rotate[i], &chUnits , &chShift);

				/* put null terminated line label into chLab using line array*/
				chIonLbl(chLab[ip], &C12O16Rotate[i]);
				frac[ip] = RadPres1/pressure.RadPres;

				ip = MIN2((long)NLINE-1,ip+1);
			}
		}

		for( i=0; i < nCORotate; i++ )
		{
			if( C13O16Rotate[i].PopHi > 1e-30 )
			{
				RadPres1 = 5.551e-2*(powi(C13O16Rotate[i].EnergyWN/
				  1.e6,4))*(C13O16Rotate[i].PopHi/C13O16Rotate[i].gHi)/
				  (C13O16Rotate[i].PopLo/C13O16Rotate[i].gLo)*
				  WidthLine(&C13O16Rotate[i])*2.;
			}
			else
			{
				RadPres1 = 0.;
			}

			if( RadPres1 > pressure.RadPres*THRESH )
			{
				char chUnits , chShift;
				iwl[ip] = iWavLen(&C13O16Rotate[i], &chUnits , &chShift);

				/* put null terminated line label into chLab using line array*/
				chIonLbl(chLab[ip], &C13O16Rotate[i]);
				frac[ip] = RadPres1/pressure.RadPres;

				ip = MIN2((long)NLINE-1,ip+1);
			}
		}

		/* now print them */
		fprintf( ioQQQ, " P(Lines):" );
		for( i=0; i < ip; i++ )
		{
			fprintf( ioQQQ, "%8.2f %4.4s%5ld,", 
				frac[i], chLab[i], iwl[i] );
		}

		/* possible that dominant contributor not in above, in which case ip is 0
		 * and we will prin contributor that came from TotalPressure */
		if( ip <1 )
		{
			fprintf( ioQQQ, " %s ", pressure.chLineRadPres);
		}
		fprintf( ioQQQ, "\n" );
	}

#	ifdef DEBUG_FUN
	fputs( " <->PrtLinePres()\n", debug_fp );
#	endif
	return;
}

