/*PrtZone print out individual zone results */
#include <string.h>
#include "cddefines.h"
/* following is quantum heating punch io that will need to be removed -
 * now it punches at all times */
#include "physconst.h"
#include "nhe3lvl.h"
#include "nhe1lvl.h"
#include "grainvar.h"
#include "nh.h"
#include "pressure.h"
#include "wind.h"
#include "converge.h"
#include "trace.h"
#include "called.h"
#include "printit.h"
#include "heating.h"
#include "he1bn.h"
#include "he3n.h"
#include "he3lines.h"
#include "secondaries.h"
#include "opacity.h"
#include "he3bn.h"
#include "phe1lv.h"
#include "photn2.h"
#include "colden.h"
#include "filfac.h"
#include "hmi.h"
#include "rfield.h"
#include "thermal.h"
#include "heat.h"
#include "compton.h"
#include "h.h"
#include "he.h"
#include "radius.h"
#include "phycon.h"
#include "hevmolec.h"
#include "abundances.h"
#include "fluct.h"
#include "ionfracs.h"
#include "hydrogenic.h"
#include "ionrange.h"
#include "elementnames.h"
#include "prtlinepres.h"
#include "pop371.h"
#include "prtzone.h"
/*PrtHydro print out hydrogenic populations */
static void PrtHydro( long ipZ );

void PrtZone(void)
{
	char chField7[8];
	char chLet, 
	  chQHMark;
	long int i, 
	  ishift, 
	  nelem ,
	  nd;
	double cdif, 
	  coninc, 
	  dmidle, 
	  fac, 
	  hatmic, 
	  rmidle;

#	ifdef DEBUG_FUN
	fputs( "<+>PrtZone()\n", debug_fp );
#	endif

	if( thermal.lgUnstable )
	{
		chLet = 'u';
	}
	else
	{
		chLet = ' ';
	}

	/* middle of zone for printing */
	rmidle = radius.Radius - radius.drad*0.5*radius.dRadSign;
	dmidle = radius.depth - radius.drad*0.5;

	/* option to print single line when quiet but tracing convergence
	 * with "trace convergence" command */
	if( called.lgTalk || trace.lgTrConvg )
	{
		/* print either ####123 or ###1234 */
		if( nzone <= 999 )
		{
			sprintf( chField7, "####%3ld", nzone );
		}
		else
		{
			sprintf( chField7, "###%4ld", nzone );
		}

		fprintf(ioQQQ, " %7.7s %cTe:",chField7, chLet);
		PrintE93(ioQQQ,phycon.te);
		fprintf(ioQQQ," Hden:");
		PrintE93(ioQQQ,phycon.hden);
		fprintf(ioQQQ," Ne:");
		PrintE93(ioQQQ,phycon.eden);
		fprintf(ioQQQ," R:");
		PrintE93(ioQQQ,rmidle );
		fprintf(ioQQQ," R-R0:");
		PrintE93(ioQQQ,dmidle);
		fprintf(ioQQQ," dR:");
		PrintE93(ioQQQ,radius.drad);
		fprintf(ioQQQ," NTR:%3ld Htot:",conv.nPres2Ioniz);
		PrintE93(ioQQQ,heat.htot);
		fprintf(ioQQQ," T912:");
		fprintf(ioQQQ,PrintEfmt("%9.2e",opac.TauAbsGeo[0][nh.ipHn[0][ipH1s]-1] ));
		fprintf(ioQQQ,"###\n");
		  
		if( trace.lgTrConvg )
		{
			fprintf( ioQQQ, " H:%10.2e%10.2e He:%10.2e%10.2e%10.2e trip%10.2e\n", 
			  h.hi/(h.hi + h.hii), 
			  h.hii/(h.hi + h.hii), 
			  xIonFracs[1][ipHELIUM]/xIonFracs[0][ipHELIUM], 
			  xIonFracs[2][ipHELIUM]/xIonFracs[0][ipHELIUM], 
			  xIonFracs[3][ipHELIUM]/xIonFracs[0][ipHELIUM], 
			  he.hei3/xIonFracs[0][ipHELIUM] );
		}
	}

	/* now return if not talking */
	if( !called.lgTalk || trace.lgTrConvg )
	{ 
#		ifdef DEBUG_FUN
		fputs( " <->PrtZone()\n", debug_fp );
#		endif
		return;
	}

	/* lgDenFluc set to true in zero, only false when variable abundances
	 * lgAbTaON set true when element table used */
	if( !fluct.lgDenFluc || abundances.lgAbTaON )
	{
		fprintf( ioQQQ, " Abun:" );
		for( i=1; i <= LIMELM; i++ )
		{
			fprintf( ioQQQ,PrintEfmt("%8.1e", xIonFracs[0][i-1] ));
		}
		fprintf( ioQQQ, "\n" );
	}

	/*-------------------------------------------------
	 * print wind parameters if windy model */
	fac = wind.windv*phycon.hden*radius.r1r0sq;
	if( wind.windv != 0. )
	{
		if( wind.AccelTot == 0. )
		{
			fac = 1.;
		}
		else
		{
			fac = wind.AccelTot;
		}
		/*fprintf( ioQQQ, " WIND; V:%9.3ekm/s G:%10.3e Accel:%10.3e Fr(cont):%6.3f Fr(line):%6.3f Fr(dP):%6.3f\n", 
		  wind.windv/1e5, -wind.agrav, wind.AccelTot, wind.AccelCont/
		  fac, wind.AccelLine/fac, wind.AccelPres/fac );*/
		fprintf( ioQQQ, " WIND; V:");
		fprintf(ioQQQ,PrintEfmt("%10.3e",wind.windv/1e5));
		fprintf( ioQQQ, "km/s G:");
		fprintf(ioQQQ,PrintEfmt("%9.2e",-wind.agrav));
		fprintf( ioQQQ, " Accel:");
		fprintf(ioQQQ,PrintEfmt("%9.2e",wind.AccelTot));
		fprintf( ioQQQ, " Fr(cont):%6.3f Fr(line):%6.3f Fr(dP):%6.3f\n", 
		  wind.AccelCont/ fac, wind.AccelLine/fac, wind.AccelPres/fac );
	}

	/* print line with radiation pressure if significant */
	if( wind.windv != 0. || pressure.pbeta > .05 )
	{
		PrtLinePres();
	}

	/*---------------------------------------------------- */
	hatmic = (h.hi + h.hii)/(h.hi + h.hii + hmi.hminus + 2.*(hmi.htwo + 
	  hmi.h2plus) + 3*hmi.h3plus);

	fprintf( ioQQQ, " Hydrogen     ");
	fprintf(ioQQQ,PrintEfmt("%9.2e",h.hi/(h.hi + h.hii)));
	fprintf(ioQQQ,PrintEfmt("%9.2e",h.hii/(h.hi + h.hii)));
	fprintf( ioQQQ, " H+o/Hden");
	fprintf(ioQQQ,PrintEfmt("%9.2e",hatmic ));
	fprintf(ioQQQ,PrintEfmt("%9.2e",hmi.hminus/phycon.hden ));
	fprintf( ioQQQ, " H-    H2");
	fprintf(ioQQQ,PrintEfmt("%9.2e",hmi.htwo/phycon.hden));
	fprintf(ioQQQ,PrintEfmt("%9.2e",hmi.h2plus/phycon.hden));
	fprintf( ioQQQ, " H2+ HeH+");
	fprintf(ioQQQ,PrintEfmt("%9.2e",hmi.hehp/phycon.hden));
	fprintf( ioQQQ, " Ho+ ColD");
	fprintf(ioQQQ,PrintEfmt("%9.2e",coldenCom.colden[IPCHI-1]));
	fprintf(ioQQQ,PrintEfmt("%9.2e",coldenCom.colden[IPCHII-1]));
	fprintf( ioQQQ, "\n");

	/* print departure coef if desired */
	if( printit.lgPrtBN )
	{
		fprintf( ioQQQ, " Hydrogen     " );
		fprintf(ioQQQ,PrintEfmt("%9.2e",  hydro.hbn[0][ipH1s]));
		fprintf(ioQQQ,PrintEfmt("%9.2e",  1.));
		fprintf( ioQQQ, " H+o/Hden");
		fprintf(ioQQQ,PrintEfmt("%9.2e", (h.hi + h.hii)/phycon.hden));
		fprintf(ioQQQ,PrintEfmt("%9.2e", hmi.hmidep));
		fprintf( ioQQQ, " H-    H2");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hmi.h2dep));
		fprintf( ioQQQ, "      H2+");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hmi.h2pdep));
		fprintf( ioQQQ, "      H3+");
		fprintf(ioQQQ,PrintEfmt("%9.2e",hmi.h3pdep));
		fprintf( ioQQQ, "\n" );
	}

	if( printit.lgPrintHeating )
	{
		fprintf( ioQQQ, "              ");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[0][0]/heat.htot));
		fprintf( ioQQQ,"         ");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[15][0]/heat.htot));
		fprintf( ioQQQ,"         ");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[16][0]/heat.htot));
		fprintf( ioQQQ,"\n");
	}

	/* temperature correspoding to radiation fields */
	coninc = 0.;
	cdif = 0.;
	for( i=0; i < rfield.nflux; i++ )
	{
		coninc += rfield.flux[i]*(rfield.anu[i]*EN1RYD);
		cdif += (rfield.ConOutNoInter[i] + rfield.outlin[i] + 
			rfield.ConOutInter[i]+rfield.ConOutRecInter[i])* (rfield.anu[i]*EN1RYD);
	}
	coninc = pow(coninc/2.997925e10/7.56464e-15,0.25);
	cdif = pow(cdif/2.997925e10/7.56464e-15,0.25);


	/* print info on hydrogen level populations */
	PrtHydro(0);

	if( printit.lgPrintHeating )
	{
		fprintf( ioQQQ, "              ");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[1][0]/heat.htot ));
		fprintf( ioQQQ, "         ");
		fprintf(ioQQQ,PrintEfmt("%9.2e", 0. ));
		fprintf( ioQQQ, " BoundCom");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[18][0]/ heat.htot));
		fprintf( ioQQQ, "   Extra:");
		fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[20][0]/heat.htot));
		fprintf( ioQQQ, "   Pairs:");
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[21][0]/ heat.htot ));
		fprintf( ioQQQ,"  H-lines\n");
	}

	/* Helium */
	if( abundances.lgElmtOn[1] )
	{
		fprintf( ioQQQ, " Helium       " );
		for( i=1; i < 4; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i][ipHELIUM]/xIonFracs[0][ipHELIUM]) );
		}
		/*fprintf( ioQQQ, "%10.3e%10.3e He I 2S3 %10.3e Comp H, C%10.3e\n", 
		  Compton.cmheat, Compton.cmcool*phycon.te, filfac.FillFac, 
		  photn2Com.photn2 );*/
		fprintf( ioQQQ, " He I2SP3");
		fprintf(ioQQQ,PrintEfmt("%9.2e", he.hei3/xIonFracs[0][ipHELIUM] ));
		fprintf(ioQQQ,PrintEfmt("%9.2e", he3lines.p2p/xIonFracs[0][ipHELIUM] ));
		fprintf( ioQQQ, " Comp H,C");
		fprintf(ioQQQ,PrintEfmt("%9.2e", Compton.cmheat ));
		fprintf(ioQQQ,PrintEfmt("%9.2e",  Compton.cmcool*phycon.te));
		fprintf( ioQQQ , " Fill Fac");
		fprintf(ioQQQ,PrintEfmt("%9.2e", filfac.FillFac));
		fprintf( ioQQQ , " Gam1/tot");
		fprintf(ioQQQ,PrintEfmt("%9.2e", photn2Com.photn2));
		fprintf( ioQQQ, "\n");

		/* option to print departure coef */
		if( printit.lgPrtBN )
		{
			fprintf( ioQQQ, " Helium       " );
			fprintf(ioQQQ,PrintEfmt("%9.2e", he1bnCOM.he1bn[0]));
			fprintf(ioQQQ,PrintEfmt("%9.2e", hydro.hbn[1][ipH1s]));
			fprintf(ioQQQ,PrintEfmt("%9.2e", 1.));
			fprintf( ioQQQ, " He I 2S3");
			fprintf(ioQQQ,PrintEfmt("%9.2e", he3bnCom.he3bn[0] ));

			/*fprintf( ioQQQ, "%10.3e%10.3e He I 2S3 %10.3e Comp H, C%10.3e\n", 
			  Compton.cmheat, Compton.cmcool*phycon.te, filfac.FillFac, 
			  photn2Com.photn2 );*/
			fprintf( ioQQQ, " Comp H,C");
			fprintf(ioQQQ,PrintEfmt("%9.2e", Compton.cmheat ));
			fprintf(ioQQQ,PrintEfmt("%9.2e", Compton.cmcool*phycon.te ));
			fprintf( ioQQQ , " Fill Fac");
			fprintf(ioQQQ,PrintEfmt("%9.2e", filfac.FillFac ));
			fprintf( ioQQQ , " Gam1/tot");
			fprintf(ioQQQ,PrintEfmt("%9.2e", photn2Com.photn2));
			fprintf( ioQQQ, "\n");
		}

		/* print heating from He (and others) if desired
		 * entry "lines" is induced line heating
		 * 1,12 ffheat:  2,3 he triplets, 1,20 compton */
		if( printit.lgPrintHeating )
		{
			/*fprintf( ioQQQ, "            %10.3e%10.3e    Lines:%10.2e%10.2e  Compton:%10.3e FF Heatig%10.3e\n", 
			  HeatingCom.heating[0][1]/heat.htot, HeatingCom.heating[1][1]/
			  heat.htot, HeatingCom.heating[22][0]/heat.htot, HeatingCom.heating[2][1]/
			  heat.htot, HeatingCom.heating[19][0]/heat.htot, HeatingCom.heating[11][0]/
			  heat.htot );*/
			fprintf( ioQQQ, "              ");
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[0][1]/heat.htot));
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[1][1]/heat.htot));
			fprintf( ioQQQ, "   Lines:");
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[22][0]/heat.htot));
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[2][1]/heat.htot));
			fprintf( ioQQQ, " Compton:");
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[19][0]/heat.htot));
			fprintf( ioQQQ, " FFHeatig");
			fprintf(ioQQQ,PrintEfmt("%9.2e",HeatingCom.heating[11][0]/heat.htot));
			fprintf( ioQQQ, "\n");
		}

		/* helium singlets and triplets */
		fac = xIonFracs[2][ipHELIUM]/xIonFracs[0][ipHELIUM];
		fprintf( ioQQQ, " He singlet   " );
		for( i=0; i < 6; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", phe1lv.he1n[i]*fac ));
		}
		fprintf( ioQQQ, " He tripl" );
		for( i=0; i < NHE3LVL; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", he3nCom.he3n[i]*fac ));
		}
		fprintf( ioQQQ, "\n" );

		/* option to print departure coef */
		if( printit.lgPrtBN )
		{
			fprintf( ioQQQ, " He singlet   " );
			for( i=0; i < (NHE1LVL + 1); i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", he1bnCOM.he1bn[i] ));
			}
			fprintf( ioQQQ, "\n" );

			fprintf( ioQQQ, " He triplet   " );
			for(i=0; i < NHE3LVL; i++)
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", he3bnCom.he3bn[i] ));
			}
			fprintf( ioQQQ, "\n" );
		}

		/* print helium ion level populations and departure coef */
		PrtHydro(1);
	}


	fprintf( ioQQQ , "               Texc(La)");
	fprintf(ioQQQ,PrintEfmt("%9.2e",  hydro.TexcLya ));
	fprintf( ioQQQ , " T(contn)");
	fprintf(ioQQQ,PrintEfmt("%9.2e",  coninc ));
	fprintf( ioQQQ , " T(diffs)");
	fprintf(ioQQQ,PrintEfmt("%9.2e",  cdif ));
	/* print the radiation to gas pressure */
	fprintf( ioQQQ , " PRad/Gas");
	fprintf(ioQQQ,PrintEfmt("%9.2e", pressure.pbeta ));
	fprintf( ioQQQ, "\n" );

	for( nd=0; nd < gv.nBin; nd++ )
	{
		/*  Change things so the quantum heated dust species are marked with an
		 *  asterisk just after the name (K Volk)
		 *  added QHMARK here and in the write statement */
		chQHMark = (char)(( gv.bin[nd]->lgQHeat && gv.bin[nd]->lgUseQHeat ) ? '*' : ' ');
		fprintf( ioQQQ, "%-12.12s%c  DustTemp",gv.bin[nd]->chDstLab, chQHMark);
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->tedust));
		fprintf( ioQQQ, " Pot Volt");
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->dstpot*EVRYD));
		fprintf( ioQQQ, " Chrg (e)");
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->DustZ));
		fprintf( ioQQQ, " drf cm/s");
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->DustDftVel));
		fprintf( ioQQQ, " Heating:");
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->GasHeatPhotoEl));
		fprintf( ioQQQ, " Frac tot");
		fprintf(ioQQQ,PrintEfmt("%9.2e", gv.bin[nd]->GasHeatPhotoEl/heat.htot));
		fprintf( ioQQQ, "\n" );
	}
	/* >>chng 00 apr 20, moved punch-out of quantum heating data to qheat(), by PvH */

	/* gas pressure, pressure due to incident radiation field, rad accel */
	fprintf( ioQQQ, " Pressure      NgasTgas");
	fprintf(ioQQQ,PrintEfmt("%9.2e", pressure.GasPres/BOLTZMANN));
	fprintf( ioQQQ, " P(total)");
	fprintf(ioQQQ,PrintEfmt("%9.2e", pressure.RadPres + pressure.GasPres));
	fprintf( ioQQQ, " P( gas )");
	fprintf(ioQQQ,PrintEfmt("%9.2e", pressure.GasPres));
	fprintf( ioQQQ, " P(Radtn)");
	fprintf(ioQQQ,PrintEfmt("%9.2e", pressure.RadPres));
	fprintf( ioQQQ, " Rad accl");
	fprintf(ioQQQ,PrintEfmt("%9.2e", wind.AccelTot));
	fprintf( ioQQQ, " ForceMul");
	fprintf(ioQQQ,PrintEfmt("%9.2e", wind.fmul));
	fprintf( ioQQQ, "\n" );

	/* heavy element molecules */
	if( hevmolec.hevmol[IPCO-1] > 0. )
	{
		fprintf( ioQQQ, " Molecules     CH/Ctot:");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPCH-1]/xIonFracs[0][5]));
		fprintf( ioQQQ, " CH+/Ctot");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPCHP-1]/xIonFracs[0][5]));
		fprintf( ioQQQ, " CO/Ctot:");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPCO-1]/xIonFracs[0][5]));
		fprintf( ioQQQ, " CO+/Ctot");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPCOP-1]/xIonFracs[0][5]));
		fprintf( ioQQQ, " H2O/Otot");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPH2O-1]/xIonFracs[0][7]));
		fprintf( ioQQQ, " OH/Ototl");
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPOH-1]/xIonFracs[0][7]));
		fprintf( ioQQQ, "\n");
	}

	/* Lithium, Beryllium */
	if( abundances.lgElmtOn[2] || abundances.lgElmtOn[3] )
	{
		fprintf( ioQQQ, " Lithium      " );
		for( i=1; i <= 4; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i][2]/MAX2(1e-35,xIonFracs[0][2]) ));
		}
		fprintf( ioQQQ, " Berylliu" );
		for( i=1; i <= 5; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e",  xIonFracs[i][3]/MAX2(1e-35,xIonFracs[0][3])) );
		}
		fprintf( ioQQQ, "\n" );

		/* option to print heating due to these stages*/
		if( printit.lgPrintHeating )
		{
			fprintf( ioQQQ, "              " );
			for( i=0; i < 3; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e",  HeatingCom.heating[i][2]/ heat.htot) );
			}
			fprintf( ioQQQ, "                    " );

			for( i=0; i < 4; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[i][3]/heat.htot ));
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* Boron */
	if( abundances.lgElmtOn[4] )
	{
		fprintf( ioQQQ, " Boron        " );
		for( i=1; i <= 6; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i][4]/MAX2(1e-35,xIonFracs[0][4]) ));
		}
		fprintf( ioQQQ, "\n" );

		/* option to print heating*/
		if( printit.lgPrintHeating )
		{ 
			fprintf( ioQQQ, "              " );
			for( i=0; i < 5; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[i][4]/heat.htot ));
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* Carbon */
	if( abundances.lgElmtOn[5] )
	{
		fprintf( ioQQQ, " Carbon       " );
		for( i=1; i <= 7; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i][5]/MAX2(1e-35,xIonFracs[0][5])) );
		}
		/* some molecules trail the line */
		fprintf( ioQQQ, " H2O+/O  " );
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPH2OP-1]/MAX2(1e-35,xIonFracs[0][7]) ));
		fprintf( ioQQQ, " OH+/Otot" );
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPOHP-1]/ MAX2(1e-35,xIonFracs[0][7]) ));
		/* print extra heating, normally zero */
		fprintf( ioQQQ, " Hex(tot)" );
		fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[20][0] ));
		fprintf( ioQQQ, "\n" );

		/* option to print heating*/
		if( printit.lgPrintHeating )
		{
			fprintf( ioQQQ, "              " );
			for( i=0; i < 6; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[i][5]/ heat.htot) );
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* Nitrogen */
	if( abundances.lgElmtOn[6] )
	{
		fprintf( ioQQQ, " Nitrogen     " );
		for( i=1; i <= 8; i++ )
		{
			fprintf(ioQQQ,PrintEfmt("%9.2e",xIonFracs[i][6]/ MAX2(1e-35,xIonFracs[0][6]) ));
		}
		fprintf( ioQQQ, " O2/Ototl" );
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPOTWO-1]/MAX2(1e-35,xIonFracs[0][7])));
		fprintf( ioQQQ, " O2+/Otot" );
		fprintf(ioQQQ,PrintEfmt("%9.2e", hevmolec.hevmol[IPO2P-1]/ MAX2(1e-35,xIonFracs[0][7]) ));
		fprintf( ioQQQ, "\n" );

		/* option to print heating*/
		if( printit.lgPrintHeating )
		{
			fprintf( ioQQQ, "              " );
			for( i=0; i < 7; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[i][6]/ heat.htot ));
			}
			fprintf( ioQQQ, "\n" );
		}
	}

	/* now print rest of elements inside loops */

	/* Oxygen through Magnesium */
	for( nelem=7; nelem < 12; ++nelem )
	{
		if( abundances.lgElmtOn[nelem] )
		{
			/* print the element name and amount of shift */
			fprintf( ioQQQ, " %10.10s   ", elementnames.chElementName[nelem]);

			for( i=1; i <= nelem+2; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i][nelem]/xIonFracs[0][nelem] ));
			}
			if( nelem==7 )
			{
				/* print secondary ionization rate for oxygen */
				fprintf( ioQQQ, " sec ion:" );
					fprintf(ioQQQ,PrintEfmt("%9.2e", Secondaries.csupra) );
			}
			fprintf( ioQQQ, "\n" );

			/* print heating but only if needed */
			if( printit.lgPrintHeating )
			{
				fprintf( ioQQQ, "              " );
				for( i=1; i <= nelem+2; i++ )
				{
					fprintf(ioQQQ,PrintEfmt("%9.2e", HeatingCom.heating[i-1][nelem]/heat.htot ));
				}
				fprintf( ioQQQ, "\n" );
			}
		}
	}

	/* Aluminium through Zinc */
	for( nelem=12; nelem < LIMELM; ++nelem )
	{
		if( abundances.lgElmtOn[nelem] )
		{
			/* number of ionization stages to print across the page */
			/*@-redef@*/
			enum {LINE=13};
			/*@+redef@*/
			ishift = MAX2(0,IonRange.IonHigh[nelem]-LINE+1);

			/* print the element name and amount of shift */
			fprintf( ioQQQ, " %10.10s%2ld ", elementnames.chElementName[nelem],ishift );

			for( i=1; i <= LINE; i++ )
			{
				fprintf(ioQQQ,PrintEfmt("%9.2e", xIonFracs[i+ishift][nelem]/xIonFracs[0][nelem]) );
			}
			fprintf( ioQQQ, "\n" );

			/* print heating but only if needed */
			if( printit.lgPrintHeating )
			{
				fprintf( ioQQQ, "              " );
				for( i=1; i <= LINE; i++ )
				{
					fprintf(ioQQQ,
						PrintEfmt("%9.2e", HeatingCom.heating[i+ishift-1][nelem]/heat.htot ));
				}
				fprintf( ioQQQ, "\n" );
			}
		}
	}

	/* call FeII print routine if large atom is turned on */
	if( FeII.lgFeIION )
	{
		FeIIPrint();
	}

#	ifdef DEBUG_FUN
	fputs( " <->PrtZone()\n", debug_fp );
#	endif
	return;
}

/*PrtHydro print out hydrogenic populations */
static void PrtHydro( long ipZ )
{
	long i ,
		limit ;

#	ifdef DEBUG_FUN
	fputs( "<+>PrtHydro()\n", debug_fp );
#	endif

	/* lgPrintHLevPops is set true with print hydrogenic command, usually false
	 * when set true, also parse in optional number of levels to print, nPrintHLevPops
	 * will be current value of nHLevels[ipZ] if nothing entered, so set levels
	 * before print command */
	if( IonRange.IonHigh[ipZ] != ipZ + 1 || !printit.lgPrintHLevPops )
	{
#		ifdef DEBUG_FUN
		fputs( " <->PrtHydro()\n", debug_fp );
#		endif
		/* return if no hydrogenic species present */
		return;
	}

	/* it is impossible for this to fail since ParsePrint set to at least 3 */
	assert( printit.nPrintHLevPops > 2);

	/* print stright level pop per ionized hydrogen */
	fprintf( ioQQQ, " %2s%2li 1S-12   ",
		elementnames.chElementSym[ipZ],(ipZ+1) );

	limit = MIN2(printit.nPrintHLevPops,12);
	for( i=ipH1s; i <= limit; i++ )
	{
		fprintf( ioQQQ,PrintEfmt("%9.2e",  hydro.hn[ipZ][i] ));
	}
	fprintf(ioQQQ,"\n");

	if( printit.nPrintHLevPops > limit )
	{
		fprintf( ioQQQ, " %2s%2li  rest   ",
		elementnames.chElementSym[ipZ], (ipZ+1) );
		for( i=(limit+1); i <= printit.nPrintHLevPops; i++ )
		{
			fprintf( ioQQQ,PrintEfmt("%9.2e", hydro.hn[ipZ][i] ));
			if( ((i-limit) % 13) == 0 )
			{
				if( i!=printit.nPrintHLevPops )
				{
					/* throw a newline every 14th column */
					fprintf( ioQQQ, "\n              " );
				}
				else
				{
					/* throw a newline every 14th column */
					fprintf( ioQQQ, "\n" );
				}
			}
		}
		if( ((printit.nPrintHLevPops-limit)%14) != 0 )
		{
			/* do not throw another if printit.nPrintHLevPops is multiple of 13 */
			fprintf( ioQQQ, "\n" );
		}
	}

	/* option to print dep coef */
	if( printit.lgPrtBN )
	{
		limit = MIN2(printit.nPrintHLevPops,12);
		/* print this line so that it agrees with level populations above */
		fprintf( ioQQQ, " %2s%2li 1S-12   ",
			elementnames.chElementSym[ipZ],(ipZ+1) );

		for( i=ipH1s; i <= limit; i++ )
		{
			fprintf( ioQQQ,PrintEfmt("%9.2e", hydro.hbn[ipZ][i]) );
		}
		fprintf( ioQQQ, "\n" );

		if( printit.nPrintHLevPops > limit )
		{
			fprintf( ioQQQ, " %2s%2li  rest   ",
			elementnames.chElementSym[ipZ], (ipZ+1) );
			for( i=(limit+1); i <= printit.nPrintHLevPops; i++ )
			{
				fprintf( ioQQQ,PrintEfmt("%9.2e", hydro.hbn[ipZ][i] ));
				if( ((i-limit) % 13) == 0 )
				{
					if( i!=printit.nPrintHLevPops )
					{
						/* throw a newline every 13th column */
						fprintf( ioQQQ, "\n              " );
					}
					else
					{
						/* throw a newline every 13th column */
						fprintf( ioQQQ, "\n" );
					}
				}
			}
			if( ((printit.nPrintHLevPops-limit)%14) != 0 )
			{
				/* do not throw another if printit.nPrintHLevPops is multiple of 13 */
				fprintf( ioQQQ, "\n" );
			}
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->PrtHydro()\n", debug_fp );
#	endif
	return;
}
