/*RT_OTS compute diffuse fields due to helium atom, ion, triplets, metal recombination,
 * called by ionize */
#include "cddefines.h"
#include "nhe3lvl.h"
#include "taulines.h"
#include "nhe1lvl.h"
#include "opacity.h"
#include "he3n.h"
#include "phe1lv.h"
#include "he.h"
#include "phycon.h"
#include "heots.h"
#include "he1tau.h"
#include "nhe1.h"
#include "he3lines.h"
#include "d10830.h"
#include "iphe1l.h"
#include "ip626.h"
#include "rt.h"
#include "he3as.h"
#include "sphere.h"
#include "ionfracs.h"
#include "pop371.h"
#include "diffheav.h"
#include "heavy.h"
#include "widlin.h"

void RT_OTS(void)
{
	long int i, 
		ip,
		ipla,
	  j, 
	  nelem,
	  n;
	float 
		diffuse ,
		difflya,
		esc,
	  escfac, 
	  ots,
	  outward;
	float totline,
	  otsfac;
	float otshere; 

#	ifdef DEBUG_FUN
	fputs( "<+>RT_OTS()\n", debug_fp );
#	endif

	/* He I recombination continuum */
	for( n=0; n < 6; n++ )
	{
		otshere = (float)(phe1lv.he1rec[0][n]*(1. - phe1lv.he1rec[2][n])*
		  phycon.eden*he.heii) ;
		if( otshere < 0. )
		{
			fprintf( ioQQQ, " RT_OTS found negative OTS He1 CON, n=%4ld val=%10.1e\n", 
			  n, otshere );
			ShowMe();
			puts( "[Stop in hediff]" );
			cdEXIT(1);
		}
		RTAddOTSCon(otshere,nhe1Com.nhe1[n]);
	}

	/* HeI recombination continua at 1.81 Ryd */
	heots.esc504 = (float)(phe1lv.he1rec[0][0]*phe1lv.he1rec[2][0]*phycon.eden*
	  he.heii);

	/* HeI Ly-alpha He I ly a
	 * escaping part of HeI Ly-alpha lya ly a */
	heots.esc584 = 
		he.heii*phe1lv.he12p*phe1lv.he1mis[0][1]*
		opac.ExpmTau[iphe1lCom.iphe1l[0][1]-1];

	/* do not want to add much outward HeI Lya since it is so important
	 * use inward looking optical depth to decide outward flux */
	otshere = (float)(phe1lv.he12p*he.heii*phe1lv.he1dst[0][1] + he.heii*phe1lv.he12p*
	  phe1lv.he1mis[0][1]*(1.f - opac.ExpmTau[iphe1lCom.iphe1l[0][1]-1]));

	if( otshere < 0. )
	{
		fprintf( ioQQQ, " RT_OTS found negative OTS He1 alhpa,%4i val=%10.1e\n", 
		  1, otshere );
		ShowMe();
		puts( "[Stop in hediff]" );
		cdEXIT(1);
	}
	RTAddOTSLin(otshere,iphe1lCom.iphe1l[0][1] );

	/* all other lines of HeI - tests show this is very small effect */
	for( j=2; j < 7; j++ )
	{
		for( i=0; i <= (j - 1); i++ )
		{
			otshere = phe1lv.he1n[j]*he.heii*phe1lv.he1dst[i][j];
			if( otshere < 0. )
			{
				fprintf( ioQQQ, " RT_OTS found negative OTS He1l%4ld%4ld val=%10.1e\n", 
				  j, i, otshere );
				ShowMe();
				puts( "[Stop in hediff]" );
				cdEXIT(1);
			}
			RTAddOTSLin(otshere,iphe1lCom.iphe1l[i][j]);
		}
	}

	/* HeI 2tripS to ground - one photon decay
	 * this is a major source of ionization for parishii */
	totline = he.heii*he3nCom.he3n[0]*he3as.a2sg;
	if( sphere.lgSphere )
	{
		/* all goes outward or ots */
		escfac = 0.;
	}
	else
	{
		escfac = 0.5f*opac.ExpmTau[ip626.ipHe23sGrnd-1];
	}

	/* number of phots escaping from illuminated face of cloud */
	heots.esc626 = (float)(totline*escfac);

	/* otsfac = 1. - ExpZone(ipHe23sGrnd)
	 * when optically thin, this goes to zero, when thick, to 100% */
	otsfac = 1.f - 2.f*escfac;

	/* >>chng 96 mar 07, above had been 1-escfac, so 50% ots when thin to cont */
	RTAddOTSLin(totline*otsfac,ip626.ipHe23sGrnd);

	/* following is zero when thick to cont, 0.5 when thin */
	outward = MAX2(0.f,1.f-otsfac-escfac);

	/* outward escaping line */
	heots.out626 = (float)(totline*outward);

	/* HeI 10830 OTS */
	RTAddOTSLin(he3lines.p2p*d10830.DR10830,he3lines.ipHe3l[0]);

	/*>>chng 99 feb 17 this code had been in metdif, moved here in cleanup */
	/* all locally destroyed two photon continua of hydrogenic H, HeI, HeII */
	/* >>chng 00 dec 27, remove these and replace with locally destroyed ots continuum,
	 * rfield.ConOTS_local_photons */
#	if 0
	if( hhe2phtots.H12 phtOTS < 0. )
	{
		fprintf( ioQQQ, " metdif found negative OTS CON, H12phtOTS val=%10.1e\n", 
		  hhe2phtots.H12 phtOTS );
		ShowMe();
		puts( "[Stop in metdif]" );
		cdEXIT(1);
	}
	RTAddOTSCon(hhe2phtots.H12 phtOTS,hhe2phtots.ipH12pht);
	{
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG )
		{
			fprintf(ioQQQ," RT_OTS expmtau %e %e\n" , 
				opac.ExpmTau[HydroLines[0][ipH2p][ipH1s].ipCont - 30 ],
				hhe2phtots.H12 phtOTS);
		}
	}

	if( hhe2phtots.He12 phtOTS < 0. )
	{
		fprintf( ioQQQ, " metdif found negative OTS CON, He12 phtOTS val=%10.1e\n", 
		  hhe2phtots.He12 phtOTS );
		ShowMe();
		puts( "[Stop in metdif]" );
		cdEXIT(1);
	}
	RTAddOTSCon(hhe2phtots.He12 phtOTS,hhe2phtots.ipHe12pht);

	if( hhe2phtots.He22 phtOTS < 0. )
	{
		fprintf( ioQQQ, " metdif found negative OTS CON, He22 phtOTS val=%10.1e\n", 
		  hhe2phtots.He22 phtOTS );
		ShowMe();
		puts( "[Stop in metdif]" );
		cdEXIT(1);
	}
	RTAddOTSCon(hhe2phtots.He22 phtOTS,hhe2phtots.ipHe22pht);
#	endif

	/* add recombination continua for elements heavier than He */
	/* nelem = 2 is Li */
	for( nelem=2; nelem < LIMELM; nelem++ )
	{
		/* do not include hydrogenic species in following */
		for( i=0; i < nelem; i++ )
		{
			if( xIonFracs[i+2][nelem] > 0. )
			{
				ip = Heavy.ipHeavy[i][nelem];

				/* this is extinction to face */
				esc = opac.ExpmTau[ip-1];

				/* DiffusHeavy is rad rec to ground, with electron density,
				 * it is evaluated in routine MakeRecomb */
				diffuse = DiffHeav.DiffusHeavy[i][nelem]*xIonFracs[i+2][nelem];

				if( opac.opac[ip-1] > 0. )
				{
					ots = diffuse*MIN2(0.f,1.f-esc);
				}
				else
				{
					ots = 0.;
				}

				if( ots < 0. )
				{
					fprintf( ioQQQ, " metdif found negative OTS heav, n=%4ld%4ld val=%10.1e nzone=%4ld\n", 
					  nelem, i, ots, nzone );
					ShowMe();
					puts( "[Stop in metdif]" );
					cdEXIT(1);
				}

				RTAddOTSCon(ots,ip);

				/* now do the recombination Lya */
				ipla = Heavy.ipLyHeavy[i][nelem];
				esc = opac.ExpmTau[ipla-1];

				/* xLyaHeavy is set to a fraction of rad rec in MakeRecomb */
				difflya = DiffHeav.xLyaHeavy[i][nelem]*xIonFracs[i+2][nelem];

				/*begin sanity check */
				if( difflya < 0. )
				{
					fprintf( ioQQQ, " METDIF Negative Lya flux for element, ion=%4ld%4ld\n", 
					  nelem, i );
					fprintf( ioQQQ, " METDIF xLyaHeavy, xIonFracs=%10.2e%10.2e\n", 
					  DiffHeav.xLyaHeavy[i][nelem], xIonFracs[i+2][nelem] );

					ShowMe();
					puts( "[Stop in metdif]" );
					cdEXIT(1);
				}
				/*end sanity check */

				/* >>chng 00 dec 22, from MIN2 to MAX2, MIN2 had effect of always
				 * setting the ots rates to zero */
				ots = difflya*MAX2(0.f,1.f-esc);

				if( ots < 0. )
				{
					fprintf( ioQQQ, " metdif found negative OTS line heav%4ld%4ld val=%10.1e\n", 
					  nelem, i, ots );
					fprintf( ioQQQ, " 1-esc =%10.2e\n", 1. - 
					  esc );
					ShowMe();
					puts( "[Stop in metdif]" );
					cdEXIT(1);
				}
				RTAddOTSLin(ots,ipla);
			}
		}
	}

	if( FeII.lgFeIION )
	{
		/* do OTS and outward parts of FeII lines, if large atom is turned on */
		Fe2RTOTS();
	}

#	ifdef DEBUG_FUN
	fputs( " <->RT_OTS()\n", debug_fp );
#	endif

	return;
}

