/*SumContinuum sum flux, otscon, otslin, ConOutInter, outlin, to form SummeDif, SummedCon SummedOcc */
/*ZeroOTSContinuum - zero out some vectors - 
 * this is only called when code initialized by ContSetIntensity */
/*ChkSumCon sanity check confirms summed continua reflect contents of individuals */
#include "cddefines.h"
#include "rfield.h"
#include "opacity.h"
#include "sumcontinuum.h"

void SumContinuum(double *SumOTS , double *RatioOTSInc , long int * ipOTSchange)
{
	long int i;

	/* when setting new to sum of current and old, this is the fraction of old to take*/
	float FracOld , FracNew ,
		changeOTS ,
		OpacRatio ;

#	ifdef DEBUG_FUN
	fputs( "<+>SumContinuum()\n", debug_fp );
#	endif

	*SumOTS = 0.;
	*RatioOTSInc = 0.;

#	define FRACOLD 0.25f
	/* fraction old and new ots rates */
	FracOld = FRACOLD;
	FracNew = 1.f - FracOld ;

	/* remember largest change in ots rates */
	changeOTS = 0.;
	*ipOTSchange = -100000;
	/*fprintf(ioQQQ," sumcontinuum zone%li 1168=%e\n", nzone,rfield.otslin[1167]);*/

	for( i=0; i < rfield.nflux; i++ )
	{
		/* >>chng 01 feb 01, define inverse opacity in safe manner */
		double CurrentInverseOpacity = 1./MAX2( SMALLDOUBLE , opac.opac[i] ) ;

		/* remember which energy had largest change in ots rates */
		if( fabs( rfield.otscon[i]+rfield.otslin[i]-rfield.otsconNew[i]-rfield.otslinNew[i])> changeOTS)
		{
			*ipOTSchange = i;

			changeOTS = (float)
			  fabs( rfield.otscon[i]+rfield.otslin[i]-rfield.otsconNew[i]-rfield.otslinNew[i]);
		}
		/* ots rates were evaluated with previous opacities, now have different
		 * opacities since OpacityAddTotal just called.  Rescale OTS rates to current opacity.
		 * tests with pure h, he constant temperature clouds shows this decreases*/
		/* >>chng 99 apr 9, rescale ots */
		OpacRatio = 1.;
		OpacRatio = (float)(opac.OldOpacSave[i]*CurrentInverseOpacity);

		/* the New vectors are the ones updated by the AddOTS routines,
		 * and have the current OTS rates.  The otscon and otslin vectors
		 * have the rates from the previous refresh of the New vectors*/
		/* here is the refresh.  all were initially set to zero in call to 
		 * ZeroOTSContinuum (below) from zero */
		rfield.otscon[i] = (rfield.otscon[i]*FracOld +rfield.otsconNew[i]*FracNew)*OpacRatio;

		/* this is local ots continuum created by destroyed diffuse continua, 
		 * currently only two-photon */
		/* >>chng 00 dec 27, define this continuum and do not count 2-photon in old var */
		rfield.ConOTS_local_OTS_rate[i] = (float)((double)rfield.ConOTS_local_photons[i]*CurrentInverseOpacity);

		/* zero out the new otscon vector*/
		rfield.otsconNew[i] = 0.;

		/* current ots will be average of old and new */
		rfield.otslin[i] = (rfield.otslin[i]*FracOld + rfield.otslinNew[i]*FracNew)*OpacRatio;

		/* zero out the new otslin vector*/
		rfield.otslinNew[i] = 0.;

		/* remember sum of ots rates for convergence criteria */
		*SumOTS += (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i];

		/* also sum incident continuum for comparison */
		*RatioOTSInc += rfield.flux[i]*opac.opac[i];

		/* >>chng 00 dec 27, include ConOTS_local_OTS_rate */
		rfield.SummedDif[i] = rfield.otscon[i] + rfield.otslin[i] + 
		  rfield.ConOutInter[i]+rfield.ConOutRecInter[i] + rfield.outlin[i] +
		  rfield.ConOTS_local_OTS_rate[i];

		rfield.SummedCon[i] = rfield.flux[i] + rfield.SummedDif[i];
		rfield.SummedOcc[i] = rfield.SummedCon[i]*rfield.convoc[i];
	}

	/* form ratio of ots interaction rates to incident continuum interaction,
	 * shows relative importance of ots */
	if( *RatioOTSInc > 0. )
	{
		*RatioOTSInc = *SumOTS / *RatioOTSInc ;
	}
	else
	{
		*RatioOTSInc = 0.;
	}
	/*fprintf(ioQQQ," sumcontinuum exit 589=%e\n", rfield.otslin[589]);*/

	{
		/* following should be set true to print strongest ots contributors */
		/*@-redef@*/
		enum {DEBUG=FALSE};
		/*@+redef@*/
		if( DEBUG && (nzone>45)/**/ )
		{
			double BigOTS;
			int ipOTS=0;
			BigOTS = -1.;
			/* find the biggest ots contributor */
			for( i=0; i < rfield.nflux; i++ )
			{
				if( (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i] > BigOTS )
				{
					BigOTS = (rfield.otscon[i] + rfield.otslin[i])*opac.opac[i];
					ipOTS = (int)i;
				}
			}
			fprintf(ioQQQ,
				" sumcontinuunots zone %li SumOTS %.2e %.2eRyd opc:%.2e OTS%.2e lin%.2e con:%.2e %s %s cell%i ots2inc%.2e\n",
				nzone,
				*SumOTS,
				rfield.anu[ipOTS] , 
				opac.opac[ipOTS],
				BigOTS ,
				rfield.otslin[ipOTS],
				rfield.otscon[ipOTS] , 
				/* line	label */
				rfield.chLineLabel[ipOTS] ,
				/* cont label*/
				rfield.chContLabel[ipOTS] ,
				ipOTS,
				*RatioOTSInc);
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->SumContinuum()\n", debug_fp );
#	endif
	return;
}

/*ZeroOTSContinuum - zero out some vectors - 
 * this is only called when code initialized by ContSetIntensity */
void ZeroOTSContinuum( void )
{
	long int i;

#	ifdef DEBUG_FUN
	fputs( "<+>ZeroOTSContinuum()\n", debug_fp );
#	endif

	/* this loop goes up to nflux itself (<=) since the highest cell
	 * will be used to pass unity through the code to verify integrations */
	for( i=0; i <= rfield.nflux; i++ )
	{
		rfield.SummedDif[i] = 0.;
		/* the four main ots vectors */
		rfield.otscon[i] = 0.;
		rfield.otslin[i] = 0.;
		rfield.otslinNew[i] = 0.;
		rfield.otsconNew[i] = 0.;

		rfield.ConOutInter[i] = 0.;
		rfield.ConOutRecInter[i] = 0.;
		rfield.outlin[i] = 0.;
		rfield.SummedDif[i] = 0.;
		/* "zero" for the summed con will be just the incident radiation field */
		rfield.SummedCon[i] = rfield.flux[i];
		rfield.SummedOcc[i] = rfield.SummedCon[i]*rfield.convoc[i];
		rfield.ConOTS_local_photons[i] = 0.;
		rfield.ConOTS_local_OTS_rate[i] = 0.;
	}

#	ifdef DEBUG_FUN
	fputs( " <->ZeroOTSContinuum()\n", debug_fp );
#	endif
	return;
}


/*ChkSumCon sanity check confirms summed continua reflect contents of individuals */
void ChkSumCon(long int ipPnt)
{
	static long int nInsane=0;
	long int i;
	float phisig;

#	ifdef DEBUG_FUN
	fputs( "<+>ChkSumCon()\n", debug_fp );
#	endif

	/* this entire sub is a sanity check
	 *begin sanity check */
	for( i=0; i < rfield.nflux; i++ )
	{
		phisig = rfield.otscon[i] + rfield.otslin[i] + rfield.ConOutInter[i] + 
		  rfield.outlin[i]+rfield.ConOutRecInter[i] +
		  rfield.ConOTS_local_OTS_rate[i];
		if( phisig > 0. )
		{
			if( fabs(rfield.SummedDif[i]/phisig-1.) > 1e-3 )
			{
				insane();
				++nInsane;
				fprintf( ioQQQ, " ChkSumCon insane SummedDif at energy%12.5e error=%10.2e i=%4ld\n", 
				  rfield.anu[i], rfield.SummedDif[i]/phisig - 1., i );
				fprintf( ioQQQ, " SummedDif, sum are%11.4e%11.4e\n", 
				  rfield.SummedDif[i], phisig );
				fprintf( ioQQQ, " otscon otslin ConOutInter outlin are%11.4e%11.4e%11.4e%11.4e\n", 
				  rfield.otscon[i], rfield.otslin[i], rfield.ConOutInter[i]+rfield.ConOutRecInter[i], 
				  rfield.outlin[i] );
				fprintf( ioQQQ, " line continuum here are %4.4s %4.4s\n", 
				  rfield.chLineLabel[i], rfield.chContLabel[i] );
			}
		}
		phisig += rfield.flux[i];
		if( phisig > 0. )
		{
			if( fabs(rfield.SummedCon[i]/phisig-1.) > 1e-3 )
			{
				insane();
				++nInsane;
				fprintf( ioQQQ, " ChkSumCon%3ld, insane SummedCon at energy%12.5e error=%10.2e i=%4ld\n", 
				  ipPnt, rfield.anu[i], rfield.SummedCon[i]/phisig - 
				  1., i );
				fprintf( ioQQQ, " SummedCon, sum are%11.4e%11.4e\n", 
				  rfield.SummedCon[i], phisig );
				fprintf( ioQQQ, " otscon otslin ConOutInter outlin flux are%11.4e%11.4e%11.4e%11.4e%11.4e\n", 
				  rfield.otscon[i], rfield.otslin[i], rfield.ConOutInter[i]+rfield.ConOutRecInter[i], 
				  rfield.outlin[i], rfield.flux[i] );
				fprintf( ioQQQ, " line continuum here are %4.4s %4.4s\n", 
				  rfield.chLineLabel[i], rfield.chContLabel[i]
				   );
			}
		}
	}

	if( nInsane > 10 )
	{
		fprintf( ioQQQ, " ChkSumCon too much insanity to continue.\n");
		ShowMe();
		puts( "[Stop in ChkSumCon]" );
		cdEXIT(1);
	}

#	ifdef DEBUG_FUN
	fputs( " <->ChkSumCon()\n", debug_fp );
#	endif
	return;
}

