/*tfidle update some temperature dependent variables */
#include "cddefines.h"
#include "cooling.h"
#include "physconst.h"
#include "phycon.h"
#include "h.h"
#include "tfidle.h"

void tfidle(void)
{
	long int i;
	static float ttused = 0.f;
	static double edused = 0.;
	static int lgZLogSet = FALSE;

#	ifdef DEBUG_FUN
	fputs( "<+>tfidle()\n", debug_fp );
#	endif

	/* check that eden not negative */
	if( phycon.eden <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron density,%10.2e\n", 
		  phycon.eden );
		puts( "[Stop in tfidle]" );
		ShowMe();
		puts( "[Stop in tfidle]" );
		cdEXIT(1);
	}

	/* check that temperature not negative */
	if( phycon.te <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron temperature,%10.2e\n", 
		  phycon.te );
		puts( "[Stop in tfidle]" );
		ShowMe();
		puts( "[Stop in tfidle]" );
		cdEXIT(1);
	}

	/* only time only, set up array of logs of charge squared */
	if( !lgZLogSet )
	{
		for( i=0; i<LIMELM; ++i )
		{
			/* this array is used to modify the log temperature array
			 * defined below, for hydrogenic species of charge i+1 */
			phycon.sqlogz[i] = (float)log10( POW2(i+1.) );
		}
		lgZLogSet = TRUE;
	}

	if( phycon.te != ttused )
	{
		ttused = phycon.te;
		/* current temperature in eV */
		phycon.te_eV = phycon.te/(float)EVDEGK;
		phycon.tesqrd = POW2(phycon.te);
		phycon.sqrte = (float)sqrt(phycon.te);
		cooling.halfte = (float)(0.5/phycon.te);
		cooling.tsq1 = 1.f/phycon.tesqrd;
		phycon.te32 = phycon.te*phycon.sqrte;
		phycon.te70 = (float)pow(phycon.te,0.70);
		phycon.te30 = (float)pow(phycon.te,0.30);
		phycon.te20 = (float)pow(phycon.te,0.20);
		phycon.te10 = (float)pow(phycon.te,0.10);
		phycon.te05 = (float)pow(phycon.te,0.05);
		phycon.te03 = (float)pow(phycon.te,0.03);
		phycon.te02 = (float)pow(phycon.te,0.02);
		phycon.te01 = (float)pow(phycon.te,0.01);
		phycon.te005 = (float)pow(phycon.te,0.005);
		phycon.te003 = (float)pow(phycon.te,0.003);
		phycon.te001 = (float)pow(phycon.te,0.001);
		phycon.teinv = 1.f/phycon.te;
		phycon.alogte = (float)log10(phycon.te);
		phycon.alogete = (float)log(phycon.te);
		phycon.telogn[0] = phycon.alogte;

		for( i=1; i < 7; i++ )
		{
			phycon.telogn[i] = phycon.telogn[i-1]*phycon.telogn[0];
		}
	}

	/* term with hi added June 4, 93, to account for warm pdr */
	phycon.edensqte = (float)((phycon.eden + h.hi/1e4)/phycon.sqrte);
	phycon.cdsqte = (float)(phycon.edensqte*8.629e-6);

	/* >>>chng 99 nov 23, removed this line, so back to old method of h coll */
	/* used for hydrogenic collisions */
	phycon.EdenHCorr = (float)(phycon.eden + h.hi*1.7e-4);


	/* phycon.ElecFrac is electron fraction, used for secondary ionization efficiency */
	phycon.ElecFrac = (float)(phycon.eden/phycon.TotalNuclei);

	phycon.alogne = (float)log10(phycon.eden);

	if( phycon.eden != edused )
	{
		edused = phycon.eden;
		phycon.SqrtEden = (float)sqrt(phycon.eden);
	}

#	ifdef DEBUG_FUN
	fputs( " <->tfidle()\n", debug_fp );
#	endif
	return;
}

