/*velset set thermal velocities for all particles in gas */
#include "cddefines.h"
#include "physconst.h"
#include "atomcwgt.h"
#include "phycon.h"
#include "wind.h"
#include "doppvel.h"
#include "velset.h"

void velset(void)
{
	long int nelem;
	double turb2;

#	ifdef DEBUG_FUN
	fputs( "<+>velset()\n", debug_fp );
#	endif


	/* usually TurbVel =0, reset with turbulence command
	 * cm/s here, but was entered in km/s with command */
	turb2 = DoppVel.TurbVel*DoppVel.TurbVel;

	/* in case of D-Critical flow include inital velocity as
	 * a component of turbulence */
	if( wind.windv0 < 0. )
	{
		turb2 += POW2(wind.windv0);
	}

	/* computes one doppler width, in cm/sec,
	 * for each element with atomic number the array index*/
	for( nelem=0; nelem < LIMELM; nelem++ )
	{
		DoppVel.doppler[nelem] = 
			/*(float)sqrt(1.651e8*phycon.te/AtomcWgt.AtomicWeight[nelem]+*/
			/* >>chng 00 may 02 to physical constants */
			(float)sqrt(2.*BOLTZMANN/ATOMIC_MASS_UNIT*phycon.te/AtomcWgt.AtomicWeight[nelem]+
		  turb2);
	}

	/* DoppVel.doppler[LIMELM] is CO, vector is dim LIMELM+1 */
	/* C12O16 */
	DoppVel.doppler[LIMELM] = 
		(float)sqrt(2.*BOLTZMANN/ATOMIC_MASS_UNIT*phycon.te/
		(AtomcWgt.AtomicWeight[5]+AtomcWgt.AtomicWeight[7]) + turb2);
	/* C13O16 */
	DoppVel.doppler[LIMELM+1] = 
		(float)sqrt(2.*BOLTZMANN/ATOMIC_MASS_UNIT*phycon.te/
		(AtomcWgt.AtomicWeight[5]*13./12.+AtomcWgt.AtomicWeight[7]) + turb2);

#	ifdef DEBUG_FUN
	fputs( " <->velset()\n", debug_fp );
#	endif
	return;
}

