/*zero zero out or initialize variables, called by cdInit, but also by func during optimization,
 * this is called before any commands are parsed */
#include <string.h>
#include <float.h>
#include "cddefines.h"
#include "physconst.h"
#include "hydrogenic.h"
#include "nfeii.h"
#include "incidspec.h"
#include "pnunit.h"
#include "o3exc.h"
#include "dtmase.h"
#include "plwidth.h"
#include "doppvel.h"
#include "mgexc.h"
#include "testit.h"
#include "grainvar.h"
#include "linesave.h"
#include "punpoint.h"
#include "thermal.h"
#include "pmp2s.h"
#include "mean.h"
#include "vrrfit.h"
#include "pop371.h"
#include "converge.h"
#include "codish.h"
#include "zonecnt.h"
#include "timesc.h"
#include "heots.h"
#include "iwavlen.h"
#include "peimbt.h"
#include "struc.h"
#include "photrate.h"
#include "phfiton.h"
#include "fe2neg.h"
#include "punchskip.h"
#include "ionfracs.h"
#include "typmatrx.h"
#include "holod.h"
#include "insanity.h"
#include "trimstg.h"
#include "fluct.h"
#include "ioreco.h"
#include "supdie.h"
#include "dtrans.h"
#include "diffheav.h"
#include "physok.h"
#include "double.h"
#include "ca2mly.h"
#include "hevmolec.h"
#include "lotsco.h"
#include "pca2ex.h"
#include "h2ozer.h"
#include "gionrc.h"
#include "jmin.h"
#include "drneg.h"
#include "coatom.h"
#include "difher.h"
#include "comneg.h"
#include "input.h"
#include "bit32.h"
#include "negoi.h"
#include "pressure.h"
#include "dndt.h"
#include "intalk.h"
#include "sumheat.h"
#include "tff.h"
#include "h2max.h"
#include "stopcalc.h"
#include "numderiv.h"
#include "wind.h"
#include "negdrg.h"
#include "cmfrac.h"
#include "gaunts.h"
#include "colden.h"
#include "stimax.h"
#include "hmi.h"
#include "dcala.h"
#include "heat.h"
#include "hgamsc.h"
#include "tehigh.h"
#include "hphoto.h"
#include "nsbig.h"
#include "dalerr.h"
#include "rfield.h"
#include "drminu.h"
#include "uspher.h"
#include "hemase.h"
#include "bndsok.h"
#include "habing.h"
#include "resion.h"
#include "thlo.h"
#include "the1lo.h"
#include "he3lines.h"
#include "chargtran.h"
#include "he.h"
#include "abundances.h"
#include "radius.h"
#include "opacity.h"
#include "neutrn.h"
#include "fe2tau.h"
#include "tgff.h"
#include "broke.h"
#include "fluors.h"
#include "max2pht.h"
#include "angllum.h"
#include "secondaries.h"
#include "called.h"
#include "phycon.h"
#include "aaaa.h"
#include "warnings.h"
#include "cooling.h"
#include "nhe1lvl.h"
#include "nhe3lvl.h"
#include "he1bn.h"
#include "he3bn.h"
#include "he3n.h"
#include "he3pht.h"
#include "he1dmp.h"
#include "fe2cool.h"
#include "fe4cool.h"
#include "zero.h"

void zero(void)
{
	long int i,
	  j, 
	  nd;

#	ifdef DEBUG_FUN
	fputs( "<+>zero()\n", debug_fp );
#	endif

	/* this routine is called exactly one time at the start of
	 * the calculation of a single model.  It is called before
	 * the boundary conditions are read in, and is never called again
	 * during that calculation.  When the code is used as a subroutine
	 * this routine is called first, when a specific model is initialized.
	 * All default variables that must be initialized before a calculation
	 * must appear in the routine.
	 */

	/* set version number */
	AAAA();

	/* these used to be in block data logic */
	Zerologic();

	/* set all initial abundances */
	ZeroAbund();

	/* zero out parameters needed by large FeII atom */
	FeIIZero();

	/* zero out warnings, cautions, notes, etc */
	wcnint();

	/* default line width for plots, in units of speed of light
	 * PunchLWidth = 1.
	 * >>chng 97 jul 10, from unity to 1000 km/sec */
	PLWidth.PunchLWidth = (float)(SPEEDLIGHT/1000.e5);

	/* limits for highest and lowest stages of ionization in TrimStage */
	trimstg.trimhi = 1e-6;
	trimstg.trimlo = 1e-10;

	/* change amgle of illumination */
	Angllum.AngleIllum = 1.;

	for( i=0; i < NZLIM; i++ )
	{
		struc.testr[i] = 0.;
		struc.volstr[i] = 0.;
		struc.radstr[i] = 0.;
		struc.histr[i] = 0.;
		struc.hiistr[i] = 0.;
		struc.ednstr[i] = 0.;
		struc.o3str[i] = 0.;
		struc.heatstr[i] = 0.;
		struc.coolstr[i] = 0.;
		struc.pressure[i] = 0.;
		struc.pdenstr[i] = 0.;
	}

	holod.fcool = 0.;
	holod.dfcool = 0.;
	holod.feheat = 0.;
	pmp2s.p1909 = 0.;
	pmp2s.p2326 = 0.;
	pmp2s.p1895 = 0.;
	pmp2s.p1666 = 0.;
	pmp2s.p1401 = 0.;

	/* this counts number of times ionize is called by PresChng, in current zone
	 * these are reset here, so that we count from first zone not search */
	conv.nPres2Ioniz = 0;

	/* tolerance in heating -  cooling balance */
	conv.toler = 0.02f;

	/* this counts how many times ionize is called in this model after startr,
	 * and is flag used by ionize to understand it is being called the first time*/
	conv.nTotalIoniz = 0;
	/* these are variables to remember the biggest error in the
	 * electron density, and the zone in qhich it occurred */
	conv.BigEdenError = 0.;
	conv.AverEdenError = 0.;
	conv.BigHeatCoolError = 0.;
	conv.AverHeatCoolError = 0.;
	conv.BigPressError = 0.;
	conv.AverPressError = 0.;

	fluct.flong = 0.;
	fluct.flcPhase = 0.;

	PhotRate.PhotScaleOn = 1.;
	vrrfit.lgVrrFit = TRUE;
	PhFitOn.lgPhFit = FALSE;

	/* this flag says whether or not to use the new integer format for FUX and X-Ray
	 * line ids in the standard line list - normally TRUE, set FALSE with the
	 * set oldwl command - used in routine iWavLen */
	lgWavLenNew = TRUE;

	/* age of the cloud, to check for time-steady */
	timesc.CloudAgeSet = -1.f;
	/* some timescale for CO and H2 */
	timesc.AgeH2MoleDest = 0.;
	timesc.AgeCOMoleDest = 0.;
	timesc.BigH2MoleForm = 0.;
	timesc.BigCOMoleForm = 0.;
	timesc.TimeH21cm = 0.;

	peimbt.tsqden = 1e7;
	codish.CODissHeat = 0.;
	NumDeriv.lgNumDeriv = FALSE;
	fluct.lgDenFluc = TRUE;
	/* nsum is line pointer within large stack of line intensities */
	LineSave.nsum = 0;

	/* set of variables used to control punch command */
	pnunit.npunch = 0;
	pnunit.lgPunContinuum = FALSE;

	/* normally is unity, set to zero when hydrogen collisions turned off,
	 * collisional ionization*/
	hydro.HColIonOn = 1.;
	/* collisional excitation */
	hydro.HColExcOn = 1.;
	/* 2s - 2p exchange collisions */
	hydro.lgC2psOn = TRUE;
	/* free free heating, cooling, net */
	hydro.FreeFreeCool = 0.;
	hydro.FreeFreeHeat = 0.;
	hydro.HFFNet = 0.;

	/* option to print emissivity instead of intensity/luminosity */
	hydro.lgHydEmiss = FALSE;
	hydro.lgHiPop2 = FALSE;
	hydro.pop2mx = 0.;
	hydro.lgHLyaMased = FALSE;

	/* this is flag indicating which type of model atom to use, 
	 * set with atom hydrogen populations, departure, lowt */
	strcpy( hydro.chTypeAtom , "none" );

	/* type of hydrogen atom topoff, options are " add" and "scal" 
	 * in versions 90 this was " add", but was "scal" in 91
	 * >>chng 99 jan 16, changed back to " add"*/
	/*strcpy( hydro.chHTopType, "scal" );*/
	strcpy( hydro.chHTopType, " add" );

	/* lowest level is 2s,
	 * in versions 90 this was equal to 10, but was 1 in 91
	 * >>chng 99 jan 16, changed back to 10*/
	/*hydro.nHTopOff = 1;*/
	hydro.nHTopOff = 10;

	/* Lya excitation temperature, counter for hotter than gas */
	hydro.TexcLya = 0.;
	hydro.TLyaMax = 0.;
	hydro.nLyaHot = 0;

	double_.DoubleTau = 1.;
	rfield.SumOutCon = 0.;
	rfield.SumIncCon = 0.;
	/* rfield.DiffPumpOn is unity unless process disabled by setting to 1
	 * with no diffuse line pumping command */
	rfield.DiffPumpOn = 1.;

	/* which matrix inversion routine to use */
	strcpy( TypMatrx.chMatrix, "linpack" );


	/**********************************************************************
	 * all parameters having to do with secondary ionization 
	 * by suprathermal electrons 
	 **********************************************************************/
	Secondaries.SetCsupra = 0.;
	Secondaries.lgCSetOn = FALSE;
	Secondaries.lgSecOFF = FALSE;

	Secondaries.heatef = 1.;
	Secondaries.efionz = 0.;
	Secondaries.exctef = 0.;
	Secondaries.csupra = 0.;
	Secondaries.x12tot = 0.;
	Secondaries.HyExc = 0.;
	Secondaries.HyIon = 0.;
	Secondaries.seccmp = 0.;
	Secondaries.scmpla = 0.;
	Secondaries.seche = 0.;
	Secondaries.helax = 0.;
	Secondaries.sec2total = 0.;

	Secondaries.Hx12[0][LIMELM] = 0.;
	Secondaries.Hx12[1][LIMELM] = 0.;
	for( i=0; i< LIMELM; ++i )
	{
		Secondaries.Hx12[0][i] = 0.;
		Secondaries.Hx12[1][i] = 0.;
		for( j=0; j<LIMELM; ++j )
		{
			Secondaries.SecondaryExcite[i][j] = 0.;
			Secondaries.SecondaryIonize[i][j] = 0.;
		}
	}

	/**********************************************************************
	 * these are options to print errors to special window,
	 * set with print errors command, 
	 * output will go to standard error
	 * defined in cdInit 
	 **********************************************************************/
	lgPrnErr = FALSE;
	ioPrnErr = stderr;

	Max2Pht.xMax2Pht = 0.;
	Fe2Neg.nFe2Neg = 0;

	/* turn off flag for insane goings */
	insanity.lgInsane = FALSE;

	/* punch every one continuum point */
	PunchSkip.ncPunchSkip = 1;

	/* flag for Lya masing */
	hydro.xLaMase = 0.;

	/* the code is ok at startup */
	broke.lgBroke = FALSE;
	broke.lgFixit = FALSE;

	/* default condition is burgess supressed, nusbaumer and storye not */
	supdie.lgSupDie[0] = TRUE;
	supdie.lgSupDie[1] = FALSE;

	/* the ipSolar array is a set of pointers used for reading
	 * in abundances with the "abundances" command
	 * the hydrogen abundance of unity is not read in */
	abundances.npSolar = LIMELM - 1;
	for( i=0; i < abundances.npSolar; i++ )
	{
		abundances.ipSolar[i] = i + 2;
	}

	/* > 1 if not first call, then just zero arrays out */
	if( nCalled == 0 )
	{
		for( i=0; i < LIMELM; i++ )
		{
			/* never turn on element if turned off on first iteration */
			abundances.lgElmtOn[i] = TRUE;
		}
	}

	/* zero out diffuse recombinaiton emission arrays */
	for( i=0; i < LIMELM; i++ )
	{
		for( j=0; j < (LIMELM - 1); j++ )
		{
			DiffHeav.DiffusHeavy[j][i] = 0.;
			DiffHeav.xLyaHeavy[j][i] = 0.;
		}
	}

	for( i=0; i < LIMELM; i++ )
	{
		for( j=0; j < LIMELM; j++ )
		{
			for( nd=0; nd < 7; nd++ )
			{
				/* must be zero since ion routines use these when
				 * not yet defined */
				PhotRate.PhotoRate[0][nd][j][i] = 0.;
				PhotRate.PhotoRate[1][nd][j][i] = 0.;
				PhotRate.PhotoRate[2][nd][j][i] = 0.;
			}
		}
	}

	/* this is the simple fred hamann feii atom */
	for( i=0; i < NFEII; i++ )
	{
		fe2tau.Fe2PopLte[i] = 0.;
		fe2tau.feopc[i] = 0.;
		fe2tau.Fe2TauLte[i] = opac.taumin;
	}

	/* zero out volume and column density save arrays */
	MeanZero();

	fluors.lgFluor = TRUE;
	physok.lgPhysOK = TRUE;

	/* parameters to do with wind */
	wind.lgWindOK = TRUE;
	wind.windv0 = 0.;
	wind.comass = 0.;
	wind.windv = 0.;
	wind.emdot = 0.;
	wind.AccelAver = 0.;
	wind.acldr = 0.;
	wind.agrav = 0.;
	wind.AccelTot = 0.;
	wind.AccelCont = 0.;
	wind.AccelLine = 0.;
	wind.AccelPres = 0.;
	wind.AccelMax = 0.;
	wind.fmul = 0.;
	wind.lgVelPos = TRUE;
	wind.dVeldRad = 0.;

	/* zero out heating rates */
	ZeroHeat();

	/* main arrays to save ionization fractions*/
	for( i=0; i < LIMELM; i++ )
	{
		for( j=0; j < (LIMELM + 1); j++ )
		{
			xIonFracs[j][i] = 0.;
		}
	}

	ca2mly.Ca2RmLya = 0.;
	pca2ex.popca2ex = 0.;

	gionrc.grecon = 1.;
	cmfrac.CarMolFrac = -1.f;
	lotsco.lgLotsCO = FALSE;
	h2ozer.lgH2Ozer = FALSE;

	/* the CO moleculae */
	comneg.lgComNeg = FALSE;
	comneg.COCoolBigFrac = 0.;
	comneg.lgCOCoolCaped = FALSE;

	coatom.catmic = 0.;
	coatom.oatmic = 0.;

	/* zero out molecular species */
	for( i=0; i < NHEVML; i++ )
	{
		hevmolec.hevmol[i] = 0.;
		hevmolec.hevcol[i] = 0.;
	}
	/* ratio of C12O16 to C13O16 */
	hevmolec.RatioC12O16_2_C13O16 = 30.;

	jmin.rjnmin = FLT_MAX;
	jmin.ajmmin = FLT_MAX;
	drneg.lgDrNeg = FALSE;

	/* this is the default allowed relative error in the electron density */
	phycon.EdenError = 0.01f;
	/* extra electron density, set with eden command */
	phycon.EdenExtra = 0.;
	/* forced electron density, set with set eden command */
	phycon.EdenSet = 0.;

	/* check whether this is a 32-bit machine
	 * use te in common, which forc will translate as a float.
	 * then set te after test */
	phycon.te = 1e-37f;
	phycon.te /= 1e40;
	/* had to move above up here to prevent ms c from using double without
	 * doing store*/
	phycon.EdenFFSum = 0.;

	/* some titles and line images */
	for( i=0; i<74; ++i)
	{
		input.chTitle[i] = ' ';
	}
	input.chTitle[75] = '\0';

	strcpy( dtrans.chDffTrns, "OU2" );
	DoppVel.TurbVel = 0.;

	/* flag for constant pressure, include continuum too */
	pressure.lgConPres = TRUE;
	/* pressure related variables */
	pressure.pinit = 0.;
	pressure.PresInteg = 0.;
	pressure.pinzon = 0.;

	pressure.lgRadPresON = FALSE;
	pressure.RadPres = 0.;
	pressure.lgPradCap = FALSE;
	pressure.lgPradDen = FALSE;
	pressure.lgRadPrsON = TRUE;
	/* normally abort when radiation pressure exceeds gas pressure in const pres mod,
	 * this is option to keep going, set with NO ABORT on constant pressure command */
	pressure.lgRadPresAbortOK = TRUE;
	strcpy( pressure.chCPres, "CDEN" );

	pressure.RadBetaMax = 0.;
	pressure.PresMax = 0.;

	/* check whether this is a 32-bit machine
	 * use te in common, which forc will translate as a float.
	 * then set te after test */
	if( phycon.te == 0. )
	{
		bit32.lgBit32 = TRUE;
	}
	else
	{
		bit32.lgBit32 = FALSE;
	}

	/* now set physical conditions array */
	phycon.te = 1e4;
	phycon.eden = -99.f;
	phycon.hden = -99.f;

	negoi.nNegOI = 0;

	/* do we want to punch negative opacities */
	opac.lgNegOpacIO = FALSE;

	strcpy( opac.chOpcTyp, "    " );

	/* smallest allowed line and Lya optical depths, reset with
	 * caseb command */
	opac.taumin = 1e-20f;
	opac.tlamin = 1e-20f;

	/* this flag says to use the static opacities,
	 * only evaluate them at start of each new zone.
	 * when set false with 
	 * no static opacities
	 * command, always reevalute them */
	opac.lgOpacStatic =  TRUE;

	/* set true in radinc if negative opacites ever occur */
	opac.lgOpacNeg = FALSE;

	/* can turn of scattering opacities for some geometries */
	opac.lgScatON = TRUE;

	for( i=0; i < NCELL; i++ )
	{
		opac.OldOpacSave[i] = 0.;
		/* >>chng 99 apr 11, only opac had been set to 0, eovrlp crashed
		 * with div by zero when called by tauout before opacities had been
		 * defined.  Now set al three to very small number,
		 * CODE CRASHED - changed back to zero, apparetly some part of
		 * code uses opacity of zero to check on continuum bounds */
		opac.opac[i] = 0.;
		/* NB read above before changing set to zero in above */
		opac.scatop[i] = 0.;
		opac.albedo[i] = 0.;
		opac.TauTotalGeo[0][i] = opac.taumin;
		opac.TauAbsGeo[0][i] = opac.taumin;
		opac.TauScatGeo[0][i] = opac.taumin;
		opac.tmn[i] = 1.;
		opac.FreeFreeOpacity[i] = 0.;
		opac.ExpZone[i] = 1.;
		opac.e2TauAbs[i] = 1.;
		opac.ExpmTau[i] = 1.;

		rfield.ConLocInter[i] = 0.;
		rfield.OccNumbDiffCont[i] = 0.;
		rfield.ConOutNoInter[i] = 0.;
		rfield.ConRefDiff[i] = 0.;
		rfield.ContBoltz[i] = 0.;

		rfield.reflin[i] = 0.;
		rfield.ConRefIncid[i] = 0.;
		gaunts.gff[i] = 1.1f;
		gaunts.gffhe2[i] = 1.1f;
		gv.GrainEmission[i] = 0.;
		gv.SilicateEmission[i] = 0.;
		gv.GraphiteEmission[i] = 0.;
	}
	gv.TotalEden = 0.;

	/* variables having to do with compiling and/or using the
	 * ancillary file of stored opacities */
	opac.lgCompileOpac = FALSE;
	/* "no file opacity" command sets following var false, says not to use file */
	opac.lgUseFileOpac = TRUE;

	rfield.nflux = NCELL;

	/* these are flags to let gff sub know to recompute ContBoltz and gss */
	tgff.tgffused = -1.f;
	tgff.tgffsued2 = -1.f;

	dndt.dDensityDT = 0.;
	HPhoto.cintot = 0.;
	resion.otsfe2 = 0.;
	tffCom.ntff = 1;
	tffCom.tff = 0.;
	h2max.BiggestH2 = -1.f;
	negdrg.lgNegGrnDrg = FALSE;
	thermal.nUnstable = 0;
	thermal.lgUnstable = FALSE;

	/* various criteria for stopping model */
	StopCalc.tauend = 0.;
	StopCalc.taunu = 0.;
	StopCalc.iptnu = NCELL;
	/* highest allowed temperature */
	StopCalc.T2High = 1e12f;
	/* highest and lowest temperatues to ever allow */
	StopCalc.TeHighest = 1e10f;
	StopCalc.TeLowest = 2.8f;
	/* stopping temperature */
	StopCalc.tend = 4000.;
	/* stop electron fraction, usually not used */
	StopCalc.StopElecFrac = -FLT_MAX;
	/* ending column densities */
	StopCalc.HColStop = 1e30f;
	StopCalc.colpls = 1e30f;
	StopCalc.colnut = 1e30f;
	StopCalc.StopElecDensity = -1e30f;

	/* limits to stop line command */
	StopCalc.nstpl = 0;
	for( i=0; i < MXSTPL; i++ )
	{
		StopCalc.stpint[i] = FLT_MAX;
		StopCalc.ipStopLin1[i] = 1;
		StopCalc.ipStopLin2[i] = 1;
		StopCalc.LineStopWl[i] = 0;
	}

	neutrn.frcneu = 0.;
	neutrn.effneu = 1.;
	neutrn.totneu = 0.;
	neutrn.lgNeutrnHeatOn = FALSE;

	hemase.lgHeMase = FALSE;
	uspher.lgUSphON = FALSE;
	stimaxCom.stimax[0] = 0.;
	stimaxCom.stimax[1] = 0.;

	hmi.htwo = 0.;
	hmi.h2plus = 0.;
	hmi.h3plus = 0.;
	hmi.htwo_star = 0.;
	hmi.hminus = 0.;
	hmi.hmihet = 0.;
	hmi.HeatH2DexcMax = 0.;
	hmi.hmitot = 0.;
	hmi.H2Opacity = 0.;
	hmi.hmidep = 1.;
	hmi.h2dep = 1.;
	hmi.h2pdep = 1.;
	hmi.h3pdep = 1.;

	dcala.oldcla = 0.;
	dcala.Ca3d = 0.;
	dcala.Ca4p = 0.;
	dcala.dstCala = 0.;

	for( i=0; i < NCOLD; i++ )
	{
		coldenCom.colden[i] = 0.;
	}

	/* these are the low and high energy bounds of the continuum */
	rfield.emm = 1.001e-8f;
	rfield.egamry = 7.354e6f;

	/* this is the faintest the high-energy tail of the continuum be */
	rfield.FluxFaint = 1e-10f;

	for( i=0; i < LIMSPC; i++ )
	{
		/* range(i,1) = 1.
		 * >>chng 96 dec 18, from inf mass ryd to H mass ryg */
		IncidSpec.range[0][i] = (float)HIONPOT;
		IncidSpec.range[1][i] = rfield.egamry;
	}

	/* reset some variable related to cooling */
	colzro();
	cooling.lgColNeg = TRUE;
	cooling.lgCNegChk = TRUE;
	cooling.CoolHeatMax = 0.;
	cooling.ipCoolHeatMax = 0;
	cooling.totcol = 0.;
	cooling.GrossHeat = 0.;
	cooling.heatl = 0.;
	cooling.coolheat = 0.;
	cooling.lgCExtraOn = FALSE;
	cooling.CoolExtra = 0.;
	cooling.ctot = 1.;

	heat.HeatNet = 0.;
	heat.htot = 1.;

	heat.power = 0.;

	radius.rinner = 0.e0;
	radius.Radius = 0.e0;
	radius.drad = 0.e0;
	radius.depth = 1e-30;
	radius.r1r0sq = 1.;
	/* this is changed with the roberto command, to go from out to in */
	radius.dRadSign = 1.;
	/* RDFALT is log of default starting radius (cm) */
	radius.rdfalt = 25.;

	/* set default cylinder thickness */
	radius.CylindHigh = 1e35f;
	radius.lgCylnOn = FALSE;

	radius.dReff = 1.;
	radius.dVeff = 1.;
	radius.dRNeff = 1.;
	radius.lgdR2Small = FALSE;

	radius.glbdst = 0.;
	radius.glbrad = 0.;

	nzone = 0;
	ZoneCnt.nprint = 1000;
	ZoneCnt.lgZoneSet = FALSE;
	ZoneCnt.lgZoneTrp = FALSE;
	ZoneCnt.lgEndDflt = TRUE;

	/* this is default number of zones
	 * >>chng 96 jun 5, from 400 to 500 for thickes corners4 grid */
	ZoneCnt.nEndDflt = 600;

	for( i=0; i < ITRDIM; i++ )
	{
		ZoneCnt.nend[i] = ZoneCnt.nEndDflt;
		radius.router[i] = 1e30;
	}

	hgamsc.BoundCompton = 0.;
	heots.esc584 = 0.;
	heots.esc626 = 0.;
	heots.esc911 = 0.;
	heots.esc910 = 0.;
	heots.esc610 = 0.;
	heots.esc660 = 0.;

	/* save initial condition for talk in case PRINT LAST used */
	intalk.lgInTalk = called.lgTalk;

	/* flags for whether continuum is defined over all energies */
	bndsok.lgMMok = TRUE;
	bndsok.lgHPhtOK = TRUE;
	bndsok.lgXRayOK = TRUE;
	bndsok.lgGamrOK = TRUE;

	tehigh.thist = 0.;
	tehigh.tlowst = 1e20f;
	nsbigCom.nsbig = 0;
	dalerr.ilt = 0;
	dalerr.iltln = 0;
	dalerr.ilthn = 0;
	dalerr.ihthn = 0;
	dalerr.ifail = 0;

	habing.lgHabing = FALSE;

	drminu.lgDrMinUsed = FALSE;

	resion.otsfe2 = 0.;

	mgexc.popmg2 = 0.;
	mgexc.rmg2l = 0.;
	o3exc.poiii2 = 0.;
	o3exc.poiii3 = 0.;
	o3exc.poiexc = 0.;

	thlo.HydTempLimit = 1000.;

	the1loCom.the1lo = 1000.;
	the1loCom.nhe1lo = 0;

	he3lines.pht2s = 0.;
	he3lines.p2s = 0.;
	he3lines.p2p = 0.;
	he3lines.p3s = 0.;
	he3lines.p3p = 0.;
	he3lines.p3d = 0.;
	he3lines.p10830 = 0.;
	he3lines.p5876 = 0.;
	he3lines.p7065 = 0.;
	he3lines.p3889 = 0.;
	he3lines.he3clx = 0.;
	o3exc.d5007r = 0.;
	o3exc.d5007t = 0.;
	o3exc.d4363 = 0.;
	o3exc.d6300 = 0.;

	/***************************************************
	 * charge transfer ionization and recombination 
	 ***************************************************/
	for( i=0; i < 2; i++ )
	{
		for( j=0; j < 2; ++j )
		{
			ChargTran.CTHion[i][j] = 0.;
			ChargTran.CTHrec[i][j] = 0.;
		}
	}
	for( i=0; i<5; ++i )
	{
		ChargTran.CTHionAgnt[i] = 0.;
	}
	ChargTran.chrexCool = 0.;
	ChargTran.chrener = 0.;

	/* HCharHeatMax, HCharCoolMax are largest fractions of heating in cur zone
	 * or cooling due to ct */
	ChargTran.HCharHeatMax = 0.;
	ChargTran.HCharCoolMax = 0.;

	/* option to turn off H charge transfer ionization */
	ChargTran.HCTOn = 0.;

	/* flag saying that charge transfer heating should be included,
	 * turned off with no CTHeat commmand */
	ChargTran.HCharHeatOn = 1.;
	for( i=0; i< LIMELM; ++i )
	{
		for( j=0; j<LIMELM; ++j )
		{
			ChargTran.HCharExcIon[i][j] = 0.;
			ChargTran.HCharExcRec[i][j] = 0.;
		}
	}

	/* >>chng 97 jan 6, from 0 to 8.5e-10*q as per Alex Dalgarno e-mail
	 * >>chng 97 feb 6, from 8.5e-10*q 1.92e-9x as per Alex Dalgarno e-mail */
	ChargTran.HCTAlex = 1.92e-9f;

	opac.telec = opac.taumin;
	opac.thmin = opac.taumin;

	opac.tpcah[0] = opac.taumin;
	opac.tpcah[1] = 1e20f;

	he.hei = 0.;
	he.heii = 0.;
	he.heiii = 0.;
	he.hei3 = 0.;
	he.hei1 = 0.;

	opac.lgTauOutOn = FALSE;

	for( i=0; i < LIMELM; i++ )
	{
		abundances.solar[i] = abundances.SolarSave[i];
		/* these are depltion scale factors */
		abundances.depset[i] = 1.;
		/*begin sanity check */
		if( abundances.solar[i] == 0. || abundances.depset[i] == 0. )
		{
			fprintf( ioQQQ, " ZERO finds insane abundance or depletion.\n" );
			fprintf( ioQQQ, " atomic number=%6ld abundance=%10.2e depletion=%10.2e\n", 
			  i, abundances.solar[i], abundances.depset[i] );
			ShowMe();
			puts( "[Stop in zero]" );
			cdEXIT(1);
		}
		/*end sanity check */
	}

	abundances.lgAbTaON = FALSE;
	abundances.lgAbnSolar = FALSE;

	/* typical ISM depletion factors, subjective mean of Cowie and Songaila
	 * and Jenkins 
	 * */
	abundances.Depletion[0] = 1.;
	abundances.Depletion[1] = 1.;
	abundances.Depletion[2] = .16f;
	abundances.Depletion[3] = .6f;
	abundances.Depletion[4] = .13f;
	abundances.Depletion[5] = 0.4f;
	abundances.Depletion[6] = 1.0f;
	abundances.Depletion[7] = 0.6f;
	abundances.Depletion[8] = .3f;
	abundances.Depletion[9] = 1.f;
	abundances.Depletion[10] = 0.2f;
	abundances.Depletion[11] = 0.2f;
	abundances.Depletion[12] = 0.01f;
	abundances.Depletion[13] = 0.03f;
	abundances.Depletion[14] = .25f;
	abundances.Depletion[15] = 1.0f;
	abundances.Depletion[16] = 0.4f;
	abundances.Depletion[17] = 1.0f;
	abundances.Depletion[18] = .3f;
	abundances.Depletion[19] = 1e-4f;
	abundances.Depletion[20] = 5e-3f;
	abundances.Depletion[21] = 8e-3f;
	abundances.Depletion[22] = 6e-3f;
	abundances.Depletion[23] = 6e-3f;
	abundances.Depletion[24] = 5e-2f;
	abundances.Depletion[25] = 0.01f;
	abundances.Depletion[26] = 0.01f;
	abundances.Depletion[27] = 0.01f;
	abundances.Depletion[28] = .1f;
	abundances.Depletion[29] = .25f;

	abundances.lgDepln = FALSE;

	abundances.ScaleMetals = 1.;

	/* option to turn off an element */
	for( i=0; i < LIMELM; i++ )
	{
		difher.CaseBCheck[i] = -FLT_MAX;
		/* set of scale factors for changing abundances with elements command */
		abundances.ScaleElement[i] = 1.;
		/* option to have abundances from table */
		abundances.lgAbunTabl[i] = FALSE;
	}

	dtMase.dTauMase = 0.;
	dtMase.lgMaserCapHit = FALSE;

	Testit.lgTestIt = FALSE;
	Testit.lgTestOn = FALSE;

	/* some grain variables */
	gv.nBin = 0;
	for( nd=0; nd < NDUST; nd++ )
	{
		gv.bin[nd] = NULL;
	}
	gv.lgDustOn = FALSE;
	gv.lgQHeatAll = FALSE;
	gv.lgQHeatOn = TRUE;
	gv.lgDHetOn = TRUE;
	gv.lgDColOn = TRUE;
	gv.GrainMetal = 1.;

	he3pht.he3photo = 0.;
	he3pht.He3Fr2Phot = 0.;
	he3pht.he3lya = 0.;

	for( i=0; i < (NHE1LVL + 1); i++ )
	{
		he1bnCOM.he1bn[i] = 1.;
	}

	/* array of lifetimes for helium lines - sum of a's down/4pi*912 */
	he1dmpCOM.he1dmp[0] = 0.;
	he1dmpCOM.he1dmp[1] = 455.1f;
	he1dmpCOM.he1dmp[2] = 72.4f;
	he1dmpCOM.he1dmp[3] = 21.9f;
	he1dmpCOM.he1dmp[4] = 8.42f;
	he1dmpCOM.he1dmp[5] = 3.77f;
	he1dmpCOM.he1dmp[6] = 1.82f;
	he1dmpCOM.he1dmp[7] = 0.939f;
	he1dmpCOM.he1dmp[8] = 0.438f;

	for( i=0; i < NHE3LVL; i++ )
	{
		he3nCom.he3n[i] = 0.;
		he3bnCom.he3bn[i] = 1.;
	}

	fe2cool.Fe2L16Tot = 0.;
	fe2cool.fe21406 = 0.;
	fe2cool.fe21507 = 0.;
	fe2cool.fe21508 = 0.;
	fe2cool.fe21609 = 0.;
	fe2cool.fe21308 = 0.;
	fe2cool.fe21207 = 0.;
	fe2cool.fe21106 = 0.;
	fe2cool.fe21006 = 0.;
	fe2cool.fe21204 = 0.;
	fe2cool.fe21103 = 0.;
	fe2cool.fe21104 = 0.;
	fe2cool.fe21001 = 0.;
	fe2cool.fe21002 = 0.;
	fe2cool.fe20201 = 0.;
	fe2cool.fe20302 = 0.;
	fe2cool.fe20706 = 0.;
	fe2cool.fe20807 = 0.;
	fe2cool.fe20908 = 0.;
	fe2cool.fe21007 = 0.;
	fe2cool.fe21107 = 0.;
	fe2cool.fe21108 = 0.;
	fe2cool.fe21110 = 0.;
	fe2cool.fe21208 = 0.;
	fe2cool.fe21209 = 0.;
	fe2cool.fe21211 = 0.;
	fe4cool.Fe4CoolTot = 0.;
	fe4cool.fe40401 = 0.;
	fe4cool.fe42836 = 0.;
	fe4cool.fe42829 = 0.;
	fe4cool.fe42567 = 0.;
	fe4cool.fe41207 = 0.;
	fe4cool.fe41206 = 0.;
	fe4cool.fe41106 = 0.;
	fe4cool.fe41007 = 0.;
	fe4cool.fe41008 = 0.;
	fe4cool.fe40906 = 0.;

#	ifdef DEBUG_FUN
	fputs( " <->zero()\n", debug_fp );
#	endif
	return;
}

